!+
! Module: adapt_merge_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: adapt_merge_mod.f90,v 1.4 2009/04/23 13:31:09 aibarra Exp $
!-
!
!  SAS Version 13.0
!  Version 1.0  2 November 2012
!
!  Makes adaptively smoothed images from merged images
!
!  adapt_merge created adaptively smothed exposure corrected and
!  background subtracted images for the XMM-ESAS package.
!
!  INPUT:
!    SN: Number of counts for smoothing
!    temsc: Threshold for masking
!    nbands: Number of bands to merge
!    iel: List of lower energy limits for the energy bands in eV
!    ieh: List of upper energy limits for the energy bands in eV
!    ibin: 1: No binning, 2: Bin by 2, 4: Bin by 4
!    ipart: Particle background control - 1: include, 2: exclude
!    isoft: Soft proton background control - 1: include, 2: exclude
!    imask: Mask control - 0: No additional masking
!          1: Read in the mask file name
!    infile: Mask file name (only if the mask control is 1)
!    ifill: Number of passes to fill in empty pixels
!
!    Examples: adapt_merge temsc=0.02 nbands=1 iel=2000 ieh=7200 
!                   ibin=2 ipart=1 isoft=0 imask=0 ifill=0
!              adapt_merge temsc=0.02 nbands=2 iel="400 750" 
!                   ieh="750 1250" ipart=1 isoft=1 imask=1 
!                   mask=mask.fits ifill=2
!    Number of passes to fill in empty pixels
!
!  OUPUT:
!    Adaptively smoothed image and radial profile QDP plot
!

module adapt_merge_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public adapt_merge

contains

  !---------------------------------------------------------------------
  subroutine adapt_merge
  !---------------------------------------------------------------------

        character(len=8) :: CTYPE(2), UNITS
        character(len=60) :: COMMENT
        character(len=132) :: INFILE, INFILES
        integer, dimension(:), allocatable :: IEL, IEH
        integer :: bitpix, block, gcount, i, ib, ibin, iehmax, &
            ielmin, ifill, iflag, ii, ios, ipeel, ir, &
            j, j1, j2, jj, jj1, jj2, k, lun, naxes(10), &
            naxis, nb, nn, noff, pcount, xdim, xdimo, &
            ydim, ydimo 
        integer :: status = 0
        integer, allocatable, dimension(:,:) :: ini4
        real :: area, b, blast, br(1000), bscale, bzero, c, clast, &
            cr(1000), e, elast, er(1000), num, numlast, r, rad, &
            rn(1000), sca, sn, sr(1000), t, temp, temsc, tlim, tn, &
            trad, xh(1000), xmax, xr(1000), yh(1000), ymax, yr(1000)
        real, allocatable, dimension(:,:) :: back, counts, expose, &
            mask, off, output, prot, size, swcx
        double precision :: cdelt(2), crpix(2), crval(2)
        logical :: clobber, extend, ioff, ipart, isoft, iswcx, &
            imask, simple

        SN = realParameter("smoothingcounts")
        TEMSC = realParameter("thresholdmasking")
        nb = parameterCount("elowlist")
        clobber = booleanParameter("clobber")

        allocate(IEL(nb),IEH(nb))
        do i=1, parameterCount("elowlist")
            IEL(i) = intParameter("elowlist",i-1)
            IEH(i) = intParameter("ehighlist",i-1)
        enddo

        IBIN = intParameter("binning")

        write(errstr,*) "Binning Parameter: ", ibin
        call message(AppMsg,Always)

        IPART = booleanParameter("withpartcontrol")
        ISOFT = booleanParameter("withsoftcontrol")
        ISWCX = booleanParameter("withswcxcontrol")
        IOFF = booleanParameter("withoffsetbkgcontrol")
        IMASK = booleanParameter("withmaskcontrol")

        write(errstr,*) "Include Particle Background: ", ipart
        call message(AppMsg,Always)
        write(errstr,*) "Include Soft Proton Background: ", isoft
        call message(AppMsg,Always)
        write(errstr,*) "Include SWCX Background: ", iswcx
        call message(AppMsg,Always)
        write(errstr,*) "Include Offset Background: ", ioff
        call message(AppMsg,Always)
        write(errstr,*) "Include Input Mask: ", imask
        call message(AppMsg,Always)

        infile = stringParameter("maskfile")
        if(imask) then
            write(errstr,*) "Mask File: ", trim(infile)
            call message(AppMsg,Always)
        endif
        IFILL = intParameter("fill")
        write(errstr,*) "Fill Parameter: ", ifill
        call message(AppMsg,Always)

        DO I=1,1000
            XH(I) = -0.1 + I/15.0
            YH(I) = 0.0
        ENDDO

!  Set the scale factor for the maps, this is the ratio of
!  one square degree to the solid angle of the original map pixel

        SCA = (60.0*60.0)*(60.0*60.0)/(2.5*2.5)

!  Enter number of counts for smoothing.  The program smoothes using
!  a circular region surrounding the pixel whose radius is increased
!  until the selected number of counts are enclosed.

        write(errstr,*) "Number of counts for smoothing: ", SN
        call message(AppMsg,Always)

!  Enter the fractional limit for template, the program will exclude
!  all pixels in the field where the mask value is less than the
!  fractional limit times the average non-zero value of the mask

        write(errstr,*) "Fractional limit for template: ", TEMSC
        call message(AppMsg,Always)

!  Enter the number of bands to add

        NN = 3
        DO IB=1,NB
            NOFF = (IB - 1)*2
            NN = 3 + NOFF + 1
            NN = 3 + NOFF + 2
            write(errstr,*) "Band: ",ib,"  Energy Limits: ", &
                        iel(ib), ieh(ib)
            call message(AppMsg,Always)
        ENDDO
        IELMIN = 10000
        IEHMAX = 0
        DO IB=1,NB
            IF(IEL(IB) .LT. IELMIN) IELMIN = IEL(IB)
            IF(IEH(IB) .GT. IEHMAX) IEHMAX = IEH(IB)
        ENDDO

!  Enter the binning control

        NN = NN + 1

!  Create the count array file names and read in the data

! 1000   FORMAT(A80)

!  Find the size of the input arrays

        IF(IEL(1) .GE. 10000) THEN
            WRITE(INFILE,3100) IEL(1), IEH(1)
 3100       FORMAT('obj-im-',I5,'-',I5,'.fits')
        ELSEIF(IEL(1) .GE. 1000) THEN
            IF(IEH(1) .GE. 10000) THEN
                WRITE(INFILE,3110) IEL(1), IEH(1)
 3110           FORMAT('obj-im-',I4,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3120) IEL(1), IEH(1)
 3120           FORMAT('obj-im-',I4,'-',I4,'.fits')
            ENDIF
        ELSE
            IF(IEH(1) .GE. 10000) THEN
                WRITE(INFILE,3130) IEL(1), IEH(1)
 3130           FORMAT('obj-im-',I3,'-',I5,'.fits')
            ELSEIF(IEH(1) .GE. 1000) THEN
                WRITE(INFILE,3140) IEL(1), IEH(1)
 3140           FORMAT('obj-im-',I3,'-',I4,'.fits')
            ELSE
                WRITE(INFILE,3150) IEL(1), IEH(1)
 3150           FORMAT('obj-im-',I3,'-',I3,'.fits')
            ENDIF
        ENDIF
        write(errstr,*) "Reading input file: ", trim(infile)
        call message(AppMsg,Sparse)

!  Get an unused lun

        call ftgiou(lun,status)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT_MERGE 01: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

!  Open the FITS file for the first observation exposure

        call ftopen(lun,infile,0,block,status)
        write(errstr,*) "ADAPT_MERGE: Opening file: ", trim(infile)
        call message(AppMsg,Sparse)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT_MERGE 02: STATUS=",status, &
                    " on return from ftopen"
            call message(AppMsg,Always)
            call error ("noOpen","Cannot open file")
        endif

!  Read the FITS file header information

        call ftghpr (lun,10,simple,bitpix,naxis,naxes, &
                pcount,gcount,extend,status)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT_MERGE 03: STATUS=",status, &
                    " on return from ftghpr"
            call message(AppMsg,Always)
            call error ("noHeader","Cannot read fits header info")
        endif
        xdim = naxes(1)
        ydim = naxes(2)
        call ftclos (lun,status)
        call ftfiou (lun,status)

!  Allocate the input array sizes

        allocate(back(xdim,ydim))
        allocate(counts(xdim,ydim))
        allocate(expose(xdim,ydim))
        allocate(mask(xdim,ydim))
        allocate(prot(xdim,ydim))
        allocate(swcx(xdim,ydim))
        allocate(off(xdim,ydim))
        allocate(ini4(xdim,ydim))

!  Now take care of the output array dimensions

        xdimo = xdim/ibin
        ydimo = ydim/ibin
        write(errstr,*) "Input Dimensions: ", xdim, ydim
        call message(AppMsg,Always)
        write(errstr,*) "Output Dimensions: ", xdimo, ydimo
        call message(AppMsg,Always)
        allocate(output(xdimo,ydimo))
        allocate(size(xdimo,ydimo))

!  Now start going through the data

        DO IB=1,NB
            IF(IEL(IB) .GE. 10000) THEN
                WRITE(INFILE,3100) IEL(IB), IEH(IB)
            ELSEIF(IEL(IB) .GE. 1000) THEN
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3110) IEL(IB), IEH(IB)
                ELSE
                    WRITE(INFILE,3120) IEL(IB), IEH(IB)
                ENDIF
            ELSE
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3130) IEL(IB), IEH(IB)
                ELSEIF(IEH(IB) .GE. 1000) THEN
                    WRITE(INFILE,3140) IEL(IB), IEH(IB)
                ELSE
                    WRITE(INFILE,3150) IEL(IB), IEH(IB)
                ENDIF
            ENDIF

            IF(IB .EQ. 1) THEN
                CALL RETFIT (INFILE,COUNTS,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            ELSE
                CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                DO I=1,xdim
                    DO II=1,ydim
                        COUNTS(I,II) = COUNTS(I,II) + MASK(I,II)
                    ENDDO
                ENDDO
            ENDIF
            write(errstr,*) "ADAPT_MERGE: Retrieving image: ", &
                            trim(infile)
            call message(AppMsg,Sparse)
            if(status .ne. 0) then
                write(errstr,*) "ADAPT_MERGE 04: STATUS=",status, &
                        " on return from retfit"
                call message(AppMsg,Always)
                call error ("noRet","Error reading image")
            endif
        ENDDO

!  Enter the exposure array file names and read in the data

        DO IB=1,NB
            IF(IEL(IB) .GE. 10000) THEN
                WRITE(INFILE,3200) IEL(IB), IEH(IB)
 3200           FORMAT('exp-im-',I5,'-',I5,'.fits')
            ELSEIF(IEL(IB) .GE. 1000) THEN
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3210) IEL(IB), IEH(IB)
 3210               FORMAT('exp-im-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3220) IEL(IB), IEH(IB)
 3220               FORMAT('exp-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3230) IEL(IB), IEH(IB)
 3230               FORMAT('exp-im-',I3,'-',I5,'.fits')
                ELSEIF(IEH(IB) .GE. 1000) THEN
                    WRITE(INFILE,3240) IEL(IB), IEH(IB)
 3240               FORMAT('exp-im-',I3,'-',I4,'.fits')
                ELSE
                    WRITE(INFILE,3250) IEL(IB), IEH(IB)
 3250               FORMAT('exp-im-',I3,'-',I3,'.fits')
                ENDIF
            ENDIF

            IF(IB .EQ. 1) THEN
                CALL RETFIT (INFILE,EXPOSE,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            ELSE
                CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                DO I=1,xdim
                    DO II=1,ydim
                        EXPOSE(I,II) = EXPOSE(I,II) + MASK(I,II)
                    ENDDO
                ENDDO
            ENDIF
            write(errstr,*) "ADAPT: Retrieving image: ", &
                            trim(infile)
            call message(AppMsg,Sparse)
            if(status .ne. 0) then
                write(errstr,*) "ADAPT_MERGE 05: STATUS=",status, &
                        " on return from retfit"
                call message(AppMsg,Always)
                call error ("noRet","Error reading image")
            endif
        ENDDO
        IF(NB .GT. 1) THEN
            DO I=1,xdim
                DO II=1,ydim
                    EXPOSE(I,II) = EXPOSE(I,II)/NB
                ENDDO
            ENDDO
        ENDIF

!  Enter the background count array file names and read in the data

        IF(IPART) THEN
            DO IB=1,NB
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3300) IEL(IB), IEH(IB)
 3300               FORMAT('back-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3310) IEL(IB), IEH(IB)
 3310                   FORMAT('back-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3320) IEL(IB), IEH(IB)
 3320                   FORMAT('back-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3330) IEL(IB), IEH(IB)
 3330                   FORMAT('back-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3340) IEL(IB), IEH(IB)
 3340                   FORMAT('back-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3350) IEL(IB), IEH(IB)
 3350                   FORMAT('back-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF

                IF(IB .EQ. 1) THEN
                    CALL RETFIT (INFILE,BACK,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,xdim
                        DO II=1,ydim
                            BACK(I,II) = BACK(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                write(errstr,*) "ADAPT_MERGE: Retrieving image: ", &
                            trim(infile)
                call message(AppMsg,Sparse)
                if(status .ne. 0) then
                    write(errstr,*) "ADAPT_MERGE 06: STATUS=",status, &
                            " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading image")
                endif
            enddo
        else
            do i=1,xdim
                do ii=1,ydim
                    back(i,ii) = 0.0
                enddo
            enddo
        endif

!  Enter the soft proton count array file names and read in the data

        IF(ISOFT) THEN
            DO IB=1,NB
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3400) IEL(IB), IEH(IB)
 3400               FORMAT('prot-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3410) IEL(IB), IEH(IB)
 3410                   FORMAT('prot-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3420) IEL(IB), IEH(IB)
 3420                   FORMAT('prot-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEL(IB) .LT. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3430) IEL(IB), IEH(IB)
 3430                   FORMAT('prot-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3440) IEL(IB), IEH(IB)
 3440                   FORMAT('prot-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3450) IEL(IB), IEH(IB)
 3450                   FORMAT('prot-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF

                IF(IB .EQ. 1) THEN
                    CALL RETFIT (INFILE,PROT,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,xdim
                        DO II=1,ydim
                            PROT(I,II) = PROT(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                write(errstr,*) "ADAPT_MERGE: Retrieving image: ", &
                        trim(infile)
                call message(AppMsg,Sparse)
                if(status .ne. 0) then
                    write(errstr,*) "ADAPT_MERGE 07: STATUS=",status, &
                            " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading image")
                endif
            ENDDO
        ELSE
            DO I=1,xdim
                DO II=1,ydim
                    PROT(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF

!  Enter the solar wind charge exchange count array file names and 
!  read in the data

        IF(ISWCX) THEN
            DO IB=1,NB
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3500) IEL(IB), IEH(IB)
 3500               FORMAT('swcx-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3510) IEL(IB), IEH(IB)
 3510                   FORMAT('swcx-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3520) IEL(IB), IEH(IB)
 3520                   FORMAT('swcx-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEL(IB) .LT. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3530) IEL(IB), IEH(IB)
 3530                   FORMAT('swcx-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3540) IEL(IB), IEH(IB)
 3540                   FORMAT('swcx-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3550) IEL(IB), IEH(IB)
 3550                   FORMAT('swcx-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF

                IF(IB .EQ. 1) THEN
                    CALL RETFIT (INFILE,SWCX,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,xdim
                        DO II=1,ydim
                            SWCX(I,II) = SWCX(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                write(errstr,*) "ADAPT_MERGE: Retrieving image: ", &
                        trim(infile)
                call message(AppMsg,Sparse)
                if(status .ne. 0) then
                    write(errstr,*) "ADAPT_MERGE 08: STATUS=",status, &
                            " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading image")
                endif
            ENDDO
        ELSE
            DO I=1,xdim
                DO II=1,ydim
                    SWCX(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF

!  Enter the soft proton offset backgrpount count array file names and 
!  read in the data

        IF(ioff) THEN
            DO IB=1,NB
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3555) IEL(IB), IEH(IB)
 3555               FORMAT('offset-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3565) IEL(IB), IEH(IB)
 3565                   FORMAT('offset-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3575) IEL(IB), IEH(IB)
 3575                   FORMAT('offset-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEL(IB) .LT. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3585) IEL(IB), IEH(IB)
 3585                   FORMAT('offset-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3595) IEL(IB), IEH(IB)
 3595                   FORMAT('offset-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3596) IEL(IB), IEH(IB)
 3596                   FORMAT('offset-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF

                IF(IB .EQ. 1) THEN
                    CALL RETFIT (INFILE,off,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,xdim
                        DO II=1,ydim
                            off(I,II) = off(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                write(errstr,*) "ADAPT_MERGE: Retrieving image: ", &
                        trim(infile)
                call message(AppMsg,Sparse)
                if(status .ne. 0) then
                    write(errstr,*) "ADAPT_MERGE 08: STATUS=",status, &
                            " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading image")
                endif
            ENDDO
        ELSE
            DO I=1,xdim
                DO II=1,ydim
                    off(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF


!  Add the background components

        DO I=1,xdim
            DO II=1,ydim
                BACK(I,II) =  BACK(I,II) + PROT(I,II) + &
                        SWCX(I,II) + off(i,ii)
            ENDDO
        ENDDO

!  Read in the template

        IF(IMASK) THEN

!  Return the mask

            CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            write(errstr,*) "ADAPT_MERGE: Retrieving image: ", &
                    trim(infile)
            call message(AppMsg,Sparse)
            if(status .ne. 0) then
                write(errstr,*) "ADAPT_MERGE 09: STATUS=",status, &
                        " on return from retfit"
                call message(AppMsg,Always)
                call error ("noRet","Error reading image")
            endif
            DO I=1,xdim
                DO II=1,ydim
                    IF(EXPOSE(I,II) .LE. 0.0) MASK(I,II) = 0.0
                ENDDO
            ENDDO
        ELSE

!  Use the exposure map as the template

            DO I=1,xdim
                DO II=1,ydim
                    MASK(I,II) = EXPOSE(I,II)
                ENDDO
            ENDDO
        ENDIF

!  Set the template limit

        J = 0
        TLIM = 0.
        DO I=1,xdim
            DO II=1,ydim
                IF(MASK(I,II) .GT. 0) THEN
                    TLIM = TLIM + MASK(I,II)
                    J = J + 1
                ENDIF
            ENDDO
        ENDDO
        TLIM = TEMSC*TLIM/J

!  Read in the filling control

        NN = NN + 1

!  Make a radial profile

        DO I=1,1000
            BR(I) = 0.0
            CR(I) = 0.0
            ER(I) = 0.0
            RN(I) = 0.0
        ENDDO
        DO I=1,xdim
            DO II=1,ydim
                IF(MASK(I,II) .GT. TLIM) THEN
                    R = SQRT((I-1000.5)**2 + (II-1000.5)**2)
                    IR = 60.0*R*CDELT(2) + 1.0
!                    IR = 2.0*60.0*R*CDELT(2)
                    IF(IR .GT. 1000) IR = 1000
                    BR(IR) = BR(IR) + BACK(I,II)
                    CR(IR) = CR(IR) + COUNTS(I,II)
                    ER(IR) = ER(IR) + EXPOSE(I,II)
                    RN(IR) = RN(IR) + 1.0
                ENDIF
            ENDDO
        ENDDO
        XMAX = 0.0
        YMAX = 0.0
        DO I=1,1000
            IF(ER(I) .GT. 0.0) THEN
                YR(I) = (CR(I) - BR(I))/ER(I)
                SR(I) = SQRT(CR(I))/ER(I)
                XR(I) = (I - 0.5)
                IF((YR(I)) .GT. YMAX) YMAX = YR(I)
                IF(XR(I) .GT. XMAX) XMAX = XR(I)
            ELSE
                YR(I) = 0.0
                SR(I) = 0.0
                XR(I) = (I - 0.5)/2.0
            ENDIF
        ENDDO

!  Write out the new FITS adaptively smoothed image file

        IF(IELMIN .GE. 10000) THEN
            WRITE(INFILE,3600) IELMIN, IEHMAX
 3600       FORMAT('radial-',I5,'-',I5,'.qdp')
            WRITE(INFILES,3605) IELMIN, IEHMAX
 3605       FORMAT('size-',I5,'-',I5,'.qdp')
        ELSEIF(IELMIN .GE. 1000) THEN
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3610) IELMIN, IEHMAX
 3610           FORMAT('radial-',I4,'-',I5,'.qdp')
                WRITE(INFILES,3615) IELMIN, IEHMAX
 3615           FORMAT('size-',I4,'-',I5,'.qdp')
            ELSE
                WRITE(INFILE,3620) IELMIN, IEHMAX
 3620           FORMAT('radial-',I4,'-',I4,'.qdp')
                WRITE(INFILES,3625) IELMIN, IEHMAX
 3625           FORMAT('size-',I4,'-',I4,'.qdp')
            ENDIF
        ELSE
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3630) IELMIN, IEHMAX
 3630           FORMAT('radial-',I3,'-',I5,'.qdp')
                WRITE(INFILES,3635) IELMIN, IEHMAX
 3635           FORMAT('size-',I3,'-',I5,'.qdp')
            ELSEIF(IEHMAX .GE. 1000) THEN
                WRITE(INFILE,3640) IELMIN, IEHMAX
 3640           FORMAT('radial-',I3,'-',I4,'.qdp')
                WRITE(INFILES,3645) IELMIN, IEHMAX
 3645           FORMAT('size-',I3,'-',I4,'.qdp')
            ELSE
                WRITE(INFILE,3650) IELMIN, IEHMAX
 3650           FORMAT('radial-',I3,'-',I3,'.qdp')
                WRITE(INFILES,3655) IELMIN, IEHMAX
 3655           FORMAT('size-',I3,'-',I3,'.qdp')
            ENDIF
        ENDIF

!  Get an unused lun

        call ftgiou(lun,status)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT_MERGE 10: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

        write(errstr,*) "ADAPT_MERGE: Creating plot file:", trim(infile)
        call message(AppMsg,Sparse)
        IF(CLOBBER) THEN
            OPEN(UNIT=LUN,STATUS='UNKNOWN',FORM='FORMATTED', &
                FILE=INFILE)
        ELSE
            OPEN(UNIT=LUN,STATUS='NEW',FORM='FORMATTED',FILE=INFILE, &
                IOSTAT=IOS)
            if(ios .ne. 0) then
                write(errstr,*) "ADAPT_MERGE 11: STATUS=",status, &
                        " Output file already exists and no clobber"
                call message(AppMsg,Always)
                call error ("noClobber","Cannot create new file")
            endif
        ENDIF

        YMAX = 1.05*YMAX
        WRITE(LUN,*) 'FONT ROMAN'
        WRITE(LUN,*) 'CS 1.5'
        WRITE(LUN,*) 'READ SERR 2'
        WRITE(LUN,*) 'LA Y Count Rate (counts/s)'
        WRITE(LUN,*) 'LA X Radius'
        WRITE(LUN,*) 'LINE OFF 2'
        WRITE(LUN,*) 'R X 0 ', XMAX
        WRITE(LUN,*) 'R Y 0 ', YMAX
        WRITE(LUN,*) 'LW 2'
        WRITE(LUN,*) 'LA F'
        WRITE(LUN,*) 'LA T Radial Profile'
        DO I=1,1000
            WRITE(LUN,*) XR(I), YR(I), SR(I)
        ENDDO
        CLOSE(LUN)

!  Bin the data

        write(errstr,*) "Start the Binning"
        call message(AppMsg,Sparse)
        DO i=1,xdimo
            j2 = i*ibin
            j1 = j2 - ibin + 1
            if(j2 .gt. xdim) j2 = xdim
            DO ii=1,ydimo
                jj2 = ii*ibin
                jj1 = jj2 - ibin + 1
                if(jj2 .gt. ydim) jj2 = ydim
                b = 0.
                c = 0.
                e = 0.
                DO j=j1,j2
                    DO jj=jj1,jj2
                        IF(MASK(J,JJ) .GT. TLIM) THEN
                            B = B + BACK(J,JJ)
                            C = C + COUNTS(J,JJ)
                            E = E + EXPOSE(J,JJ)
                        ENDIF
                    ENDDO
                ENDDO
                BACK(I,II) = B
                COUNTS(I,II) = C
                EXPOSE(I,II) = E
            ENDDO
        ENDDO

!  Now smooth the data

        write(errstr,*) "Start the Smoothing"
        call message(AppMsg,Sparse)
        DO I=1,xdimo
            IF(MOD(I,25) .EQ. 0) then
                write(errstr,*) "Step ",I," out of ",xdimo
                call message(AppMsg,Always)
            endif
            DO II=1,ydimo
                IF(EXPOSE(I,II) .GT. 0.) THEN
                    B = 0.
                    C = 0.
                    E = 0.
                    NUM = 0
                    K = -1
                    DO WHILE (C .LT. SN)
                        BLAST = B
                        CLAST = C
                        ELAST = E
                        NUMLAST = NUM
                        K = K + 1
                        IF(K .EQ. 1) THEN
                            RAD = 1.5*1.5
                        ELSEIF(K .EQ. 2) THEN
                            RAD = 2.5*2.5
                        ELSEIF(K .EQ. 3) THEN
                            RAD = 3.2*3.2
                        ELSE
                            RAD = K*K*1.05*1.05
                        ENDIF
                        J1 = I - K
                        IF(J1 .LT. 1) J1 = 1
                        J2 = I + K
                        IF(J2 .GT. xdimo) J2 = xdimo
                        JJ1 = II - K
                        IF(JJ1 .LT. 1) JJ1 = 1
                        JJ2 = II + K
                        IF(JJ2 .GT. ydimo) JJ2 = ydimo
                        B = 0.
                        C = 0.
                        E = 0.
                        NUM = 0
                        DO J=J1,J2
                            DO JJ=JJ1,JJ2
                                TRAD = (J-I)**2 + (JJ-II)**2
!                                TRAD = SQRT(TRAD)
                                IF((EXPOSE(J,JJ) .GE. 0.) .AND. &
                                        (TRAD .LE. RAD)) THEN
                                    IF(RAD .EQ. 0.0) THEN
                                        B = B + BACK(J,JJ)
                                        C = C + COUNTS(J,JJ)
                                        E = E + EXPOSE(J,JJ)
                                    ELSE
!                                        T = EXP(-((TRAD/RAD)**2))
!                                        B = B + BACK(J,JJ)*T
!                                        C = C + COUNTS(J,JJ)*T
!                                        E = E + EXPOSE(J,JJ)*T
!
!                                        B = B + BACK(J,JJ)
!                                        C = C + COUNTS(J,JJ)
!                                        E = E + EXPOSE(J,JJ)
!
                                        B = B + BACK(J,JJ)* &
                                            (RAD - TRAD/1.0)/RAD
                                        C = C + COUNTS(J,JJ)* &
                                            (RAD - TRAD/1.0)/RAD
                                        E = E + EXPOSE(J,JJ)* &
                                            (RAD - TRAD/1.0)/RAD
                                    ENDIF
                                    NUM = NUM + 1
                                ENDIF
                            ENDDO
                        ENDDO
                    ENDDO
                    IF((K .GT. 1) .AND. (NUMLAST .GT. 0.)) THEN
                        TEMP = SCA*(CLAST - BLAST)/ELAST
                        OUTPUT(I,II) = TEMP
                    ELSEIF(NUM .GT. 0.) THEN
                        TEMP = SCA*(C - B)/E
                        OUTPUT(I,II) = TEMP
                    ELSE
                        OUTPUT(I,II) = TEMP
                    ENDIF
                    RAD = 60.0*CDELT(2)*SQRT(RAD)
                    SIZE(I,II) = RAD
                    IR = 15*RAD + 1
                    YH(IR) = YH(IR) + 1
                ENDIF
            ENDDO
        ENDDO

!  Get an unused lun

        call ftgiou(lun,status)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT_MERGE 12: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

        write(errstr,*) "ADAPT_MERGE: Creating plot file:", &
                trim(infiles)
        call message(AppMsg,Sparse)
        IF(CLOBBER) THEN
            OPEN(UNIT=LUN,STATUS='UNKNOWN',FORM='FORMATTED', &
                FILE=INFILES)
        ELSE
            OPEN(UNIT=LUN,STATUS='NEW',FORM='FORMATTED',FILE=INFILES, &
                IOSTAT=IOS)
            if(ios .ne. 0) then
                write(errstr,*) "ADAPT_MERGE 13: STATUS=",status, &
                        " Output file already exists and no clobber"
                call message(AppMsg,Always)
                call error ("noClobber","Cannot create new file")
            endif
        ENDIF

        YMAX = 0.0
        DO I=1,100
            IF(YH(I) .GT. YMAX) YMAX = YH(I)
        ENDDO
        YMAX = 1.05*YMAX

        WRITE(LUN,*) 'VIEWPORT 0.15 0.15 0.95 0.85'
        WRITE(LUN,*) 'FONT ROMAN'
        WRITE(LUN,*) 'CS 1.8'
        WRITE(LUN,*) 'LA X Conical Sampling FWHM (arcmin)'
        WRITE(LUN,*) 'LA Y Number'
        WRITE(LUN,*) 'LA F ', INFILES
        WRITE(LUN,*) 'COL 1 ON 2'
        WRITE(LUN,*) 'R X 0 5'
        WRITE(LUN,*) 'R Y 0 ', YMAX
        WRITE(LUN,*) 'LW 2'
        WRITE(LUN,*) 'LA F'
        WRITE(LUN,*) 'LA T Smoothing FWHM Histogram'
        WRITE(LUN,*) 'LINE STEP'
        DO I=1,1000
            WRITE(LUN,*) XH(I), YH(I)
        ENDDO
        CLOSE(LUN)

!  Write out the new FITS adaptively smoothed image file

        IF(IELMIN .GE. 10000) THEN
            WRITE(INFILE,3700) IELMIN, IEHMAX
 3700       FORMAT('adapt-',I5,'-',I5,'.fits')
            WRITE(INFILES,3705) IELMIN, IEHMAX
 3705       FORMAT('size-',I5,'-',I5,'.fits')
        ELSEIF(IELMIN .GE. 1000) THEN
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3710) IELMIN, IEHMAX
 3710           FORMAT('adapt-',I4,'-',I5,'.fits')
                WRITE(INFILES,3715) IELMIN, IEHMAX
 3715           FORMAT('size-',I4,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3720) IELMIN, IEHMAX
 3720           FORMAT('adapt-',I4,'-',I4,'.fits')
                WRITE(INFILES,3725) IELMIN, IEHMAX
 3725           FORMAT('size-',I4,'-',I4,'.fits')
            ENDIF
        ELSE
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3730) IELMIN, IEHMAX
 3730           FORMAT('adapt-',I3,'-',I5,'.fits')
                WRITE(INFILES,3735) IELMIN, IEHMAX
 3735           FORMAT('size-',I3,'-',I5,'.fits')
            ELSEIF(IEHMAX .GE. 1000) THEN
                WRITE(INFILE,3740) IELMIN, IEHMAX
 3740           FORMAT('adapt-',I3,'-',I4,'.fits')
                WRITE(INFILES,3745) IELMIN, IEHMAX
 3745           FORMAT('size-',I3,'-',I4,'.fits')
            ELSE
                WRITE(INFILE,3750) IELMIN, IEHMAX
 3750           FORMAT('adapt-',I3,'-',I3,'.fits')
                WRITE(INFILES,3755) IELMIN, IEHMAX
 3755           FORMAT('size-',I3,'-',I3,'.fits')
            ENDIF
        ENDIF

        COMMENT = 'Adaptive-filter smoothed count-rate image for XMM'
        CDELT(1) = ibin*CDELT(1)
        CDELT(2) = ibin*CDELT(2)
        CRPIX(1) = xdimo/2 + 0.5
        CRPIX(2) = ydimo/2 + 0.5
        UNITS = 'c/s/deg2'
        BZERO = 0.0
        BSCALE = 1.0
        AREA = 60.0*CDELT(2)

        IPEEL = IFILL - 1
        IF(IFILL .GT. 0) THEN
            write(errstr,*) "Starting infill process"
            call message(AppMsg,Sparse)
            DO IB=1,IFILL
                write(errstr,2100) ib
                call message(AppMsg,Sparse)
 2100           FORMAT(1X,'Starting infill pass: ',I3)
                DO I=2,xdimo-1
                    IF(MOD(I,100) .EQ. 0) then
                        write(errstr,*) "Step ",i," out of ",xdimo
                        call message(AppMsg,Always)
                    endif
                    DO II=2,ydimo-1
                        IF(OUTPUT(I,II) .EQ. 0.0) THEN
                            T = 0.0
                            TN = 0.0
                            DO J=I-1,I+1
                                DO JJ=II-1,II+1
                                    IF(OUTPUT(J,JJ) .NE. 0.0) THEN
                                        T = T + OUTPUT(J,JJ)
                                        TN = TN + 1
                                    ENDIF
                                ENDDO
                            ENDDO
                            IF(TN .GE. 3.0) THEN
                                COUNTS(I,II) = T/TN
                            ELSE
                                COUNTS(I,II) = 0.0
                            ENDIF
                        ELSE
                            COUNTS(I,II) = OUTPUT(I,II)
                        ENDIF
                    ENDDO
                ENDDO
                DO I=2,xdimo-1
                    DO II=2,ydimo-1
                        OUTPUT(I,II) = COUNTS(I,II)
                    ENDDO
                ENDDO
                write(errstr,2200) ib
                call message(AppMsg,Sparse)
 2200           FORMAT(1X,'Infill pass',I3,' complete')
            ENDDO
        ENDIF
        IFLAG = 1
        BZERO = 0.0
        BSCALE = 1.0

        CALL PUTFIT(INFILE,OUTPUT,INI4,IFLAG,UNITS, &
                COMMENT,xdimo,ydimo,BZERO,BSCALE,CTYPE,CRVAL, &
                CDELT,CRPIX,CLOBBER,STATUS)
        write(errstr,*) "ADAPT_MERGE: Writing image: ", &
                trim(infile)
        call message(AppMsg,Sparse)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT_MERGE 14: STATUS=",status, &
                    " on return from putfit"
            call message(AppMsg,Always)
            call error ("noPut","Error writing image")
        endif

        UNITS = 'arcmin'
        COMMENT = 'Adaptive-filter smoothing kernal'

        CALL PUTFIT(INFILES,SIZE,INI4,IFLAG,UNITS, &
                COMMENT,xdimo,ydimo,BZERO,BSCALE,CTYPE,CRVAL, &
                CDELT,CRPIX,CLOBBER,STATUS)
        write(errstr,*) "ADAPT_MERGE: Writing image: ", &
                trim(infile)
        call message(AppMsg,Sparse)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT_MERGE 15: STATUS=",status, &
                    " on return from putfit"
            call message(AppMsg,Always)
            call error ("noPut","Error writing image")
        endif

        return

    end subroutine adapt_merge

end module adapt_merge_mod
