!+
! Module: comb_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
! SLS 7 May 2009
!
! Comments:
!
! $Id: comb_mod.f90,v 1.4 2009/05/26 12:53:39 aibarra Exp $
!-
!
!  SAS Version 13.0
!  Version 2.2  5 November 2012
!
!  Combines images from multiple exposures from a single observation
!
!  comb is part of the XMM-ESAS package and combines images from
!  different exposures and MOS instruments from the same observation
!  in preparation for adapt and other analysis.  Count, exposure,
!  background, soft proton, and charge exchange images are combined, 
!  although if the background, soft proton, charge exchange images 
!  are missing they are ignored.
!  Masking can be done at the same time using the cheese images.
!
!  Input:
!
!    CalDB directory
!    Particle background control
!    Soft proton background control
!    Number of bands to merge
!    Repeat for the number of bands
!        Lower energy limit for the energy band in eV
!        Upper energy limit for the energy band in eV
!    Masking control:
!        0 => no masking
!        1 =>  Point Source Masking - Use either output from cheese or 
!              the combined band output from cheese-bands
!        11 => Point Source Masking - Use the soft band output from 
!              cheese-bands
!        12 => Point Source Masking - Use the hard band output from 
!              cheese-bands
!        2 => masking using the normal mask images produced by
!              eexpmap
!        3 =>  Point Source Masking - Use the full band output from 
!              make-mask-merge
!        31 => Point Source Masking - Use the soft band output from 
!              make-mask-merge
!        32 => Point Source Masking - Use the hard band output from 
!              make-mask-merge
!    Number of data exposures (i.e., number of prefixes)
!    List of prefixes (e.g., 1S001, 2U002)
!
!    Example: comb 1 1 1 350 1250 0 2 1S001 2S002
!             comb 1 0 2 350 800 800 1250 1 3 1S001 2S002 S003
!
!  Output:
!    Merged count and exposure images with merged backgorund
!    and soft proton images if they exist.

module comb_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public comb

contains

  !---------------------------------------------------------------------
  subroutine comb
  !---------------------------------------------------------------------
        integer :: bitpix, block, ebmax, ebmin, ehigh, elow, &
            gcount, group, i, ib, idet, ihigh, ii, ilow, imask, &
            ini4(900,900), islow, ishigh, lun, luna, lunb, n, &
            naxes(3), naxis, nbands, ndata, nn, noff, pcount 
        integer :: status = 0
        integer, dimension(:), allocatable :: iel, ieh

        real :: alpha, fscal, image(900,900), mask(900,900,10), &
            nback(20), ncounts(20), ootsca, outim(900,900), ttot
        real :: totback = 0.0
        real :: totcou = 0.0
        real :: totexp = 0.0
        real :: totprot = 0.0
        real :: totswcx = 0.0

        double precision cdelt(2), crpix(2), crval(2)

        character (len=1) :: pr(4), pre(5)
        character (len=4) :: filt, pref
        character (len=5) :: prefi
        character (len=8) :: ctype(2), filter
        character (len=20) :: chname
        character (len=25) :: mode
        character (len=80) :: com
        character (len=132) :: im_name, im_nameo, imo_name, xmmdir
        character (len=5), dimension(:), allocatable :: prefix

        logical :: clobber, ipart, iprot, iswcx
        logical :: doit = .true.

        equivalence (filter,filt)
        equivalence (pref,pr)
        equivalence (prefi,pre)

        xmmdir = stringParameter("caldb")
        clobber = booleanParameter("clobber")
        ipart = booleanParameter("withpartcontrol")
        iprot = booleanParameter("withsoftcontrol")
        iswcx = booleanParameter("withswcxcontrol")
        nbands = parameterCount("elowlist")
        alpha = realParameter("alpha")

        allocate(iel(nbands),ieh(nbands))
        do i=1, parameterCount("elowlist")
            iel(i) = intParameter("elowlist",i-1)
            ieh(i) = intParameter("ehighlist",i-1)
        end do

        imask = intParameter("mask")
        ndata = parameterCount("prefixlist")
        allocate(prefix(ndata))
        do i=1, parameterCount("prefixlist")
            prefix(i) = stringParameter("prefixlist",i-1)
        end do

        ebmin = 100000
        ebmax = 0
        do ib=1,nbands
            noff = (ib - 1)*2
            nn = 4 + noff + 1
            nn = 4 + noff + 2
            if(iel(ib) .lt. ebmin) ebmin = iel(ib)
            if(ieh(ib) .gt. ebmax) ebmax = ieh(ib)
        enddo
        ilow = ebmin
        ihigh = ebmax
        ib = 0

!  Set the band

        islow = (ilow + 25)/50
        ishigh = (ihigh + 25)/50

        write(errstr,*) "Number of Energy Bands to Bin: ", nbands
        call message(AppMsg,Always)
        do ib=1,nbands
            write(errstr,*) "    Band ", ib," Energy Limits", &
                        iel(ib), ieh(ib)
            call message(AppMsg,Always)
        enddo
        write(errstr,*) "Particle background control: ", ipart
        call message(AppMsg,Always)
        write(errstr,*) "Soft proton background control: ", iprot
        call message(AppMsg,Always)
        write(errstr,*) "SWCX background control: ", iswcx
        call message(AppMsg,Always)
        write(errstr,*) "Masking control: ", imask
        call message(AppMsg,Always)
        write(errstr,*) "Exposure sets: ", ndata
        call message(AppMsg,Always)
        write(errstr,*) "Scaling spectral index: ", alpha
        call message(AppMsg,Always)
        write(errstr,*) "List of prefixes:"
        call message(AppMsg,Always)
        do i=1,ndata
            write(errstr,*) "    prefix: ", trim(prefix(i))
            call message(AppMsg,Always)
        enddo
        write(errstr,*) "Clobber: ", clobber
        call message(AppMsg,Always)

!  If masking is desired, read in the masks

        if((imask .eq. 1) .or. (imask .eq. 11) .or.  &
                (imask .eq. 12)) then
            if(imask .eq. 1) then
                chname = '-cheese.fits'
            elseif(imask .eq. 11) then
                chname = '-cheese-s.fits'
            elseif(imask .eq. 12) then
                chname = '-cheese-h.fits'
            endif
            do n=1,ndata
                prefi = prefix(n)
                if((pre(1) .eq. '1') .or. (pre(1) .eq. '2')) then
                    write(im_name,2999) prefix(n), chname
 2999               format('mos',a5,a20)
                    if(pre(1) .eq. '1') idet = 1
                    if(pre(1) .eq. '2') idet = 2
                else
                    idet = 3
                    do i=1,4
                        pr(i) = pre(i)
                    enddo
                    write(im_name,3000) pref, chname
 3000               format('pn',a4,a20)
                endif
                write(errstr,*) "Retrieving mask image: ", &
                        trim(im_name)
                call message(AppMsg,Sparse)
                call retfit(im_name,image,ini4, &
                        1,900,900,ctype,crval,cdelt,crpix,status)
                if(status .ne. 0) then
                    write(errstr,*) "01a: STATUS=", &
                            status, " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading mask image")
                endif
                do i=1,900
                    do ii=1,900
                        mask(i,ii,n) = image(i,ii)
                    enddo
                enddo
            enddo
        elseif(imask .eq. 2) then
            do n=1,ndata
                prefi = prefix(n)
                if((pre(1) .eq. '1') .or. (pre(1) .eq. '2')) then
                    if(iel(1) .ge. 1000) then
                        if(ieh(1) .ge. 10000) then
                            write(im_name,3001) prefix(n), &
                                    iel(1), ieh(1)
 3001                       format('mos',a5,'-mask-im-',i4,'-', &
                                    i5,'.fits')
                        else
                            write(im_name,3002) prefix(n), &
                                    iel(1), ieh(1)
 3002                       format('mos',a5,'-mask-im-',i4,'-', &
                                    i4,'.fits')
                        endif
                    elseif(ieh(1) .lt. 1000) then
                        write(im_name,3003) prefix(n), iel(1), ieh(1)
 3003                   format('mos',a5,'-mask-im-',i3,'-',i3,'.fits')
                    elseif(ieh(1) .ge. 10000) then
                        write(im_name,3004) prefix(n), iel(1), ieh(1)
 3004                   format('mos',a5,'-mask-im-',i3,'-',i5,'.fits')
                    else
                        write(im_name,3005) prefix(n), iel(1), ieh(1)
 3005                   format('mos',a5,'-mask-im-',i3,'-',i4,'.fits')
                    endif
                else
                    do i=1,4
                        pr(i) = pre(i)
                    enddo
                    if(iel(1) .ge. 1000) then
                        if(ieh(1) .ge. 10000) then
                            write(im_name,3006) pref, iel(1), ieh(1)
 3006                       format('pn',a4,'-mask-im-',i4,'-', &
                                    i5,'.fits')
                        else
                            write(im_name,3007) pref, iel(1), ieh(1)
 3007                       format('pn',a4,'-mask-im-',i4,'-', &
                                    i4,'.fits')
                        endif
                    elseif(ieh(1) .lt. 1000) then
                        write(im_name,3008) pref, iel(1), ieh(1)
 3008                   format('pn',a4,'-mask-im-',i3,'-',i3,'.fits')
                    elseif(ieh(1) .ge. 10000) then
                        write(im_name,3009) pref, iel(1), ieh(1)
 3009                   format('pn',a4,'-mask-im-',i3,'-',i5,'.fits')
                    else
                        write(im_name,3010) pref, iel(1), ieh(1)
 3010                   format('pn',a4,'-mask-im-',i3,'-',i4,'.fits')
                    endif
                endif

                write(errstr,*) "Retrieving mask image: ", &
                        trim(im_name)
                call message(AppMsg,Sparse)
                call retfit(im_name,image,ini4, &
                        1,900,900,ctype,crval,cdelt,crpix,status)
                if(status .ne. 0) then
                    write(errstr,*) "01b: STATUS=", &
                            status, " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading mask image")
                endif
                do i=1,900
                    do ii=1,900
                        mask(i,ii,n) = image(i,ii)
                    enddo
                enddo
            enddo
        elseif((imask .eq. 3) .or. (imask .eq. 31) &
                .or. (imask .eq. 32)) then
            if(imask .eq. 3) then
                chname = '-msl-cheese.fits'
            elseif(imask .eq. 31) then
                chname = '-msl-cheese-s.fits'
            elseif(imask .eq. 32) then
                chname = '-msl-cheese-h.fits'
            endif
            do n=1,ndata
                prefi = prefix(n)
                if((pre(1) .eq. '1') .or. (pre(1) .eq. '2')) then
                    write(im_name,3011) prefix(n), chname
 3011               format('mos',a5,a20)
                else
                    do i=1,4
                        pr(i) = pre(i)
                    enddo
                    write(im_name,3016) pref, chname
 3016               format('pn',a4,a20)
                endif
                write(errstr,*) "Retrieving mask image: ", &
                        trim(im_name)
                call message(AppMsg,Sparse)
                call retfit(im_name,image,ini4, &
                        1,900,900,ctype,crval,cdelt,crpix,status)
                if(status .ne. 0) then
                    write(errstr,*) "01c: STATUS=", &
                            status, " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading mask image")
                endif
                do i=1,900
                    do ii=1,900
                        mask(i,ii,n) = image(i,ii)
                    enddo
                enddo
            enddo
        elseif(imask .eq. 0) then

!  Make full-field masks

            do n=1,ndata
                do i=1,900
                    do ii=1,900
                        mask(i,ii,n) = 1.0
                    enddo
                enddo
            enddo
        endif

!  Zero the output images

        do i=1,900
            do ii=1,900
                outim(i,ii) = 0.0
            enddo
        enddo

!  Read in the count images

        do ib=1,nbands
            ncounts(ib) = 0.0
            do n=1,ndata
                elow = iel(ib)
                ehigh = ieh(ib)
                prefi = prefix(n)
                if((pre(1) .eq. '1') .or. (pre(1) .eq. '2')) then
                    if(elow .ge. 1000) then
                        if(ehigh .ge. 10000) then
                            write(im_name,3031) prefix(n), elow, ehigh
 3031                       format('mos',a5,'-obj-im-',i4,'-', &
                                     i5,'.fits')
                        else
                            write(im_name,3032) prefix(n), elow, ehigh
 3032                       format('mos',a5,'-obj-im-',i4,'-', &
                                     i4,'.fits')
                        endif
                    elseif(ehigh .lt. 1000) then
                        write(im_name,3033) prefix(n), elow, ehigh
 3033                   format('mos',a5,'-obj-im-',i3,'-',i3,'.fits')
                    elseif(ehigh .ge. 10000) then
                        write(im_name,3034) prefix(n), elow, ehigh
 3034                   format('mos',a5,'-obj-im-',i3,'-',i5,'.fits')
                    else
                        write(im_name,3035) prefix(n), elow, ehigh
 3035                   format('mos',a5,'-obj-im-',i3,'-',i4,'.fits')
                    endif
                else
                    do i=1,4
                        pr(i) = pre(i)
                    enddo
                    if(elow .ge. 1000) then
                        if(ehigh .ge. 10000) then
                            write(im_name,3036) pref, elow, ehigh
 3036                       format('pn',a4,'-obj-im-',i4,'-', &
                                     i5,'.fits')
                            write(im_nameo,2936) pref, elow, ehigh
 2936                       format('pn',a4,'-obj-im-',i4,'-', &
                                     i5,'-oot.fits')
                        else
                            write(im_name,3037) pref, elow, ehigh
 3037                       format('pn',a4,'-obj-im-',i4,'-', &
                                     i4,'.fits')
                            write(im_nameo,2937) pref, elow, ehigh
 2937                       format('pn',a4,'-obj-im-',i4,'-', &
                                     i4,'-oot.fits')
                        endif
                    elseif(ehigh .lt. 1000) then
                        write(im_name,3038) pref, elow, ehigh
 3038                   format('pn',a4,'-obj-im-',i3,'-',i3,'.fits')
                        write(im_nameo,2938) pref, elow, ehigh
 2938                   format('pn',a4,'-obj-im-',i3,'-',i3,'-oot.fits')
                    elseif(ehigh .ge. 10000) then
                        write(im_name,3039) pref, elow, ehigh
 3039                   format('pn',a4,'-obj-im-',i3,'-',i5,'.fits')
                        write(im_nameo,2939) pref, elow, ehigh
 2939                   format('pn',a4,'-obj-im-',i3,'-',i5,'-oot.fits')
                    else
                        write(im_name,3040) pref, elow, ehigh
 3040                   format('pn',a4,'-obj-im-',i3,'-',i4,'.fits')
                        write(im_nameo,2940) pref, elow, ehigh
 2940                   format('pn',a4,'-obj-im-',i3,'-',i4,'-oot.fits')
                    endif
                endif

                write(errstr,*) "Retrieving count image: ", &
                        trim(im_name)
                call message(AppMsg,Sparse)
                call retfit(im_name,image,ini4, &
                        1,900,900,ctype,crval,cdelt,crpix,status)
                if(status .ne. 0) then
                    write(errstr,*) "02a: STATUS=", &
                            status, " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading count image")
                endif

!  Combine the images with masking

                ttot = 0.0
                do i=1,900
                    do ii=1,900
                        outim(i,ii) = outim(i,ii) + &
                                image(i,ii)*mask(i,ii,n)
                        ttot = ttot + image(i,ii)*mask(i,ii,n)
                        ncounts(ib) = ncounts(ib) + &
                                image(i,ii)*mask(i,ii,n)
                    enddo
                enddo
                write(errstr,*) prefix(n), " Number of counts:", ttot
                call message(AppMsg,Always)
                totcou = totcou + ttot

                if((pre(1) .ne. '1') .and. (pre(1) .ne. '2')) then
                    write(errstr,*) "Retrieving OOT count image:", &
                            trim(im_nameo)
                    call message(AppMsg,Sparse)
                    call retfit(im_nameo,image,ini4, &
                        1,900,900,ctype,crval,cdelt,crpix,status)
                    if(status .ne. 0) then
                        write(errstr,*) "02b: STATUS=", &
                                status, " on return from retfit"
                        call message(AppMsg,Always)
                        call error ("noRet","Error reading count image")
                    endif

!  This is a bit painful, but we need to know the SUBMODE
!  keyword for the proper scaling of the OOT events

!  Get an unused lun

                    call ftgiou(lun,status)
                    if(status .ne. 0) then
                        write(errstr,*) "03: STATUS=", & 
                                status, " on return from ftgiou"
                        call message(AppMsg,Always)
                        call error ("noLUN","Cannot aquire LUN")
                    endif

!  Open the FITS file

                    write(errstr,*) "Opening image: ", & 
                                trim(im_nameo)
                    call message(AppMsg,Sparse)
                    call ftopen (lun,im_nameo,0,block,status)
                    if(status .ne. 0) then
                        write(errstr,*) "04: STATUS=",status, &
                                " on return from ftopen"
                        call message(AppMsg,Always)
                        call error ("noOpen","Cannot open image file")
                    endif

!  Read in the mode information if a PN observation

                    call ftgkys (lun,'SUBMODE',mode,com,status)
                    if(status .ne. 0) then
                        write(errstr,*) "05: STATUS=",status, &
                                " on return from ftgkys"
                        call message(AppMsg,Always)
                        call error ("noKey", &
                                "Cannot access SUBMODE keyword")
                    else
                        if(mode .eq. 'PrimeFullWindow') ootsca = 0.063
                        if(mode .eq. 'PrimeFullWindowExtended') &
                                ootsca = 0.023
                        if(mode .eq. 'PrimeFullWindowExtended5') &
                                ootsca = 0.013
                    endif
                    write(errstr,*) "OOT scale factor: ", ootsca
                    call message(AppMsg,Sparse)

!  Close the FITS file

                    call ftclos(lun,status)
                    if(status .ne. 0) then
                        write(errstr,*) "06: STATUS=",status, &
                                " on return from ftclos"
                        call message(AppMsg,Always)
                        call error ("noClose","Cannot close image file")
                    endif
                    call ftfiou (lun,status)
                    if(status .ne. 0) then
                        write(errstr,*) "07: STATUS=",status, &
                                " on return from ftfiou"
                        call message(AppMsg,Always)
                        call error ("noFree","Cannot free LUN")
                    endif

                    ttot = 0.0
                    do i=1,900
                        do ii=1,900
                            outim(i,ii) = outim(i,ii) - &
                                    ootsca*image(i,ii)*mask(i,ii,n)
                            ttot = ttot + image(i,ii)*mask(i,ii,n)
                            ncounts(ib) = ncounts(ib) - &
                                    ootsca*image(i,ii)*mask(i,ii,n)
                        enddo
                    enddo
                    totcou = totcou - ootsca*ttot
                    write(errstr,*) prefix(n), "Number of OOT counts:", &
                                    ootsca*ttot
                    call message(AppMsg,Always)
                endif
            enddo
        enddo
        write(errstr,*) "Total number of counts:", totcou
        call message(AppMsg,Always)

!  Now write out the merged count file, first get two unused luns

        call ftgiou(luna,status)
        if(status .ne. 0) then
            write(errstr,*) "08a: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif
        call ftgiou(lunb,status)
        if(status .ne. 0) then
            write(errstr,*) "08b: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

!  Open the old image FITS file

        write(errstr,*) "Opening template image: ", trim(im_name)
        call message(AppMsg,Sparse)
        call ftopen (luna,im_name,0,block,status)
        if(status .ne. 0) then
            write(errstr,*) "09: STATUS=",status, &
                    " on return from ftopen"
            call message(AppMsg,Always)
            call error ("noOpen","Cannot open image file")
        endif

!  Create the output file name

        if(ebmin .ge. 1000) then
            if(ebmax .ge. 10000) then
                write(imo_name,3041) ebmin, ebmax
 3041           format('comb-obj-im-',i4,'-',i5,'.fits')
            else
                write(imo_name,3042) ebmin, ebmax
 3042           format('comb-obj-im-',i4,'-',i4,'.fits')
            endif
        elseif(ebmax .lt. 1000) then
            write(imo_name,3043) ebmin, ebmax
 3043       format('comb-obj-im-',i3,'-',i3,'.fits')
        elseif(ebmax .ge. 10000) then
            write(imo_name,3044) ebmin, ebmax
 3044       format('comb-obj-im-',i3,'-',i5,'.fits')
        else
            write(imo_name,3045) ebmin, ebmax
 3045       format('comb-obj-im-',i3,'-',i4,'.fits')
        endif
        write(errstr,*) "Output file name: ", trim(imo_name)
        call message(AppMsg,Sparse)

!  Check to see if there is an existing file

        write(errstr,*) "Atempting to open output image file"
        call message(AppMsg,Sparse)
        call ftopen (lunb,imo_name,0,block,status)
        if(status .eq.  0) then
            write(errstr,*) "File exists with output file name"
            call message(AppMsg,Sparse)
            if(clobber) then
                write(errstr,*) "Clobber the file and continue"
                call message(AppMsg,Sparse)
                call ftdelt(lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "10: STATUS=",status, &
                            " on return from ftdelt"
                    call message(AppMsg,Always)
                    call error ("badDelete","Error deleting file")
                endif
            else
                write(errstr,*) "11: Output file: ", trim(imo_name)
                call message(AppMsg,Always)
                call error ("noClobber","File exists and no clobber")
            endif
        else
            write(errstr,*) "No existing file so continue"
            call message(AppMsg,Sparse)
        endif

        status = 0

        write(errstr,*) "Initialize output image: ", trim(imo_name)
        call message(AppMsg,Sparse)
        call ftinit (lunb,imo_name,2880,status)
        if(status .ne.  0) then
            write(errstr,*) "12: STATUS=",status, &
                    " on return from ftinit"
            call message(AppMsg,Always)
            call error ("noInit","Unable to initialize output image")
        endif

!  Copy the fits header information from the old to new files

        call ftcphd (luna,lunb,status)
        if(status .ne.  0) then
            write(errstr,*) "13: STATUS=",status, &
                    " on return from ftcphd"
            call message(AppMsg,Always)
            call error ("noCopyHeader","Unable to copy header")
        endif

        bitpix = -32
        naxis = 2
        naxes(1) = 900
        naxes(2) = 900
        pcount = 0
        gcount = 1
        group = 1
        call ftpdef (lunb,bitpix,naxis,naxes,pcount, &
                gcount,.FALSE.,status)
        if(status .ne.  0) then
            write(errstr,*) "14: STATUS=",status, &
                    " on return from ftpdef"
            call message(AppMsg,Always)
            call error ("noDef","Unable to define image")
        endif
        call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
        if(status .ne.  0) then
            write(errstr,*) "15: STATUS=",status, &
                    " on return from ftmkey"
            call message(AppMsg,Always)
            call error ("noKeyMod","Unable to modify keyword")
        endif

!  Write the output image

        write(errstr,*) "Write output image: ", trim(imo_name)
        call message(AppMsg,Sparse)
        call ftp2de (lunb,group,naxes(1),naxes(1),naxes(2), &
                    outim,status)
        if(status .ne.  0) then
            write(errstr,*) "16: STATUS=",status, &
                    " on return from ftp2de"
            call message(AppMsg,Always)
            call error ("noImageWrite","Unable write image")
        endif

!  Close the template and output files and release their luns

        call ftclos(luna,status)
        if(status .ne. 0) then
            write(errstr,*) "17a: STATUS=",status, &
                    " on return from ftclos"
            call message(AppMsg,Always)
            call error ("noClose","Cannot close file")
        endif
        call ftclos(lunb,status)
        if(status .ne. 0) then
            write(errstr,*) "17b: STATUS=",status, &
                    " on return from ftclos"
            call message(AppMsg,Always)
            call error ("noClose","Cannot close file")
        endif
        call ftfiou (luna,status)
        if(status .ne. 0) then
            write(errstr,*) "18a: STATUS=",status, &
                    " on return from ftfiou"
            call message(AppMsg,Always)
            call error ("noFree","Cannot free LUN")
        endif
        call ftfiou (lunb,status)
        if(status .ne. 0) then
            write(errstr,*) "18a: STATUS=",status, &
                    " on return from ftfiou"
            call message(AppMsg,Always)
            call error ("noFree","Cannot free LUN")
        endif

!  Zero the background count array 

        do ib=1,nbands
            nback(ib) = 0.0
        enddo

!  Combine the model QPB count images 

        if(ipart) then
            doit = .TRUE.

!  Zero the output images

            do i=1,900
                do ii=1,900
                    outim(i,ii) = 0.0
                enddo
            enddo

!  Read in the QPB background images

            do ib=1,nbands
                do n=1,ndata
                    elow = iel(ib)
                    ehigh = ieh(ib)
                    if(doit .eqv. .TRUE.) then
                        prefi = prefix(n)
                        if((pre(1) .eq. '1') .or. &
                                (pre(1) .eq. '2')) then
                            if(elow .ge. 1000) then
                                if(ehigh .ge. 10000) then
                                    write(im_name,3071) prefix(n), &
                                        elow, ehigh
 3071                               format('mos',a5,'-back-im-sky-', &
                                            i4,'-',i5,'.fits')
                                else
                                    write(im_name,3072) prefix(n), &
                                            elow, ehigh
 3072                               format('mos',a5,'-back-im-sky-', &
                                            i4,'-',i4,'.fits')
                                endif
                            elseif(ehigh .lt. 1000) then
                                write(im_name,3073) prefix(n),elow,ehigh
 3073                           format('mos',a5,'-back-im-sky-',i3, &
                                        '-',i3,'.fits')
                            elseif(ehigh .ge. 10000) then
                                write(im_name,3074) prefix(n),elow,ehigh
 3074                           format('mos',a5,'-back-im-sky-',i3, &
                                        '-',i5,'.fits')
                            else
                                write(im_name,3075) prefix(n),elow,ehigh
 3075                           format('mos',a5,'-back-im-sky-',i3, &
                                        '-',i4,'.fits')
                            endif
                        else
                            do i=1,4
                                pr(i) = pre(i)
                            enddo
                            if(elow .ge. 1000) then
                                if(ehigh .ge. 10000) then
                                    write(im_name,3076) prefix(n), &
                                            elow, ehigh
 3076                               format('pn',a4,'-back-im-sky-', &
                                            i4,'-',i5,'.fits')
                                else
                                    write(im_name,3077) prefix(n), &
                                            elow, ehigh
 3077                               format('pn',a4,'-back-im-sky-', &
                                            i4,'-',i4,'.fits')
                                    endif
                            elseif(ehigh .lt. 1000) then
                                write(im_name,3078) prefix(n),elow,ehigh
 3078                           format('pn',a4,'-back-im-sky-',i3, &
                                        '-',i3,'.fits')
                            elseif(ehigh .ge. 10000) then
                                write(im_name,3079) prefix(n),elow,ehigh
 3079                           format('pn',a4,'-back-im-sky-',i3, &
                                        '-',i5,'.fits')
                            else
                                write(im_name,3080) prefix(n),elow,ehigh
 3080                           format('pn',a4,'-back-im-sky-',i3, &
                                        '-',i4,'.fits')
                            endif
                        endif

                        write(errstr,*) "Retrieving image: ", &
                                trim(im_name)
                        call message(AppMsg,Sparse)
                        call retfit(im_name,image,ini4, &
                              1,900,900,ctype,crval,cdelt,crpix,status)
                        if(status .ne. 0) then
                          write(errstr,*) "19: STATUS=",status, &
                                  " on return from retfit"
                            call message(AppMsg,Sparse)
                          write(errstr,*) "Error reading QPB image"
                          call message(AppMsg,Sparse)
                          write(errstr,*) &
                              "    Not a problem but deselect QDP in"
                          call message(AppMsg,Sparse)
                          write(errstr,*) &
                              "    adapt and bin_image"
                          call message(AppMsg,Sparse)
                          doit = .FALSE.
                        else

!  Combine the images

                          ttot = 0.0
                          do i=1,900
                              do ii=1,900
                                  outim(i,ii) = outim(i,ii) + &
                                          image(i,ii)*mask(i,ii,n)
                                  ttot = ttot + image(i,ii)*mask(i,ii,n)
                                  nback(ib) = nback(ib) +  &
                                          image(i,ii)*mask(i,ii,n)
                              enddo
                          enddo
                          totback = totback + ttot
                          write(errstr,*) prefix(n), " QPB counts:", ttot
                          call message(AppMsg,Always)
                        endif
                    endif
                enddo
            enddo
            status = 0

!  Now write out the merged QPB count file, first get two unused luns

            if(doit .eqv. .TRUE.) then
                write(errstr,*) "Total QPB counts:", totback
                call message(AppMsg,Always)
                call ftgiou(luna,status)
                if(status .ne. 0) then
                    write(errstr,*) "20a: STATUS=",status, &
                            " on return from ftgiou"
                    call message(AppMsg,Always)
                    call error ("noLUN","Cannot aquire LUN")
                endif
                call ftgiou(lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "20b: STATUS=",status, &
                            " on return from ftgiou"
                    call message(AppMsg,Always)
                    call error ("noLUN","Cannot aquire LUN")
                endif

!  Open the old image FITS file

                write(errstr,*) "Opening template image: ", &
                            trim(im_name)
                call message(AppMsg,Sparse)
                call ftopen (luna,im_name,0,block,status)
                if(status .ne. 0) then
                    write(errstr,*) "21: STATUS=",status, &
                            " on return from ftopen"
                    call message(AppMsg,Always)
                    call error ("noOpen","Cannot open image file")
                endif

!  Create the output particle background file name

                if(ebmin .ge. 1000) then
                    if(ebmax .ge. 10000) then
                        write(imo_name,3081) ebmin, ebmax
 3081                   format('comb-back-im-sky-',i4,'-',i5,'.fits')
                    else
                        write(imo_name,3082) ebmin, ebmax
 3082                   format('comb-back-im-sky-',i4,'-',i4,'.fits')
                    endif
                elseif(ebmax .lt. 1000) then
                    write(imo_name,3083) ebmin, ebmax
 3083               format('comb-back-im-sky-',i3,'-',i3,'.fits')
                elseif(ebmax .ge. 10000) then
                    write(imo_name,3084) ebmin, ebmax
 3084               format('comb-back-im-sky-',i3,'-',i5,'.fits')
                else
                    write(imo_name,3085) ebmin, ebmax
 3085               format('comb-back-im-sky-',i3,'-',i4,'.fits')
                endif

!  Check to see if there is an existing file

                write(errstr,*) "Output QPB file name: ", &
                        trim(imo_name)
                call message(AppMsg,Sparse)
                write(errstr,*) &
                        "Atempting to open output image file"
                call message(AppMsg,Sparse)
                call ftopen (lunb,imo_name,0,block,status)
                if(status .eq.  0) then
                    write(errstr,*) &
                            "File exists with output file name"
                    call message(AppMsg,Sparse)
                    if(clobber) then
                        write(errstr,*) &
                                "Clobber the file and continue"
                        call message(AppMsg,Sparse)
                        call ftdelt(lunb,status)
                        if(status .ne. 0) then
                            write(errstr,*) "22: STATUS=", &
                                    status," on return from ftdelt"
                            call message(AppMsg,Always)
                            call error ("badDelete",&
                                    "Error deleting file")
                        endif
                    else
                        write(errstr,*) "23: Output file: ", &
                                trim(imo_name)
                        call message(AppMsg,Sparse)
                        call error &
                            ("noClobber","File exists and no clobber")
                    endif
                else
                    write(errstr,*) &
                             "No existing file so continue"
                    call message(AppMsg,Sparse)
                endif
                status = 0

                write(errstr,*) "Initialize output image: ", &
                        trim(imo_name)
                call message(AppMsg,Sparse)
                call ftinit (lunb,imo_name,2880,status)
                if(status .ne.  0) then
                    write(errstr,*) "24: STATUS=",status, &
                            " on return from ftinit"
                    call message(AppMsg,Always)
                    call error ("noInit",&
                            "Unable to initialize output image")
                endif

!  Copy the fits header information from the old to new files

                call ftcphd (luna,lunb,status)
                if(status .ne.  0) then
                    write(errstr,*) "25: STATUS=",status, &
                            " on return from ftcphd"
                    call message(AppMsg,Always)
                    call error ("noCopyHeader","Unable to copy header")
                endif

                bitpix = -32
                naxis = 2
                naxes(1) = 900
                naxes(2) = 900
                pcount = 0
                gcount = 1
                group = 1
                call ftpdef (lunb,bitpix,naxis,naxes,pcount, &
                       gcount,.FALSE.,status)
                if(status .ne.  0) then
                    write(errstr,*) "26: STATUS=",status, &
                            " on return from ftpdef"
                    call message(AppMsg,Always)
                    call error ("noDef","Unable to define image")
                endif
                call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
                if(status .ne.  0) then
                    write(errstr,*) "27: STATUS=",status, &
                            " on return from ftmkey"
                    call message(AppMsg,Always)
                    call error ("noKeyMod","Unable to modify keyword")
                endif

!  Write the output image

                call ftp2de (lunb,group,naxes(1),naxes(1), &
                        naxes(2),outim,status)
                if(status .ne.  0) then
                    write(errstr,*) "28: STATUS=",status, &
                            " on return from ftp2de"
                    call message(AppMsg,Always)
                    call error ("noImageWrite","Unable write image")
                endif

!  Close the output file

                call ftclos(luna,status)
                if(status .ne. 0) then
                    write(errstr,*) "29a: STATUS=",status, &
                            " on return from ftclos"
                    call message(AppMsg,Always)
                    call error ("noClose","Cannot close file")
                endif
                call ftclos(lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "29b: STATUS=",status, &
                            " on return from ftclos"
                    call message(AppMsg,Always)
                    call error ("noClose","Cannot close file")
                endif
                call ftfiou (luna,status)
                if(status .ne. 0) then
                    write(errstr,*) "30a: STATUS=",status, &
                            " on return from ftfiou"
                    call message(AppMsg,Always)
                    call error ("noFree","Cannot free LUN")
                endif
                call ftfiou (lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "30b: STATUS=",status, &
                            " on return from ftfiou"
                    call message(AppMsg,Always)
                    call error ("noFree","Cannot free LUN")
                endif
            endif
        endif

!  Set up for the soft proton images
!  First, reset control parameters and zero the output images

        if(iprot) then
            doit = .TRUE.
            do i=1,900
                do ii=1,900
                    outim(i,ii) = 0.0
                enddo
            enddo

!  Read in the soft proton images

            do ib=1,nbands
                do n=1,ndata
                    elow = iel(ib)
                    ehigh = ieh(ib)
                    if(doit .eqv. .TRUE.) then
                        prefi = prefix(n)
                        if((pre(1) .eq. '1') .or. &
                                (pre(1) .eq. '2')) then
                          if(elow .ge. 1000) then
                            if(ehigh .ge. 10000) then
                              write(im_name,3091) prefix(n), &
                                           elow, ehigh
 3091                         format('mos',a5,'-prot-im-sky-',i4, &
                                           '-',i5,'.fits')
                            else
                              write(im_name,3092) prefix(n), &
                                       elow, ehigh
 3092                         format('mos',a5,'-prot-im-sky-',i4, &
                                       '-',i4,'.fits')
                            endif
                          elseif(ehigh .lt. 1000) then
                            write(im_name,3093) prefix(n), elow, ehigh
 3093                       format('mos',a5,'-prot-im-sky-',i3,'-', &
                                    i3,'.fits')
                          elseif(ehigh .ge. 10000) then
                            write(im_name,3094) prefix(n), elow, ehigh
 3094                       format('mos',a5,'-prot-im-sky-',i3,'-', &
                                    i5,'.fits')
                          else
                            write(im_name,3095) prefix(n), elow, ehigh
 3095                       format('mos',a5,'-prot-im-sky-',i3,'-', &
                                    i4,'.fits')
                          endif
                        else
                          do i=1,4
                            pr(i) = pre(i)
                          enddo
                          if(elow .ge. 1000) then
                            if(ehigh .ge. 10000) then
                              write(im_name,3096) prefix(n), elow, ehigh
 3096                         format('pn',a4,'-prot-im-sky-',i4, &
                                    '-',i5,'.fits')
                            else
                              write(im_name,3097) prefix(n), elow, ehigh
 3097                         format('pn',a4,'-prot-im-sky-',i4, &
                                        '-',i4,'.fits')
                            endif
                          elseif(ehigh .lt. 1000) then
                            write(im_name,3098) prefix(n), elow, ehigh
 3098                       format('pn',a4,'-prot-im-sky-',i3,'-', &
                                    i3,'.fits')
                          elseif(ehigh .ge. 10000) then
                            write(im_name,3099) prefix(n), elow, ehigh
 3099                       format('pn',a4,'-prot-im-sky-',i3,'-', &
                                    i5,'.fits')
                          else
                            write(im_name,3100) prefix(n), elow, ehigh
 3100                       format('pn',a4,'-prot-im-sky-',i3,'-', &
                                    i4,'.fits')
                          endif
                        endif

                        write(errstr,*) "Retrieving SP image: ", &
                                trim(im_name)
                        call message(AppMsg,Sparse)
                        call retfit(im_name,image,ini4, &
                           1,900,900,ctype,crval,cdelt,crpix,status)
                        if(status .ne. 0) then
                          write(errstr,*) "STATUS=",status, &
                                    " on return from retfit"
                          call message(AppMsg,Sparse)
                          write(errstr,*) "Error reading QPB image"
                          call message(AppMsg,Sparse)
                          write(errstr,*) &
                              "    Not a problem but deselect SP in"
                          call message(AppMsg,Sparse)
                          write(errstr,*) &
                              "    adapt and bin_image"
                          call message(AppMsg,Sparse)
                          doit = .FALSE.
                        endif

!  Combine the images

                        ttot = 0.0
                        do i=1,900
                            do ii=1,900
                                outim(i,ii) = outim(i,ii) + &
                                        image(i,ii)*mask(i,ii,n)
                                ttot = ttot + image(i,ii)*mask(i,ii,n)
                                nback(ib) = nback(ib) + &
                                        image(i,ii)*mask(i,ii,n)
                            enddo
                        enddo
                        totprot = totprot + ttot
                        write(errstr,*) prefix(n), " SP counts:", ttot
                        call message(AppMsg,Always)
                    endif
                enddo
            enddo

!  Now write out the merged soft proton file, first get two unused luns

            if(doit .eqv. .TRUE.) then
                write(errstr,*) "Total SP counts:", totprot
                call message(AppMsg,Always)
                call ftgiou(luna,status)
                if(status .ne. 0) then
                    write(errstr,*) "31a: STATUS=",status, &
                            " on return from ftgiou"
                    call message(AppMsg,Always)
                    call error ("noLUN","Cannot aquire LUN")
                endif
                call ftgiou(lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "31b: STATUS=",status, &
                            " on return from ftgiou"
                    call message(AppMsg,Always)
                    call error ("noLUN","Cannot aquire LUN")
                endif

!  Open the old image FITS file

                write(errstr,*) "Opening image: ", trim(im_name)
                call message(AppMsg,Sparse)
                call ftopen (luna,im_name,0,block,status)
                if(status .ne. 0) then
                    write(errstr,*) "32: STATUS=",status, &
                            " on return from ftopen"
                    call message(AppMsg,Always)
                    call error ("noOpen","Cannot open image file")
                endif

!  Create the output soft proton file name

                if(ebmin .ge. 1000) then
                    if(ebmax .ge. 10000) then
                        write(imo_name,3101) ebmin, ebmax
 3101                   format('comb-prot-im-sky-',i4,'-',i5,'.fits')
                    else
                        write(imo_name,3102) ebmin, ebmax
 3102                   format('comb-prot-im-sky-',i4,'-',i4,'.fits')
                    endif
                elseif(ebmax .lt. 1000) then
                    write(imo_name,3103) ebmin, ebmax
 3103               format('comb-prot-im-sky-',i3,'-',i3,'.fits')
                elseif(ebmax .ge. 10000) then
                    write(imo_name,3104) ebmin, ebmax
 3104               format('comb-prot-im-sky-',i3,'-',i5,'.fits')
                else
                    write(imo_name,3105) ebmin, ebmax
 3105               format('comb-prot-im-sky-',i3,'-',i4,'.fits')
                endif

!  Check to see if there is an existing file

                write(errstr,*) "Output file name: ", trim(imo_name)
                call message(AppMsg,Sparse)
                write(errstr,*) "Atempting to open output image file"
                call message(AppMsg,Sparse)
                call ftopen (lunb,imo_name,0,block,status)
                if(status .eq.  0) then
                    write(errstr,*) "File exists with output file name"
                    call message(AppMsg,Sparse)
                    if(clobber) then
                        write(errstr,*) "Clobber the file and continue"
                        call message(AppMsg,Sparse)
                        call ftdelt(lunb,status)
                        if(status .ne. 0) then
                            write(errstr,*) "33: STATUS=", &
                                    status," on return from ftdelt"
                            call message(AppMsg,Always)
                            call error ("badDelete",&
                                    "Error deleting file")
                        endif
                    else
                        write(errstr,*) "34: Output file: ", &
                                trim(imo_name)
                        call message(AppMsg,Always)
                        call error &
                            ("noClobber","File exists and no clobber")
                    endif
                else
                    write(errstr,*) "No existing file so continue"
                    call message(AppMsg,Sparse)
                endif
                status = 0

                write(errstr,*) "Initialize output image: ", &
                        trim(imo_name)
                call message(AppMsg,Sparse)
                call ftinit (lunb,imo_name,2880,status)
                if(status .ne.  0) then
                    write(errstr,*) "35: STATUS=",status, &
                            " on return from ftinit"
                    call message(AppMsg,Always)
                    call error ("noInit",&
                            "Unable to initialize output image")
                endif

!  Copy the fits header information from the old to new files

                call ftcphd (luna,lunb,status)
                if(status .ne.  0) then
                    write(errstr,*) "36: STATUS=",status, &
                            " on return from ftcphd"
                    call message(AppMsg,Always)
                    call error ("noCopyHeader","Unable to copy header")
                endif

                bitpix = -32
                naxis = 2
                naxes(1) = 900
                naxes(2) = 900
                pcount = 0
                gcount = 1
                group = 1
                call ftpdef (lunb,bitpix,naxis,naxes,pcount, &
                        gcount,.FALSE.,status)
                if(status .ne.  0) then
                    write(errstr,*) "37: STATUS=",status, &
                            " on return from ftpdef"
                    call message(AppMsg,Always)
                    call error ("noDef","Unable to define image")
                endif
                call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
                if(status .ne.  0) then
                    write(errstr,*) "38: STATUS=",status, &
                            " on return from ftmkey"
                    call message(AppMsg,Always)
                    call error ("noKeyMod","Unable to modify keyword")
                endif

!  Write the output image

                call ftp2de (lunb,group,naxes(1),naxes(1),naxes(2), &
                        outim,status)
                if(status .ne.  0) then
                    write(errstr,*) "39: STATUS=",status, &
                            " on return from ftp2de"
                    call message(AppMsg,Always)
                    call error ("noImageWrite","Unable write image")
                endif

!  Close the output file

                call ftclos(luna,status)
                if(status .ne. 0) then
                    write(errstr,*) "40a: STATUS=",status, &
                            " on return from ftclos"
                    call message(AppMsg,Always)
                    call error ("noClose","Cannot close file")
                endif
                call ftclos(lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "40b: STATUS=",status, &
                            " on return from ftclos"
                    call message(AppMsg,Always)
                    call error ("noClose","Cannot close file")
                endif
                call ftfiou (luna,status)
                if(status .ne. 0) then
                    write(errstr,*) "41a: STATUS=",status, &
                            " on return from ftfiou"
                    call message(AppMsg,Always)
                    call error ("noFree","Cannot free LUN")
                endif
                call ftfiou (lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "41b: STATUS=",status, &
                            " on return from ftfiou"
                    call message(AppMsg,Always)
                    call error ("noFree","Cannot free LUN")
                endif
            endif
        endif

!  Set up for the swcx images
!  First, reset control parameters and zero the output images

        if(iswcx) then
            status = 0
            doit = .TRUE.
            do i=1,900
                do ii=1,900
                    outim(i,ii) = 0.0
                enddo
            enddo

!  Read in the swcx images

            do ib=1,nbands
                do n=1,ndata
                    elow = iel(ib)
                    ehigh = ieh(ib)
                    if(doit .eqv. .TRUE.) then
                        prefi = prefix(n)
                        if((pre(1) .eq. '1') .or. &
                                    (pre(1) .eq. '2')) then
                          if(elow .ge. 1000) then
                            if(ehigh .ge. 10000) then
                              write(im_name,3111) prefix(n), &
                                        elow, ehigh
 3111                         format('mos',a5,'-swcx-im-sky-',i4, &
                                        '-',i5,'.fits')
                            else
                              write(im_name,3112) prefix(n), &
                                        elow, ehigh
 3112                         format('mos',a5,'-swcx-im-sky-',i4, &
                                        '-',i4,'.fits')
                            endif
                          elseif(ehigh .lt. 1000) then
                            write(im_name,3113) prefix(n), elow, ehigh
 3113                       format('mos',a5,'-swcx-im-sky-',i3,'-', &
                                    i3,'.fits')
                          elseif(ehigh .ge. 10000) then
                            write(im_name,3114) prefix(n), elow, ehigh
 3114                       format('mos',a5,'-swcx-im-sky-',i3,'-', &
                                    i5,'.fits')
                          else
                            write(im_name,3115) prefix(n), elow, ehigh
 3115                       format('mos',a5,'-swcx-im-sky-',i3,'-', &
                                    i4,'.fits')
                          endif
                        else
                          do i=1,4
                            pr(i) = pre(i)
                          enddo
                          if(elow .ge. 1000) then
                            if(ehigh .ge. 10000) then
                              write(im_name,3116) prefix(n), &
                                        elow, ehigh
 3116                         format('pn',a4,'-swcx-im-sky-',i4, &
                                        '-',i5,'.fits')
                            else
                              write(im_name,3117) prefix(n), &
                                        elow, ehigh
 3117                         format('pn',a4,'-swcx-im-sky-',i4, &
                                        '-',i4,'.fits')
                            endif
                          elseif(ehigh .lt. 1000) then
                            write(im_name,3118) prefix(n), elow, ehigh
 3118                       format('pn',a4,'-swcx-im-sky-',i3,'-', &
                                    i3,'.fits')
                          elseif(ehigh .ge. 10000) then
                            write(im_name,3119) prefix(n), elow, ehigh
 3119                       format('pn',a4,'-swcx-im-sky-',i3,'-', &
                                    i5,'.fits')
                          else
                            write(im_name,3120) prefix(n), elow, ehigh
 3120                       format('pn',a4,'-swcx-im-sky-',i3,'-', &
                                    i4,'.fits')
                          endif
                        endif

                        write(errstr,*) "Retrieving SWCX image: ", &
                                trim(im_name)
                        call message(AppMsg,Sparse)
                        call retfit(im_name,image,ini4, &
                              1,900,900,ctype,crval,cdelt,crpix,status)
                        if(status .ne. 0) then
                          write(errstr,*) "STATUS=",status, &
                                    " on return from retfit"
                          call message(AppMsg,Sparse)
                          write(errstr,*) &
                                    "Error reading SWCX image"
                          call message(AppMsg,Sparse)
                          write(errstr,*) &
                              "    Not a problem but deselect SWXC in"
                          call message(AppMsg,Sparse)
                          write(errstr,*) &
                              "    adapt and bin_image"
                          call message(AppMsg,Sparse)
                          doit = .FALSE.
                        else

!  Combine the images

                          ttot = 0.0
                          do i=1,900
                            do ii=1,900
                              outim(i,ii) = outim(i,ii) + &
                                        image(i,ii)*mask(i,ii,n)
                              ttot = ttot + image(i,ii)*mask(i,ii,n)
                              nback(ib) = nback(ib) +  &
                                        image(i,ii)*mask(i,ii,n)
                            enddo
                          enddo
                          totswcx = totswcx + ttot
                          write(errstr,*) "SWCX counts:", ttot
                          call message(AppMsg,Always)
                        endif
                    endif
                enddo
            enddo

!  Now write out the merged SWCX file, first get two unused luns

            if(doit .eqv. .TRUE.) then
                write(errstr,*) "Total SWXC counts:", totswcx
                call message(AppMsg,Always)
                call ftgiou(luna,status)
                if(status .ne. 0) then
                    write(errstr,*) "42a: STATUS=",status, &
                            " on return from ftgiou"
                    call message(AppMsg,Always)
                    call error ("noLUN","Cannot aquire LUN")
                endif
                call ftgiou(lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "42b: STATUS=",status, &
                            " on return from ftgiou"
                    call message(AppMsg,Always)
                    call error ("noLUN","Cannot aquire LUN")
                endif

!  Open the old image FITS file

                write(errstr,*) "Opening image: ", trim(im_name)
                call message(AppMsg,Sparse)
                call ftopen (luna,im_name,0,block,status)
                if(status .ne. 0) then
                    write(errstr,*) "43: STATUS=",status, &
                            " on return from ftopen"
                    call message(AppMsg,Always)
                    call error ("noOpen","Cannot open image file")
                endif

!  Create the output soft proton file name

                if(ebmin .ge. 1000) then
                    if(ebmax .ge. 10000) then
                        write(imo_name,3121) ebmin, ebmax
 3121                   format('comb-swcx-im-sky-',i4,'-',i5,'.fits')
                    else
                        write(imo_name,3122) ebmin, ebmax
 3122                   format('comb-swcx-im-sky-',i4,'-',i4,'.fits')
                    endif
                elseif(ebmax .lt. 1000) then
                    write(imo_name,3123) ebmin, ebmax
 3123               format('comb-swcx-im-sky-',i3,'-',i3,'.fits')
                elseif(ebmax .ge. 10000) then
                    write(imo_name,3124) ebmin, ebmax
 3124               format('comb-swcx-im-sky-',i3,'-',i5,'.fits')
                else
                    write(imo_name,3125) ebmin, ebmax
 3125               format('comb-swcx-im-sky-',i3,'-',i4,'.fits')
                endif

!  Check to see if there is an existing file

                write(errstr,*) "Output file name: ", &
                        trim(imo_name)
                call message(AppMsg,Sparse)
                write(errstr,*) "Atempting to open output image file"
                call message(AppMsg,Sparse)
                call ftopen (lunb,imo_name,0,block,status)
                if(status .eq.  0) then
                    write(errstr,*) &
                            "File exists with output file name"
                    call message(AppMsg,Sparse)
                    if(clobber) then
                        write(errstr,*) &
                                "Clobber the file and continue"
                        call message(AppMsg,Sparse)
                        call ftdelt(lunb,status)
                        if(status .ne. 0) then
                            write(errstr,*) "44: STATUS=", &
                                    status," on return from ftdelt"
                            call message(AppMsg,Always)
                            call error ("badDelete",&
                                    "Error deleting file")
                        endif
                    else
                        write(errstr,*) "45: Output file: ", &
                                trim(imo_name)
                        call message(AppMsg,Always)
                        call error &
                            ("noClobber","File exists and no clobber")
                    endif
                else
                    write(errstr,*) "No existing file so continue"
                    call message(AppMsg,Sparse)
                endif
                status = 0

                write(errstr,*) "Initialize output image: ", &
                        trim(imo_name)
                call message(AppMsg,Sparse)
                call ftinit (lunb,imo_name,2880,status)
                if(status .ne.  0) then
                    write(errstr,*) "46: STATUS=",status, &
                            " on return from ftinit"
                    call message(AppMsg,Always)
                    call error ("noInit",&
                            "Unable to initialize output image")
                endif

!  Copy the fits header information from the old to new files

                call ftcphd (luna,lunb,status)
                if(status .ne.  0) then
                    write(errstr,*) "47: STATUS=",status, &
                            " on return from ftcphd"
                    call message(AppMsg,Always)
                    call error ("noCopyHeader","Unable to copy header")
                endif

                bitpix = -32
                naxis = 2
                naxes(1) = 900
                naxes(2) = 900
                pcount = 0
                gcount = 1
                group = 1
                call ftpdef (lunb,bitpix,naxis,naxes,pcount, &
                        gcount,.FALSE.,status)
                if(status .ne.  0) then
                    write(errstr,*) "48: STATUS=",status, &
                            " on return from ftpdef"
                    call message(AppMsg,Always)
                    call error ("noDef","Unable to define image")
                endif
                call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
                if(status .ne.  0) then
                    write(errstr,*) "49: STATUS=",status, &
                            " on return from ftmkey"
                    call message(AppMsg,Always)
                    call error ("noKeyMod","Unable to modify keyword")
                endif

!  Write the output image

                call ftp2de (lunb,group,naxes(1),naxes(1),naxes(2), &
                        outim,status)
                if(status .ne.  0) then
                    write(errstr,*) "50: STATUS=",status, &
                            " on return from ftp2de"
                    call message(AppMsg,Always)
                    call error ("noImageWrite","Unable write image")
                endif

!  Close the output file

                call ftclos(luna,status)
                if(status .ne. 0) then
                    write(errstr,*) "51a: STATUS=",status, &
                            " on return from ftclos"
                    call message(AppMsg,Always)
                    call error ("noClose","Cannot close file")
                endif
                call ftclos(lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "51b: STATUS=",status, &
                            " on return from ftclos"
                    call message(AppMsg,Always)
                    call error ("noClose","Cannot close file")
                endif

                call ftfiou (luna,status)
                if(status .ne. 0) then
                    write(errstr,*) "52a: STATUS=",status, &
                            " on return from ftfiou"
                    call message(AppMsg,Always)
                    call error ("noFree","Cannot free LUN")
                endif
                call ftfiou (lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "52b: STATUS=",status, &
                            " on return from ftfiou"
                    call message(AppMsg,Always)
                    call error ("noFree","Cannot free LUN")
                endif
            endif
        endif

!  Zero the output images

        do i=1,900
            do ii=1,900
                outim(i,ii) = 0.0
            enddo
        enddo

!  Read in the exposure images

        do ib=1,nbands
            do n=1,ndata
                elow = iel(ib)
                ehigh = ieh(ib)
                prefi = prefix(n)
                if((pre(1) .eq. '1') .or. (pre(1) .eq. '2')) then
                    if(pre(1) .eq. '1') idet = 1
                    if(pre(1) .eq. '2') idet = 2
                else
                    idet = 3
                    do i=1,4
                        pr(i) = pre(i)
                    enddo
                endif
                if((pre(1) .eq. '1') .or. (pre(1) .eq. '2')) then
                    if(elow .ge. 1000) then
                        if(ehigh .ge. 10000) then
                            write(im_name,3051) prefix(n), elow, ehigh
 3051                       format('mos',a5,'-exp-im-',i4,'-', &
                                    i5,'.fits')
                        else
                            write(im_name,3052) prefix(n), elow, ehigh
 3052                       format('mos',a5,'-exp-im-',i4,'-', &
                                    i4,'.fits')
                        endif
                    elseif(ehigh .lt. 1000) then
                        write(im_name,3053) prefix(n), elow, ehigh
 3053                   format('mos',a5,'-exp-im-',i3,'-',i3,'.fits')
                    elseif(ehigh .ge. 10000) then
                        write(im_name,3054) prefix(n), elow, ehigh
 3054                   format('mos',a5,'-exp-im-',i3,'-',i5,'.fits')
                    else
                        write(im_name,3055) prefix(n), elow, ehigh
 3055                   format('mos',a5,'-exp-im-',i3,'-',i4,'.fits')
                    endif
                else
                    do i=1,4
                        pr(i) = pre(i)
                    enddo
                    if(elow .ge. 1000) then
                        if(ehigh .ge. 10000) then
                            write(im_name,3056) pref, elow, ehigh
 3056                       format('pn',a4,'-exp-im-',i4,'-', &
                                    i5,'.fits')
                        else
                            write(im_name,3057) pref, elow, ehigh
 3057                       format('pn',a4,'-exp-im-',i4,'-', &
                                    i4,'.fits')
                        endif
                    elseif(ehigh .lt. 1000) then
                        write(im_name,3058) pref, elow, ehigh
 3058                   format('pn',a4,'-exp-im-',i3,'-',i3,'.fits')
                    elseif(ehigh .ge. 10000) then
                        write(im_name,3059) pref, elow, ehigh
 3059                   format('pn',a4,'-exp-im-',i3,'-',i5,'.fits')
                    else
                        write(im_name,3060) pref, elow, ehigh
 3060                   format('pn',a4,'-exp-im-',i3,'-',i4,'.fits')
                    endif
                endif

!  Now get the image

                write(errstr,*) "Retrieving exposure image: ", &
                            trim(im_name)
                call message(AppMsg,Sparse)
                call retfit(im_name,image,ini4, &
                        1,900,900,ctype,crval,cdelt,crpix,status)
                if(status .ne. 0) then
                    write(errstr,*) "53: STATUS=",status, &
                            " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Cannot return image")
                endif

!  This is a bit painful, but we need to know the FILTER
!  keyword for the proper scaling of the OOT events

!  Get an unused lun

                call ftgiou(lun,status)
                if(status .ne. 0) then
                    write(errstr,*) "54: STATUS=",status, &
                            " on return from ftgiou"
                    call message(AppMsg,Always)
                    call error ("noLUN","Cannot aquire LUN")
                endif

!  Open the FITS file

                write(errstr,*) "Opening image: ", trim(im_name)
                call message(AppMsg,Sparse)
                call ftopen (lun,im_name,0,block,status)
                if(status .ne. 0) then
                    write(errstr,*) "55: STATUS=",status, &
                            " on return from ftopen"
                    call message(AppMsg,Always)
                    call error ("noOpen","Cannot open image file")
                endif

!  Get the filter and then the filter scale factor

                call ftgkys (lun,'FILTER',filter,com,status)
                if(status .ne. 0) then
                    write(errstr,*) "56: STATUS=",status, &
                            " on return from ftopen"
                    call message(AppMsg,Always)
                    call error ("noGetKey","Cannot return keyword")
                else
                    call hard_scale (xmmdir,alpha,idet, &
                            filt,islow,ishigh,fscal)
                    write(errstr,*) "Detector, Filter: ", &
                            idet, '  ', filt
                    call message(AppMsg,Sparse)
                    write(errstr,*) "Filter scale factor: ", fscal
                    call message(AppMsg,Sparse)
                endif

!  Close the FITS file

                call ftclos(lun,status)
                if(status .ne. 0) then
                    write(errstr,*) "57: STATUS=",status, &
                            " on return from ftclos"
                    call message(AppMsg,Always)
                    call error ("noClose","Cannot close file")
                endif

!  Now get the image

                write(errstr,*) "Retrieving exposure image: ", &
                        trim(im_name)
                call message(AppMsg,Sparse)
                call retfit(im_name,image,ini4, &
                        1,900,900,ctype,crval,cdelt,crpix,status)
                if(status .ne. 0) then
                    write(errstr,*) "58: STATUS=", &
                            status, " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading exposure image")
                endif

!  Combine and mask the images

                ttot = 0.0
                do i=1,900
                    do ii=1,900
                        if(nbands .gt. 1) then
                            outim(i,ii) = outim(i,ii) + &
                                    (ncounts(ib) - nback(ib))* &
                                    fscal*image(i,ii)*mask(i,ii,n)
                            ttot = ttot + fscal*image(i,ii)*mask(i,ii,n)
                        else
                            outim(i,ii) = outim(i,ii) +  &
                                    fscal*image(i,ii)*mask(i,ii,n)
                            ttot = ttot + fscal*image(i,ii)*mask(i,ii,n)
                        endif
                    enddo
                enddo
                totexp = totexp + ttot
                write(errstr,*) prefix(n), " Exposure:", ttot
                call message(AppMsg,Always)
            enddo
        enddo
        write(errstr,*) "Total Exposure:", totexp
        call message(AppMsg,Always)

        if(nbands .gt. 1) then
            ttot = 0.0
            do ib=1,nbands
                ttot = ttot + (ncounts(ib) - nback(ib))
                write(errstr,*) "Total number of counts in band:", &
                        ib,":", ncounts(ib)
                call message(AppMsg,Sparse)
                write(errstr,*) "Total number of background counts ", &
                        "in band:",ib,":", nback(ib)
                call message(AppMsg,Sparse)
            enddo
            write(errstr,*) "Total counts:", ttot
            call message(AppMsg,Sparse)
            do i=1,900
                do ii=1,900
                    outim(i,ii) = outim(i,ii)/ttot
                enddo
            enddo
        endif

!  Now write out the merged count file, first get two unused luns

        call ftgiou(luna,status)
        if(status .ne. 0) then
            write(errstr,*) "59a: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif
        call ftgiou(lunb,status)
        if(status .ne. 0) then
            write(errstr,*) "59b: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

!  Open the old image FITS file

        write(errstr,*) "Opening template image: ", trim(im_name)
        call message(AppMsg,Sparse)
        call ftopen (luna,im_name,0,block,status)
        if(status .ne. 0) then
            write(errstr,*) "60: STATUS=",status, &
                    " on return from ftopen"
            call message(AppMsg,Always)
            call error ("noOpen","Cannot open image file")
        endif

!  Create the output exposure file name

        if(ebmin .ge. 1000) then
            if(ebmax .ge. 10000) then
                write(imo_name,3061) ebmin, ebmax
 3061           format('comb-exp-im-',i4,'-',i5,'.fits')
            else
                write(imo_name,3062) ebmin, ebmax
 3062           format('comb-exp-im-',i4,'-',i4,'.fits')
            endif
        elseif(ebmax .lt. 1000) then
            write(imo_name,3063) ebmin, ebmax
 3063       format('comb-exp-im-',i3,'-',i3,'.fits')
        elseif(ebmax .ge. 10000) then
            write(imo_name,3064) ebmin, ebmax
 3064       format('comb-exp-im-',i3,'-',i5,'.fits')
        else
            write(imo_name,3065) ebmin, ebmax
 3065       format('comb-exp-im-',i3,'-',i4,'.fits')
        endif

!  Check to see if there is an existing file

        write(errstr,*) "Output file name: ", trim(imo_name)
        call message(AppMsg,Sparse)
        write(errstr,*) "Atempting to open output exposure file"
        call message(AppMsg,Sparse)
        call ftopen (lunb,imo_name,0,block,status)
        if(status .eq.  0) then
            write(errstr,*) "File exists with output file name"
            call message(AppMsg,Sparse)
            if(clobber) then
                write(errstr,*) "Clobber the file and continue"
                call message(AppMsg,Sparse)
                call ftdelt(lunb,status)
                if(status .ne. 0) then
                    write(errstr,*) "61: STATUS=", &
                            status," on return from ftdelt"
                    call message(AppMsg,Always)
                    call error ("badDelete","Error deleting file")
                endif
            else
                write(errstr,*) "62: Output file: ", &
                        trim(imo_name)
                call message(AppMsg,Always)
                call error ("noClobber","File exists and no clobber")
            endif
        else
            write(errstr,*) "No existing file so continue"
            call message(AppMsg,Sparse)
        endif
        status = 0

        write(errstr,*) "Initialize output image: ", &
                        trim(imo_name)
        call message(AppMsg,Sparse)
        call ftinit (lunb,imo_name,2880,status)
        if(status .ne.  0) then
            write(errstr,*) "63: STATUS=",status, &
                    " on return from ftinit"
            call message(AppMsg,Always)
            call error ("noInit","Unable to initialize output image")
        endif

!  Copy the fits header information from the old to new files

        call ftcphd (luna,lunb,status)
        if(status .ne.  0) then
            write(errstr,*) "64: STATUS=",status, &
                            " on return from ftcphd"
            call message(AppMsg,Always)
            call error ("noCopyHeader","Unable to copy header")
        endif

        bitpix = -32
        naxis = 2
        naxes(1) = 900
        naxes(2) = 900
        pcount = 0
        gcount = 1
        group = 1
        call ftpdef (lunb,bitpix,naxis,naxes,pcount, &
                gcount,.FALSE.,status)
        if(status .ne.  0) then
            write(errstr,*) "65: STATUS=",status, &
                            " on return from ftpdef"
            call message(AppMsg,Always)
            call error ("noDef","Unable to define image")
        endif
        call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
        if(status .ne.  0) then
            write(errstr,*) "66: STATUS=",status, &
                            " on return from ftmkey"
            call message(AppMsg,Always)
            call error ("noKeyMod","Unable to modify keyword")
        endif

!  Write the output image

        write(errstr,*) "Write output image: ", trim(imo_name)
        call message(AppMsg,Sparse)
        call ftp2de (lunb,group,naxes(1),naxes(1),naxes(2), &
                    outim,status)
        if(status .ne.  0) then
            write(errstr,*) "67: STATUS=",status, &
                            " on return from ftp2de"
            call message(AppMsg,Always)
            call error ("noImageWrite","Unable write image")
        endif

!  Close the output file

        call ftclos(luna,status)
        if(status .ne. 0) then
            write(errstr,*) "68a: STATUS=",status, &
                            " on return from ftclos"
            call message(AppMsg,Always)
            call error ("noClose","Cannot close file")
        endif
        call ftclos(lunb,status)
        if(status .ne. 0) then
            write(errstr,*) "68b: STATUS=",status, &
                            " on return from ftclos"
            call message(AppMsg,Always)
            call error ("noClose","Cannot close file")
        endif
        call ftfiou (luna,status)
        if(status .ne. 0) then
            write(errstr,*) "69a: STATUS=",status, &
                            " on return from ftfiou"
            call message(AppMsg,Always)
            call error ("noFree","Cannot free LUN")
        endif
        call ftfiou (lunb,status)
        if(status .ne. 0) then
            write(errstr,*) "69b: STATUS=",status, &
                            " on return from ftfiou"
            call message(AppMsg,Always)
            call error ("noFree","Cannot free LUN")
        endif

        write(errstr,*) "Total Observed Counts:", totcou
        call message(AppMsg,Always)
        write(errstr,*) "Total Exposure:", totexp
        call message(AppMsg,Always)
        write(errstr,*) "Total Particle Background Counts:", totback
        call message(AppMsg,Always)
        write(errstr,*) "Total Soft Proton Counts:", totprot
        call message(AppMsg,Always)
        write(errstr,*) "Total Solar Wind Charge Exchange Counts:", &
                totswcx
        call message(AppMsg,Always)

        return

    deallocate(iel,ieh,prefix)

  end subroutine comb

end module comb_mod
