!+
! Module: esprep_mosaic_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: esprep_mosaic_mod.f90,v 1.2 2012/03/25 13:27:08 Snowden Exp $
!-
module esprep_mosaic_mod

  use dal
  use param
  use errorHandling

  implicit none

  private

  public esprep_mosaic

  contains

  !---------------------------------------------------------------------
  subroutine esprep_mosaic
  !---------------------------------------------------------------------

        character (len=4) :: coord
        character (len=8) :: colnam(12), tform(12), ttype(12), tunit(12)
        character (len=20) :: extname
        character (len=80) :: com, eventsin, eventsout
        logical :: anyf, clobber

        integer :: block, blockin, hdutypein, &
            hdutypeout, i, icol, idum, ix, ixmax, ixmin, &
            ixtin, ixtout, iy, iymax, iymin, iytin, iytout, & 
            lunin, lunout, nchan, ncols, &
            tfields, varidat
        integer :: status = 0
        real :: decobj, pas, raobj
        double precision :: dec, dect, dum, dxpix, dypix, pa, ra, &
            rat, rot, x, xinc, xmax, xmin, xpix, xrpix, xrval, &
            y, yinc, ymax, ymin, ypix, yrpix, yrval

!  Grab the parameters

        eventsin = stringParameter("eventsin")
        write(errstr,*) "Input event file: ", trim(eventsin)
        call message(AppMsg,Always)
        eventsout = stringParameter("eventsout")
        write(errstr,*) "Output event file: ", trim(eventsout)
        call message(AppMsg,Always)
        raobj = realParameter("raobj")
        write(errstr,*) "Subpointing RA: ", raobj
        call message(AppMsg,Always)
        decobj = realParameter("decobj")
        write(errstr,*) "Subpointing Dec: ", decobj
        call message(AppMsg,Always)
        pas = realParameter("pa")
        write(errstr,*) "Rotation Angle: ", pas
        call message(AppMsg,Always)
        clobber = booleanParameter("clobber")
        write(errstr,*) "Clobber: ", clobber
        call message(AppMsg,Always)

        coord = '-TAN'

!  Get an unused lun for the caldb file

        call ftgiou(lunin,status)
        if(status .ne. 0) then
            write(errstr,*) "01: STATUS=", & 
                    status, " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

!  Open the FITS file

        write(errstr,*) "Opening input event file: ", trim(eventsin)
        call message(AppMsg,Sparse)
        call ftopen (lunin,eventsin,0,blockin,status)
        if(status .ne. 0) then
            write(errstr,*) "02: STATUS=",status, &
                     " on return from ftopen"
            call message(AppMsg,Always)
            call error ("noOpen","Cannot open input event file")
        endif

        call ftgiou(lunout,status)
        if(status .ne. 0) then
            write(errstr,*) "03: STATUS=", & 
                    status, " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

!  Initialize the output FITS file

        write(errstr,*) "Opening output event file: ", trim(eventsout)
        call message(AppMsg,Sparse)
        call ftopen (lunout,eventsout,0,block,status)
        if(status .eq. 0) then
            write(errstr,*) "File exists with output file name"
            call message(AppMsg,Sparse)
            if(clobber) then
                write(errstr,*) "Clobber the file and continue"
                call message(AppMsg,Sparse)
                call ftdelt(lunout,status)
                if(status .ne. 0) then
                    write(errstr,*) "04: STATUS=",status, &
                            " on return from ftdelt"
                    call message(AppMsg,Always)
                    call error ("badDelete","Error deleting file")
                endif
            else
                write(errstr,*) "05: Output file: ", trim(eventsout)
                call message(AppMsg,Always)
                call error ("noClobber","File exists and no clobber")
            endif
        else
            write(errstr,*) "No existing file so continue"
            call message(AppMsg,Sparse)
        endif

        status = 0

!  Initialize the output file

        write(errstr,*) "Initialize output image: ", trim(eventsout)
        call message(AppMsg,Sparse)
        call ftinit (lunout,eventsout,blockin,status)
        if(status .ne.  0) then
            write(errstr,*) "06: STATUS=",status, &
                    " on return from ftinit"
            call message(AppMsg,Always)
            call error ("noInit","Unable to initialize output image")
        endif

!  Copy the old file to the new file

        call ftcpfl (lunin,lunout,1,1,1,status)
        if(status .ne.  0) then
            write(errstr,*) "07: STATUS=",status, &
                    " on return from ftcphd"
            call message(AppMsg,Always)
            call error ("noCopyHeader","Unable to copy header")
        endif
        call ftclos(lunout,status)
        if(status .ne. 0) then
            write(errstr,*) "08: STATUS=",status, &
                    " on return from ftclos"
            call message(AppMsg,Always)
            call error ("noClose","Cannot close file")
        endif
        call ftopen (lunout,eventsout,1,blockin,status)
        if(status .ne. 0) then
            write(errstr,*) "09: STATUS=",status, &
                    " on return from ftopen"
            write(errstr,*) "Output event file: ", trim(eventsout)
            call message(AppMsg,Always)
            call error ("noOpen","Cannot open output event file")
        endif


!  Change to reference position values in the primary header

        call ftgkyd (lunin,'RA_OBJ',ra,com,status)
        call ftgkyd (lunin,'DEC_OBJ',dec,com,status)
        ra = raobj
        dec = decobj
        pa = pas
        call ftgkyd (lunin,'RA_OBJ',dum,com,status)
        call ftukyd (lunout,'RA_OBJ',ra,12,com,status)
        call ftgkyd (lunin,'DEC_OBJ',dum,com,status)
        call ftukyd (lunout,'DEC_OBJ',dec,12,com,status)
        call ftgkyd (lunin,'RA_NOM',dum,com,status)
        call ftukyd (lunout,'RA_NOM',ra,12,com,status)
        call ftgkyd (lunin,'DEC_NOM',dum,com,status)
        call ftukyd (lunout,'DEC_NOM',dec,12,com,status)
        call ftgkyd (lunin,'REFXCRVL',dum,com,status)
        call ftukyd (lunout,'REFXCRVL',ra,12,com,status)
        call ftgkyd (lunin,'REFYCRVL',dum,com,status)
        call ftukyd (lunout,'REFYCRVL',dec,12,com,status)
        call ftgkyd (lunin,'RA_PNT',dum,com,status)
        call ftukyd (lunout,'RA_PNT',ra,12,com,status)
        call ftgkyd (lunin,'DEC_PNT',dum,com,status)
        call ftukyd (lunout,'DEC_PNT',dec,12,com,status)
        call ftgkyd (lunin,'PA_PNT',dum,com,status)
        call ftukyd (lunout,'PA_PNT',pa,12,com,status)
        if(status .ne. 0) then
            write(errstr,*) "10: STATUS=",status, &
                    " on return from ft*kyd"
            write(errstr,*) "Output event file: ", trim(eventsout)
            call message(AppMsg,Always)
            call error ("noKeygetupdate","Cannot get or update keyword")
        endif
        call ftflus(lunout,status)
        if(status .ne. 0) then
            write(errstr,*) "11: STATUS=",status, &
                    " on return from ftflus"
            write(errstr,*) "Output event file: ", trim(eventsout)
            call message(AppMsg,Always)
            call error ("noFlush","Cannot flush the event file")
        endif

!  Move to the correct extension

        call ftmrhd (lunin,1,hdutypein,status)
        if(status .ne. 0) then
            write(errstr,*) "12: STATUS=",status, &
                    " on return from ftmrhd"
            write(errstr,*) "Input event file: ", trim(eventsin)
            call message(AppMsg,Always)
            call error ("noMove","Cannot move to the event extension")
        endif

        call ftmrhd (lunout,1,hdutypeout,status)
        if(status .ne. 0) then
            write(errstr,*) "13: STATUS=",status, &
                    " on return from ftmrhd"
            write(errstr,*) "Output event file: ", trim(eventsout)
            call message(AppMsg,Always)
            call error ("noMove","Cannot move to the event extension")
        endif

!  Read the FITS file header information

        call ftghbn (lunin,12,nchan,tfields,ttype,tform, &
                tunit,extname,varidat,status)
        if(status .ne. 0) then
            write(errstr,*) "14: STATUS=",status, &
                    " on return from ftghbn"
            call message(AppMsg,Always)
            write(errstr,*) "Output event file: ", trim(eventsout)
            call message(AppMsg,Always)
            call error ("noHeader","Cannot read the header information")
        endif
        write(errstr,*) "Extension: ", extname
        call message(AppMsg,Sparse)
        if(extname .ne. 'EVENTS') then
            write(errstr,*) "15: Incorrect extension: ", extname, &
                    " should be EVENTS"
            call message(AppMsg,Always)
            write(errstr,*) "Input event file: ", trim(eventsin)
            call message(AppMsg,Always)
            call error ("noHeader","Cannot read the header information")
        endif

!  Read in the WCS information

        call ftgkyd (lunin,'REFXCRVL',xrval,com,status)
        call ftgkyd (lunin,'REFYCRVL',yrval,com,status)
        call ftgkyd (lunin,'REFXCRPX',xrpix,com,status)
        call ftgkyd (lunin,'REFYCRPX',yrpix,com,status)
        call ftgkyd (lunin,'REFXCDLT',xinc,com,status)
        call ftgkyd (lunin,'REFYCDLT',yinc,com,status)

         if(status .ne. 0) then
            write(errstr,*) "16: STATUS=",status, &
                    " on return from ftgkyd"
            call message(AppMsg,Always)
            call error ("noKeyGet","Cannot get keywords")
        endif

        write(errstr,*) "Input File: ", eventsin
        call message(AppMsg,Sparse)
        write(errstr,*) "Subpoint Central Coords: ", ra, dec
        call message(AppMsg,Sparse)
        write(errstr,*) "Rotation Angle: ", pa
        call message(AppMsg,Sparse)
        write(errstr,*) "Project Central Coords: ", xrval, yrval
        call message(AppMsg,Sparse)
        write(errstr,*) "Project Central Pixel: ", xrpix, yrpix
        call message(AppMsg,Sparse)
        write(errstr,*) "Project Pixel Size: ", xinc, yinc
        call message(AppMsg,Sparse)
        write(errstr,*) "Coordinates: ", coord
        call message(AppMsg,Sparse)

!  Get the X,Y offsets

        rot = 0
        call ftxypx (ra,dec,xrval,yrval,xrpix,yrpix, &
                 xinc,yinc,rot,coord,xpix,ypix,status) 
        if(status .ne. 0) then
            write(errstr,*) "17: STATUS=",status, &
                    " on return from ftxypx"
            call message(AppMsg,Always)
            call error ("badTransform","Failure on transforming to X,Y")
        endif
        write(errstr,*) "Subpoint Central Pixel: ", xpix, ypix
        call message(AppMsg,Sparse)
        dxpix = xrpix - xpix
        dypix = yrpix - ypix
        write(errstr,*) "Required Pixel Offset: ", dxpix, dypix
        call message(AppMsg,Sparse)

!  Read in the column names

        call ftgkns (lunin,'TTYPE',1,10,colnam,ncols,status)
        if(status .ne. 0) then
            write(errstr,*) "18: STATUS=",status, &
                    " on return from ftgkns"
            call message(AppMsg,Always)
            call error ("badReadColumnNames",&
                    "Failure on reading the column names")
        endif

!  Extract the column numbers

        ix = 0
        iy = 0
        do icol=1,ncols
            if(colnam(icol) .eq. 'X') ix = icol
            if(colnam(icol) .eq. 'Y') iy = icol
        enddo

!  Cycle through the events

        xmin = 10000000
        xmax = 0
        ymin = 10000000
        ymax = 0
        do i=1,nchan
            if(ix .ne. 0) call ftgcvj(lunin,ix,i,1,1,0, &
                  ixtin,anyf,status)
            if(iy .ne. 0) call ftgcvj(lunin,iy,i,1,1,0, &
                  iytin,anyf,status)
            x = ixtin
            y = iytin
            if(status .ne. 0) then
                write(errstr,*) "19: STATUS=",status, &
                        " on return from ftgcvj"
                call message(AppMsg,Always)
                call error ("badReadData",&
                        "Failure on reading data")
            endif

!  Transform the coords

            call ftwldp(x,y,xrval,yrval,xrpix,yrpix, &
                    xinc,yinc,rot,coord,rat,dect,status)
            if(status .ne. 0) then
                write(errstr,*) "20: STATUS=",status, &
                        " on return from ftwldp"
                call message(AppMsg,Always)
                call error ("badTransform",&
                        "Failure on transformation")
            endif
            call ftxypx (rat,dect,ra,dec,xrpix,yrpix, &
                        xinc,yinc,rot,coord,xpix,ypix,status) 
            if(status .ne. 0) then
                write(errstr,*) "21: STATUS=",status, &
                        " on return from ftxypx"
                call message(AppMsg,Always)
                call error ("badTransform",&
                        "Failure on transformation")
            endif
            ixtout = xpix
            iytout = ypix
            if(xpix .lt. xmin) xmin = xpix
            if(xpix .gt. xmax) xmax = xpix
            if(ypix .lt. ymin) ymin = ypix
            if(ypix .gt. ymax) ymax = ypix

!  Write the transformed values into the table

            call ftpclj(lunout,ix,i,1,1,ixtout,status)
            call ftpclj(lunout,iy,i,1,1,iytout,status)
            if(status .ne. 0) then
                write(errstr,*) "22: STATUS=",status, &
                        " on return from ftpcvj"
                call message(AppMsg,Always)
                call error ("badReadData",&
                        "Failure on writing data")
            endif
        enddo

!  Change to reference position values

        call ftgkyd (lunout,'RA_OBJ',dum,com,status)
        call ftukyd (lunout,'RA_OBJ',ra,12,com,status)
        call ftgkyd (lunout,'DEC_OBJ',dum,com,status)
        call ftukyd (lunout,'DEC_OBJ',dec,12,com,status)
        call ftgkyd (lunout,'RA_NOM',dum,com,status)
        call ftukyd (lunout,'RA_NOM',ra,12,com,status)
        call ftgkyd (lunout,'DEC_NOM',dum,com,status)
        call ftukyd (lunout,'DEC_NOM',dec,12,com,status)
        call ftgkyd (lunout,'REFXCRVL',dum,com,status)
        call ftukyd (lunout,'REFXCRVL',ra,12,com,status)
        call ftgkyd (lunout,'REFYCRVL',dum,com,status)
        call ftukyd (lunout,'REFYCRVL',dec,12,com,status)
        call ftgkyd (lunout,'RA_PNT',dum,com,status)
        call ftukyd (lunout,'RA_PNT',ra,12,com,status)
        call ftgkyd (lunout,'DEC_PNT',dum,com,status)
        call ftukyd (lunout,'DEC_PNT',dec,12,com,status)
        call ftgkyd (lunout,'PA_PNT',dum,com,status)
        call ftukyd (lunout,'PA_PNT',pa,12,com,status)
        call ftgkyd (lunout,'TCRVL6',dum,com,status)
        call ftukyd (lunout,'TCRVL6',ra,12,com,status)
        call ftgkyd (lunout,'TCRVL7',dum,com,status)
        call ftukyd (lunout,'TCRVL7',dec,12,com,status)
        if(status .ne. 0) then
            write(errstr,*) "23: STATUS=",status, &
                    " on return from ftgkyd or ftukyd"
            call message(AppMsg,Always)
            call error ("badKeyUpdate","Failure on updating keywords")
        endif

        ixmin = xmin + 0.5
        ixmax = xmax + 0.5
        iymin = ymin + 0.5
        iymax = ymax + 0.5

        call ftgkyj (lunout,'REFXDMIN',idum,com,status)
        call ftukyj (lunout,'REFXDMIN',ixmin,com,status)
        call ftgkyj (lunout,'REFXDMAX',idum,com,status)
        call ftukyj (lunout,'REFXDMAX',ixmax,com,status)
        call ftgkyj (lunout,'REFYDMIN',idum,com,status)
        call ftukyj (lunout,'REFYDMIN',iymin,com,status)
        call ftgkyj (lunout,'REFYDMAX',idum,com,status)
        call ftukyj (lunout,'REFYDMAX',iymax,com,status)
        call ftgkyj (lunout,'TDMIN6',idum,com,status)
        call ftukyj (lunout,'TDMIN6',ixmin,com,status)
        call ftgkyj (lunout,'TDMAX6',idum,com,status)
        call ftukyj (lunout,'TDMAX6',ixmax,com,status)
        call ftgkyd (lunout,'TCRVL6',dum,com,status)
        call ftukyd (lunout,'TCRVL6',ra,12,com,status)
        call ftgkyj (lunout,'TDMIN7',idum,com,status)
        call ftukyj (lunout,'TDMIN7',iymin,com,status)
        call ftgkyj (lunout,'TDMAX7',idum,com,status)
        call ftukyj (lunout,'TDMAX7',iymax,com,status)
        call ftgkyd (lunout,'TCRVL7',dum,com,status)
        call ftukyd (lunout,'TCRVL7',dec,12,com,status)
        if(status .ne. 0) then
            write(errstr,*) "24: STATUS=",status, &
                    " on return from ftgky* or ftuky*"
            call message(AppMsg,Always)
            call error ("badKeyUpdate","Failure on updating keywords")
        endif

!  Now go to the primary header

        call ftmrhd (lunin,-1,hdutypein,status)
        if(status .ne. 0) then
            write(errstr,*) "25: STATUS=",status, &
                    " on return from ftmrhd"
            call message(AppMsg,Always)
            call error ("badMove","Failure on moving to extension")
        endif
        call ftmrhd (lunout,-1,hdutypeout,status)
        if(status .ne. 0) then
            write(errstr,*) "26: STATUS=",status, &
                    " on return from ftmrhd"
            call message(AppMsg,Always)
            call error ("badMove","Failure on moving to extension")
        endif

        call ftgkyj (lunout,'REFXDMIN',idum,com,status)
        call ftukyj (lunout,'REFXDMIN',ixmin,com,status)
        call ftgkyj (lunout,'REFXDMAX',idum,com,status)
        call ftukyj (lunout,'REFXDMAX',ixmax,com,status)
        call ftgkyj (lunout,'REFYDMIN',idum,com,status)
        call ftukyj (lunout,'REFYDMIN',iymin,com,status)
        call ftgkyj (lunout,'REFYDMAX',idum,com,status)
        call ftukyj (lunout,'REFYDMAX',iymax,com,status)
        if(status .ne. 0) then
            write(errstr,*) "27: STATUS=",status, &
                    " on return from ftgky* or ftuky*"
            call message(AppMsg,Always)
            call error ("badKeyUpdate","Failure on updating keywords")
        endif
        call ftmrhd (lunin,2,hdutypein,status)
        if(status .ne. 0) then
            write(errstr,*) "28: STATUS=",status, &
                    " on return from ftmrhd"
            call message(AppMsg,Always)
            call error ("badMove","Failure on moving to extension")
        endif
        call ftmrhd (lunout,2,hdutypeout,status)
        call ftmrhd (lunin,2,hdutypein,status)
        if(status .ne. 0) then
            write(errstr,*) "29: STATUS=",status, &
                    " on return from ftmrhd"
            call message(AppMsg,Always)
            call error ("badMove","Failure on moving to extension")
        endif

        call ftgkyd (lunin,'RA_OBJ',ra,com,status)
        call ftgkyd (lunin,'DEC_OBJ',dec,com,status)
        call ftgkyd (lunin,'REFXCRVL',dum,com,status)
        call ftukyd (lunout,'REFXCRVL',ra,12,com,status)
        call ftgkyd (lunin,'REFYCRVL',dum,com,status)
        call ftukyd (lunout,'REFYCRVL',dec,12,com,status)
        call ftgkyd (lunin,'RA_PNT',dum,com,status)
        call ftukyd (lunout,'RA_PNT',ra,12,com,status)
        call ftgkyd (lunin,'DEC_PNT',dum,com,status)
        call ftukyd (lunout,'DEC_PNT',dec,12,com,status)
        call ftgkyd (lunin,'PA_PNT',dum,com,status)
        call ftukyd (lunout,'PA_PNT',pa,12,com,status)
        call ftgkyj (lunout,'REFXDMIN',idum,com,status)
        call ftukyj (lunout,'REFXDMIN',ixmin,com,status)
        call ftgkyj (lunout,'REFXDMAX',idum,com,status)
        call ftukyj (lunout,'REFXDMAX',ixmax,com,status)
        call ftgkyj (lunout,'REFYDMIN',idum,com,status)
        call ftukyj (lunout,'REFYDMIN',iymin,com,status)
        call ftgkyj (lunout,'REFYDMAX',idum,com,status)
        call ftukyj (lunout,'REFYDMAX',iymax,com,status)
        if(status .ne. 0) then
            write(errstr,*) "30: STATUS=",status, &
                    " on return from ftgky* or ftuky*"
            call message(AppMsg,Always)
            call error ("badKeyUpdate","Failure on updating keywords")
        endif

!  Write out the updated event file

        call ftclos (lunin,status)
        if(status .ne. 0) then
            write(errstr,*) "31: STATUS=",status, &
                     " on return from ftclos"
            call message(AppMsg,Always)
            call error ("noClose","Cannot close image file")
        endif
        call ftclos (lunout,status)
        if(status .ne. 0) then
            write(errstr,*) "32: STATUS=",status, &
                     " on return from ftclos"
            call message(AppMsg,Always)
            call error ("noClose","Cannot close image file")
        endif

        return

  end subroutine esprep_mosaic

end module esprep_mosaic_mod
