!
! Module: esprep_mosaic_time
!
! SAS Version 12.0
! Version 0.1 17 October 2012
!
! Authors:  Steve Snowden - NASA/GSFC
!          
! Description: esprep_mosaic_time sorts through the attitude file
!   to find times of stable pointing periods (subexposures of the
!   mosaic-mode observations, and then writes out the ra, dec, and
!   position angle of the subpointing for use as input to esmosaic_prep.
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
        module esprep_mosaic_time_mod

        use dal
        use param 
        use errorHandling
        use marq_mod
        use f90_unix_proc

        implicit none 
  
        private 

        public esprep_mosaic_time

        contains 
  
!---------------------------------------------------------------------
        subroutine esprep_mosaic_time
!---------------------------------------------------------------------

        character(len=8) :: tform(22), ttype(22), tunit(22)
        character(len=11) :: sdec, sra
        character(len=15) :: teste
        character(len=17) :: tests
        character(len=19) :: tend, tstart, valtime
        character(len=20) :: extname
        character(len=80) :: dummy
        character(len=1024) :: atsfile, sasfile
        logical anyf

        integer ::  block, hdutype, &
            i, icol, id, ide, idec, idecd, idecm, ids, &
            iflag, ih, ihe, ihs, imie, imis, imi, &
            imo, imoe, imos, ios, ipa, ira, iram, irah, ivd, ivt, &
            iy, iye, iys, nt, &
            lunats, lunout1, lunout2, lunsas, n, nchan, &
            ncols, ssi, status, tfields, varidat
!
        real, allocatable, dimension(:) :: pa, valdur
        real :: dec1(100), declook(100), decs, &
            palook(100), pa1(100), ra1(100), ralook(100), ras, &
            s, se, ss, t, ti(100)

        double precision, allocatable, dimension(:) :: dayarr, dec, ra
        double precision :: day, daye, days, dect(100), &
            pat(100), rat(100)

!  Get the input file names, these are the *ATS.FIT and *SUM.SAS files

        atsfile = stringParameter("atsfile")
        sasfile = stringParameter("sasfile") 
        write(errstr,*) 'ESPREP_MOSAIC_TIME: Retrieving parameters '
        call message(AppMsg,Noisy)

!  Get unused luns for the SAS, ATS, and output files

        status = 0
        call ftgiou(lunsas,status)
        call ftgiou(lunats,status)
        call ftgiou(lunout1,status)
        call ftgiou(lunout2,status)
        if(status .ne. 0) then
            write(errstr,*) "01: STATUS=", status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

!  Get the start and stop date/times of the observation

        open(lunsas,form='formatted',status='old',file=trim(sasfile))
        ios = 0
        do while (ios .eq. 0)
            read(lunsas,1000) dummy
 1000       format(a80)
            read(dummy,1010) tests
 1010       format(26x,a17)
            if(tests .eq. 'Actual Start Time') then
                read(dummy,1020) tstart
 1020           format(a19)
            endif
            read(dummy,1030) teste
 1030       format(26x,a15)
            if(teste .eq. 'Actual End Time') then
                read(dummy,1020) tend
                ios = 1
            endif
        enddo

!  Convert the start and stop times to days

        read(tstart,3000) iys, imos, ids, ihs, imis, ss
 3000   format(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f5.2)
        read(tend,3000) iye, imoe, ide, ihe, imie, se

        call timeday(iys,imos,ids,ihs,imis,ss,days)
        call timeday(iye,imoe,ide,ihe,imie,se,daye)
        print *, 'Start Day: ', days
        print *, 'End Day: ', daye
        close(lunsas)

!  Open the *ATS.FIT FITS file

        call ftopen (lunats,trim(atsfile),0,block,status)
        if(status .ne. 0) then
            write(errstr,*) "02: STATUS=",status, &
                    " on return from ftopen"
            call message(AppMsg,Always)
            call error ("noOpen","Cannot open image file")
        endif

!  Move to the correct extension

        call ftmrhd (lunats,1,hdutype,status)
        if(status .ne. 0) then
            write(errstr,*) "03: STATUS=",status, &
                    " on return from ftmrhd"
            call message(AppMsg,Always)
            call error ("noMove","Cannot move to new extension")
        endif

!  Read the FITS file header information

        ncols = 22
        call ftghbn (lunats,ncols,nchan,tfields,ttype,tform, &
                    tunit,extname,varidat,status)
        if(status .ne.  0) then
            write(errstr,*) "04: STATUS=",status, &
                    " on return from ftghbn"
            call message(AppMsg,Always)
            call error ("noReadHeader","Unable to read header info")
        endif
        if(extname .ne. 'SCATS1') then
            write(errstr,*) "05: Wrong extension, should be SCATS1"
            call message(AppMsg,Always)
            call error ("wrongExtension","Wrong extension")
        endif

!  Dimension the arrays

        allocate(valdur(nchan))
        allocate(ra(nchan))
        allocate(dec(nchan))
        allocate(pa(nchan))
        allocate(dayarr(nchan))

!  Extract the column numbers

        ivt = 0
        ivd = 0
        ira = 0
        idec = 0
        ipa = 0
        do icol=1,ncols
            if(ttype(icol) .eq. 'VALTIME') ivt = icol
            if(ttype(icol) .eq. 'VALDUR') ivd = icol
            if(ttype(icol) .eq. 'VIEWRA') ira = icol
            if(ttype(icol) .eq. 'VIEWDECL') idec = icol
            if(ttype(icol) .eq. 'ASTPOS') ipa = icol
        enddo

!  Cycle through the entries

        n = 0
        do i=1,nchan
            if(ivt .ne. 0) then

!  Get the day of the step

                call ftgcvs(lunats,ivt,i,1,1,' ',valtime,anyf,status)
                read(valtime,3005) iy, imo, id, ih, imi, ssi
 3005           format(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,i2)
                s = ssi
                call timeday(iy,imo,id,ih,imi,s,day)
                if((day .ge. days) .and. (day .le. daye)) then
                    n = n + 1

!  Get the day, look direction, and duration of the step

                    dayarr(n) = day
                    call ftgcve(lunats,ivd,i,1,1,0, &
                            valdur(n),anyf,status)
                    call ftgcvs(lunats,ira,i,1,1,' ', &
                            sra,anyf,status)
                    call ftgcvs(lunats,idec,i,1,1,' ', &
                            sdec,anyf,status)
                    call ftgcve(lunats,ipa,i,1,1,0, &
                            pa(n),anyf,status)
                    if(status .ne.  0) then
                        write(errstr,*) "06: STATUS=",status, &
                                " on return from ftghbn"
                        call message(AppMsg,Always)
                        call error ("noReadHeader",&
                                "Unable to read header info")
                    endif
                    write(errstr,3008) valdur(n), sra, sdec
 3008               format("Dur: ",f7.0,"  RA: ",a11,"  Dec: ",a11)
                     call message(AppMsg,Noisy)

!  Convert the look direction to decimal degrees

                    read(sra,3010) irah, iram, ras
 3010               format(i2,1x,i2,1x,f5.2)
                    ra(n) = 15.0*(irah + iram/60.0 + ras/3600.0)
                    read(sdec,3020) idecd, idecm, decs
 3020               format(i3,1x,i2,1x,f4.1)
                    t = idecm/60.0 + decs/3600.0
                    if(idecd .lt. 0) dec(n) = idecd - t
                    if(idecd .ge. 0) dec(n) = idecd + t
                endif
            endif
        enddo

!  Now sort through the steps and group the data in what
!  may be individual obs.

        iflag = 0
        nt = 0
        do i=1,n
            if(valdur(i) .gt. 0.5) then
                if(iflag .eq. 0) then
                    nt = nt + 1
                    ti(nt) = valdur(i)
                    rat(nt) = ra(i)*valdur(i)
                    dect(nt) = dec(i)*valdur(i)
                    pat(nt) = pa(i)*valdur(i)
                    iflag = 1
                    ra1(nt) = ra(i)
                    dec1(nt) = dec(i)
                    pa1(nt) = pa(i)
                else
                    ti(nt) = ti(nt) + valdur(i)
                    rat(nt) = rat(nt) + ra(i)*valdur(i)
                    dect(nt) = dect(nt) + dec(i)*valdur(i)
                    pat(nt) = pat(nt) + pa(i)*valdur(i)
                endif
            else
                iflag = 0
            endif
        enddo

        close(lunsas)

!  Write out the look directions and position angles for the
!  subpointing.  mosaic-times.txt contains the weighted average
!  while mosaic-times1.txt has the first direction of the set.

        open(lunout1,form='formatted',status='unknown', &
                file='mosaic-times.txt')
        do i=1,nt
            ralook(i) = rat(i)/ti(i)
            declook(i) = dect(i)/ti(i)
            palook(i) = pat(i)/ti(i)
            write(errstr,3030) i, ralook(i), declook(i), &
                    palook(i), ti(i)
 3030       format(i2," RA:",f7.3," Dec:",f7.3, &
                    " PA:",f7.3," Exp:",f7.0)
            call message(AppMsg,Noisy)
            write(lunout1,2000) i, ralook(i), declook(i), palook(i)
 2000       format(I4,3f12.7)
        enddo

        close(lunout1)

        end subroutine esprep_mosaic_time
 
!--------------------------------------------------
!  Subroutines
!--------------------------------------------------

        subroutine timeday(iy,imo,id,ih,imi,s,day)

        implicit none

        integer :: iy, imo, id, ih, imi
        real :: s
        double precision :: day

        day = (iy-2000)*365.0 + 1.0
        if(iy .gt. 2004) day = day + 1
        if(iy .gt. 2008) day = day + 1
        if(iy .gt. 2012) day = day + 1
        if(iy .gt. 2016) day = day + 1
        if(iy .gt. 2020) day = day + 1
        if(imo .eq. 1) then
            day = day + id - 1
        elseif(imo .eq. 2) then
            day = 31 + id - 1
        elseif(imo .eq. 3) then
            day = 59 + id - 1
        elseif(imo .eq. 4) then
            day = 90 + id - 1
        elseif(imo .eq. 5) then
            day = 120 + id - 1
        elseif(imo .eq. 6) then
            day = 151 + id - 1
        elseif(imo .eq. 7) then
            day = 181 + id - 1
        elseif(imo .eq. 8) then
            day = 212 + id - 1
        elseif(imo .eq. 9) then
            day = 243 + id - 1
        elseif(imo .eq. 10) then
            day = 273 + id - 1
        elseif(imo .eq. 11) then
            day = 304 + id - 1
        elseif(imo .eq. 12) then
            day = 334 + id - 1
        endif

        day = day + (ih + imi/60.0 + s/3600.0)/24.0

        return

        end subroutine timeday

        end module esprep_mosaic_time_mod
