!+
! Module: merge_comp_xmm_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
! SLS 7 May 2009
!
! Comments:
!
! $Id: merge_comp_xmm_mod.f90,v 1.4 2009/05/26 12:53:39 aibarra Exp $
!-
!  SAS Version 13.0
!  Version 5.0  2 November 2012
!
!  Merges various observations into one large mosaic
!
!  MERGE-COMP-XMM mosaics count, modeled background count, soft proton
!  count, and exposure images from multiple pointings into one image.  
!  The output projection is the zenith equal-area azimuthal, which can
!  be approximated for display purposes by using tangential-projection
!  software
!
!  INPUT:
!    xmmdir: Directory containing the ESAS calibration files
!    dirfile: Filename of directory list
!    coord: Coordinate Control:
!       1 => ecliptic
!       2 => equatorial
!       3 => galactic
!    crvaln: Central Coordinates, longitude and latitude
!    scale: Pixel Size (arc minutes)
!    compon: Component Selection
!       1 => count image
!       2 => exposure
!       3 => QPB counts
!       4 => SP counts
!       5 => SWCX counts
!       6 => Cheese masks from cheese or cheese-bands
!       7 => Cheese masks from make_mask_merge
!    iel: Lower energy limit for the energy band in eV
!    ieh: Upper energy limit for the energy band in eV
!    maskcon: Mask control
!       0 =>  No Masking
!       1 =>  Point Source Masking - Use either output from cheese or 
!             the combined band output from cheese-bands
!       11 => Point Source Masking - Use the soft band output from 
!             cheese-bands
!       12 => Point Source Masking - Use the hard band output from 
!             cheese-bands
!       2 =>  Good Area Masking
!       3 =>  Point Source Masking - Use the full band output from 
!             make-mask-merge
!       31 => Point Source Masking - Use the soft band output from 
!             make-mask-merge
!       32 => Point Source Masking - Use the hard band output from 
!             make-mask-merge
!     alpha: Spectral Scale Factor - Assumed spectrum for scaling 
!       exposure images.  1.0 < alpha < 2.4
!
!  Example: merge_comp_xmm caldb=/CalDB dirfile=dir.dat coord=2 
!                 crvaln1=194.6 crvaln2=27.7 pixelsize=0.0666667  
!                 component=1 elow=400 ehigh=750 maskcontrol=0
!                 alpha=1.7
!
!  OUTPUT:
!    Mosaicked image
!

module merge_comp_xmm_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public merge_comp_xmm

contains

  !---------------------------------------------------------------------
  subroutine merge_comp_xmm
  !---------------------------------------------------------------------

        character(len=1), dimension(30) :: compi, compio, maski, outfi
        character(len=1), dimension(132) :: dirstr, infi, infio
        character(len=4) :: filt, ctypn, ctypo
        character(len=8) :: filter, units
        character(len=8), dimension(2) :: ctype, ctypen, ctypeo
        character(len=25) :: submode
        character(len=30) :: comp, compo
        character(len=30) :: maskna = '-cheese.fits'
        character(len=60) :: com, comment
        character(len=80) :: dir
        character(len=132) :: dirfile, xmmdir
        character(len=132) :: outfile = '    '
        character(len=132) :: infile = '    '
        character(len=132) :: infileo = '    '
        integer :: bitpix, block, coord, compon, gcount, i, iadd, &
            idet, ieh, iel, ierr, iflag, ii, ini4(900,900), &
            ios, ipixcrd(2), islow, ishigh, &
            itot, lun, luno, maskcon, n, naxes(10), naxis, pcount, &
            xdim, ydim
        integer, allocatable, dimension(:,:) :: ini4l
        integer :: err = 0
        integer :: imed = 0
        integer :: ipnobj = 0
        integer :: ithick = 0
        integer :: ithin = 0
        integer :: istat = 0
        integer :: status = 0
        real :: alpha, bscale, bzero, fscal, input(900,900), &
            inputo(900,900), mask(900,900), ootsca, scale
        real, allocatable, dimension(:,:) :: merg
        double precision, dimension(2) :: cdelt, cdeltn, cdelto, &
            crpix, crpixn, crpixo, crval, crvaln, crvalo, pixcrd
        double precision :: CROTN, CROTO, CDEC, CRA, dbin, dbout, & 
            ddeccen, delang, djul, dlin, dlout, dracen, dpdec, dpra, &
            FIELD, XPIXIN, YPIXIN
        double precision, dimension(19) :: linn
        double precision :: drad = 3.141592654D0
        logical :: anyf, clobber, extend, nomask, simple
        eqUIVALENCE (COMP,COMPI)
        eqUIVALENCE (COMPO,COMPIO)
        eqUIVALENCE (DIR,DIRSTR)
        eqUIVALENCE (FILT,FILTER)
        eqUIVALENCE (infile,INFI)
        eqUIVALENCE (infileO,INFIO)
        eqUIVALENCE (MASKNA,MASKI)
        eqUIVALENCE (OUTFILE,OUTFI)

        xmmdir = stringParameter("caldb")
        dirfile = stringParameter("dirfile")
        coord = intParameter("coord")
        crvaln(1) = realParameter("crvaln1")
        crvaln(2) = realParameter("crvaln2")
        scale = realParameter("pixelsize")
        compon = intParameter("component")
        iel = intParameter("elow")
        ieh = intParameter("ehigh")
        xdim = intParameter("xdim")
        ydim = intParameter("ydim")
        maskcon = intParameter("maskcontrol")
        alpha = realParameter("alpha")
        clobber = booleanParameter("clobber")

        allocate(merg(xdim,ydim))
        allocate(ini4l(xdim,ydim))

        drad = drad/180.D0

!  Get unused luns

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_COMP_XMM POSITION 01'
            print *, 'STATUS FTGIOU =', status
            stop
        endif

        call ftgiou(luno,status)
        if(status .ne. 0) then
            print *, 'MERGE-COMP-XMM POSITION 02'
            print *, 'STATUS FTGIOU =', status
        endif

!  List the program control parameters.

        print *, 'Assumed Spectral Index: ', alpha
        print *, 'Directory File: ', dirfile
        print *, 'Coord System', coord
        if(coord .eq. 1) then
            ctypen(1) = 'ELON-TAN'
            ctypen(2) = 'ELAT-TAN'
        elseif(coord .eq. 2) then
            ctypen(1) = 'RA---TAN'
            ctypen(2) = 'DEC--TAN'
        elseif(coord .eq. 3) then
            ctypen(1) = 'GLON-TAN'
            ctypen(2) = 'GLAT-TAN'
        endif

        ctypn = '-TAN'
        print *, 'Central Coords: ', crvaln(1), crvaln(2)
        djul = 2451544.5D0
        print *, 'Energy Range: ', iel, ieh
        print *, 'Output Array Size: ', xdim, ydim
        print *, 'Pixel Sixe: ', scale
        print *, 'Clobber: ', clobber

        if(coord .eq. 1) then
            dlin = drad*crvaln(1)
            dbin = drad*crvaln(2)
            call equecl(djul,.FALSE.,.FALSE., &
                    dlin,dbin,dlout,dbout,istat)
            if(istat .NE. 0) then
                print*, 'MERGE_COMP_XMM POSITION 07'
                print*,'MERGE_COMP_XMM: Error on return ', &
                        'from eqUECL'
                print*,'istat =', istat
                ierr = 3
            else
                dracen = dlout/drad
                ddeccen = dbout/drad
            endif

!  Leave in equatorial coordinates

        elseif(coord .eq. 2) then
            dracen = crvaln(1)
            ddeccen = crvaln(2)

!  Transform from galactic to equatorial coordinates

        elseif(coord .eq. 3) then
            dlin = drad*crvaln(1)
            dbin = drad*crvaln(2)
            call equgal(djul,.FALSE.,dlin,dbin,dlout,dbout,istat)
            if(istat .NE. 0) then
                print*,'MERGE_COMP_XMM POSITION 08'
                print*,'MERGE_COMP_XMM: Error on return ', &
                        'from eqUGAL'
                ierr = 3
            else
                dracen = dlout/drad
                ddeccen = dbout/drad
            endif
        endif
        dracen = drad*dracen
        ddeccen = drad*ddeccen

! Create output file name

        if(compon .eq. 1) then
            if(IEL .GE. 1000) then
                if(IEH .GE. 10000) then
                    WRITE(COMP,3105) IEL, IEH
 3105               FORMAT('obj-im-',I4,'-',I5,'.fits')
                else
                    WRITE(COMP,3115) IEL, IEH
 3115               FORMAT('obj-im-',I4,'-',I4,'.fits')
                endif
            elseif(IEH .LT. 1000) then
                WRITE(COMP,3125) IEL, IEH
 3125           FORMAT('obj-im-',I3,'-',I3,'.fits')
            elseif(IEH .GE. 10000) then
                WRITE(COMP,3135) IEL, IEH
 3135           FORMAT('obj-im-',I3,'-',I5,'.fits')
            else
                WRITE(COMP,3145) IEL, IEH
 3145           FORMAT('obj-im-',I3,'-',I4,'.fits')
            endif
        elseif(compon .eq. 2) then
            if(IEL .GE. 1000) then
                if(IEH .GE. 10000) then
                    WRITE(COMP,3205) IEL, IEH
 3205               FORMAT('exp-im-',I4,'-',I5,'.fits')
                else
                    WRITE(COMP,3215) IEL, IEH
 3215               FORMAT('exp-im-',I4,'-',I4,'.fits')
                endif
            elseif(IEH .LT. 1000) then
                WRITE(COMP,3225) IEL, IEH
 3225           FORMAT('exp-im-',I3,'-',I3,'.fits')
            elseif(IEH .GE. 10000) then
                WRITE(COMP,3235) IEL, IEH
 3235           FORMAT('exp-im-',I3,'-',I5,'.fits')
            else
                WRITE(COMP,3245) IEL, IEH
 3245           FORMAT('exp-im-',I3,'-',I4,'.fits')
            endif
        elseif(compon .eq. 3) then
            if(IEL .GE. 1000) then
                if(IEH .GE. 10000) then
                    WRITE(COMP,3305) IEL, IEH
 3305               FORMAT('back-im-',I4,'-',I5,'.fits')
                else
                    WRITE(COMP,3315) IEL, IEH
 3315               FORMAT('back-im-',I4,'-',I4,'.fits')
                endif
            elseif(IEH .LT. 1000) then
                WRITE(COMP,3325) IEL, IEH
 3325           FORMAT('back-im-',I3,'-',I3,'.fits')
           elseif(IEH .GE. 10000) then
                WRITE(COMP,3335) IEL, IEH
 3335           FORMAT('back-im-',I3,'-',I5,'.fits')
            else
                WRITE(COMP,3345) IEL, IEH
 3345           FORMAT('back-im-',I3,'-',I4,'.fits')
            endif
        elseif(compon .eq. 4) then
            if(IEL .GE. 1000) then
                if(IEH .GE. 10000) then
                    WRITE(COMP,3405) IEL, IEH
 3405               FORMAT('prot-im-',I4,'-',I5,'.fits')
                else
                    WRITE(COMP,3415) IEL, IEH
 3415               FORMAT('prot-im-',I4,'-',I4,'.fits')
                endif
            elseif(IEH .LT. 1000) then
                WRITE(COMP,3425) IEL, IEH
 3425           FORMAT('prot-im-',I3,'-',I3,'.fits')
            elseif(IEH .GE. 10000) then
                WRITE(COMP,3435) IEL, IEH
 3435           FORMAT('prot-im-',I3,'-',I5,'.fits')
            else
                WRITE(COMP,3445) IEL, IEH
 3445           FORMAT('prot-im-',I3,'-',I4,'.fits')
            endif
        elseif(compon .eq. 5) then
            if(IEL .GE. 1000) then
                if(IEH .GE. 10000) then
                    WRITE(COMP,3505) IEL, IEH
 3505               FORMAT('swcx-im-',I4,'-',I5,'.fits')
                else
                    WRITE(COMP,3515) IEL, IEH
 3515               FORMAT('swcx-im-',I4,'-',I4,'.fits')
                endif
            elseif(IEH .LT. 1000) then
                WRITE(COMP,3525) IEL, IEH
 3525           FORMAT('swcx-im-',I3,'-',I3,'.fits')
            elseif(IEH .GE. 10000) then
                WRITE(COMP,3535) IEL, IEH
 3535           FORMAT('swcx-im-',I3,'-',I5,'.fits')
            else
                WRITE(COMP,3545) IEL, IEH
 3545           FORMAT('swcx-im-',I3,'-',I4,'.fits')
            endif
        elseif(compon .eq. 6) then
            if(IEL .GE. 1500) then
                COMP = 'cheese-im-h.fits'
            elseif(IEH .LT. 1500) then
                COMP = 'cheese-im-s.fits'
            else
                COMP = 'cheese-im.fits'
            endif
        elseif(compon .eq. 7) then
            if(IEL .GE. 1500) then
                COMP = 'msl-cheese-im-h.fits'
            elseif(IEH .LT. 1500) then
                COMP = 'msl-cheese-im-s.fits'
            else
                COMP = 'msl-cheese-im.fits'
            endif 
        endif
        OUTFILE = COMP
        print *, 'OUTFILE: ', outfile

!  Set the maximum separation for a pointing to still 
!  possibly be in the image

        FIELD = 0.0174533D0*(SCALE* &
            sqrt(1.0*(xdim*xdim+ydim*ydim))/2.0 + 15.0)/60.0

        LINN(1) = 0.0D0
        LINN(2) = 2.0D0
        LINN(3) = 2.0D0
        CRPIXN(1) = xdim/2.0 + 0.5
        CRPIXN(2) = ydim/2.0 + 0.5
        CDELTN(1) = -SCALE/60.0
        CDELTN(2) = SCALE/60.0
        CROTN = 0.0
        print *, 'Merged image coordinate system: ', CTYPN
        print *, 'Merged image coordinate ID: ', &
                        ctypen(1), '  ', ctypen(2)
        print *, 'Merged image central coordinates: ', &
                        crvaln(1), CRVALN(2)
        print *, 'Merged image reference pixel: ', &
                        CRPIXN(1), CRPIXN(2)
        print *, 'Merged image pixel sizes: ', &
                        CDELTN(1), CDELTN(2)
        print *, 'Merged image rotation angle: ', CROTN
!        print *, 'Merged image NAXES, NAXIS: ', NAXIS, &
!                    (NAXES(I),I=1,NAXIS)

        print *, 'Cast Component = ', compon
        print *, 'Energy Range = ', IEL, IEH

!  Set the band

        islow = (iel + 25)/50
        ishigh = (ieh + 25)/50

        print *, 'Masking Control: ', maskcon

        IADD = 0
        OPEN(UNIT=77,STATUS='OLD',FORM='FORMATTED',FILE=dirfile, &
                iostat=ios)
        IOS = 0
        DO WHILE (IOS .eq. 0)
            READ(77,1020,IOSTAT=IOS) DIR
 1020       format(a80)
            if(IOS .eq. 0) then
                IPNOBJ = 0
                print *, ' '
                print *, 'ADD NEW ARRAY'
                print *, DIR

!  Set the input file name

                DO I=1,110
                    INFI(I) = ' '
                    INFIO(I) = ' '
                enddo
                I = 1

                DO WHILE (DIRSTR(I) .NE. ' ')
                    INFI(I) = DIRSTR(I)
                    INFIO(I) = DIRSTR(I)
                    I = I + 1
                enddo
                N = I - 1
                if(DIRSTR(N-5) .eq. 's') then
                    READ(DIRSTR(N-4),3900) IDET
 3900               FORMAT(I1)
                elseif(DIRSTR(N-5) .eq. 'p') then
                    IDET = 3
                endif

                if(compon .eq. 1) then
                    if(IEL .GE. 1000) then
                        if(IEH .GE. 10000) then
                            WRITE(COMP,3100) IEL, IEH
 3100                       FORMAT('-obj-im-',I4,'-',I5,'.fits')
                        else
                            WRITE(COMP,3110) IEL, IEH
 3110                       FORMAT('-obj-im-',I4,'-',I4,'.fits')
                        endif
                    elseif(IEH .LT. 1000) then
                        WRITE(COMP,3120) IEL, IEH
 3120                   FORMAT('-obj-im-',I3,'-',I3,'.fits')
                    elseif(IEH .GE. 10000) then
                        WRITE(COMP,3130) IEL, IEH
 3130                   FORMAT('-obj-im-',I3,'-',I5,'.fits')
                    else
                        WRITE(COMP,3140) IEL, IEH
 3140                   FORMAT('-obj-im-',I3,'-',I4,'.fits')
                    endif
                    if(IDET .eq. 3) then
                        IPNOBJ = 1
                        if(IEL .GE. 1000) then
                            if(IEH .GE. 10000) then
                                WRITE(COMPO,3102) IEL, IEH
 3102                           FORMAT('-obj-im-',I4,'-',I5,'-oot.fits')
                            else
                                WRITE(COMPO,3112) IEL, IEH
 3112                           FORMAT('-obj-im-',I4,'-',I4,'-oot.fits')
                            endif
                        elseif(IEH .LT. 1000) then
                            WRITE(COMPO,3122) IEL, IEH
 3122                       FORMAT('-obj-im-',I3,'-',I3,'-oot.fits')
                        elseif(IEH .GE. 10000) then
                            WRITE(COMPO,3132) IEL, IEH
 3132                       FORMAT('-obj-im-',I3,'-',I5,'-oot.fits')
                        else
                            WRITE(COMPO,3142) IEL, IEH
 3142                       FORMAT('-obj-im-',I3,'-',I4,'-oot.fits')
                        endif
                    endif
                elseif(compon .eq. 2) then
                    if(IEL .GE. 1000) then
                        if(IEH .GE. 10000) then
                            WRITE(COMP,3200) IEL, IEH
 3200                       FORMAT('-exp-im-',I4,'-',I5,'.fits')
                        else
                            WRITE(COMP,3210) IEL, IEH
 3210                       FORMAT('-exp-im-',I4,'-',I4,'.fits')
                        endif
                    elseif(IEH .LT. 1000) then
                        WRITE(COMP,3220) IEL, IEH
 3220                   FORMAT('-exp-im-',I3,'-',I3,'.fits')
                    elseif(IEH .GE. 10000) then
                        WRITE(COMP,3230) IEL, IEH
 3230                   FORMAT('-exp-im-',I3,'-',I5,'.fits')
                    else
                        WRITE(COMP,3240) IEL, IEH
 3240                   FORMAT('-exp-im-',I3,'-',I4,'.fits')
                    endif
                elseif(compon .eq. 3) then
                    if(IEL .GE. 1000) then
                        if(IEH .GE. 10000) then
                            WRITE(COMP,3300) IEL, IEH
 3300                       FORMAT('-back-im-sky-',I4,'-',I5,'.fits')
                        else
                            WRITE(COMP,3310) IEL, IEH
 3310                       FORMAT('-back-im-sky-',I4,'-',I4,'.fits')
                        endif
                    elseif(IEH .LT. 1000) then
                        WRITE(COMP,3320) IEL, IEH
 3320                   FORMAT('-back-im-sky-',I3,'-',I3,'.fits')
                    elseif(IEH .GE. 10000) then
                        WRITE(COMP,3330) IEL, IEH
 3330                   FORMAT('-back-im-sky-',I3,'-',I5,'.fits')
                    else
                        WRITE(COMP,3340) IEL, IEH
 3340                   FORMAT('-back-im-sky-',I3,'-',I4,'.fits')
                    endif
                elseif(compon .eq. 4) then
                    if(IEL .GE. 1000) then
                        if(IEH .GE. 10000) then
                            WRITE(COMP,3400) IEL, IEH
 3400                       FORMAT('-prot-im-sky-',I4,'-',I5,'.fits')
                        else
                            WRITE(COMP,3410) IEL, IEH
 3410                       FORMAT('-prot-im-sky-',I4,'-',I4,'.fits')
                        endif
                    elseif(IEH .LT. 1000) then
                        WRITE(COMP,3420) IEL, IEH
 3420                   FORMAT('-prot-im-sky-',I3,'-',I3,'.fits')
                    elseif(IEH .GE. 10000) then
                        WRITE(COMP,3430) IEL, IEH
 3430                   FORMAT('-prot-im-sky-',I3,'-',I5,'.fits')
                    else
                        WRITE(COMP,3440) IEL, IEH
 3440                   FORMAT('-prot-im-sky-',I3,'-',I4,'.fits')
                    endif
                elseif(compon .eq. 5) then
                    if(IEL .GE. 1000) then
                        if(IEH .GE. 10000) then
                            WRITE(COMP,3500) IEL, IEH
 3500                       FORMAT('-swcx-im-sky-',I4,'-',I5,'.fits')
                        else
                            WRITE(COMP,3510) IEL, IEH
 3510                       FORMAT('-swcx-im-sky-',I4,'-',I4,'.fits')
                        endif
                    elseif(IEH .LT. 1000) then
                        WRITE(COMP,3520) IEL, IEH
 3520                   FORMAT('-swcx-im-sky-',I3,'-',I3,'.fits')
                    elseif(IEH .GE. 10000) then
                        WRITE(COMP,3530) IEL, IEH
 3530                   FORMAT('-swcx-im-sky-',I3,'-',I5,'.fits')
                    else
                        WRITE(COMP,3540) IEL, IEH
 3540                   FORMAT('-swcx-im-sky-',I3,'-',I4,'.fits')
                    endif
                elseif(compon .eq. 6) then
                    if(IEL .GE. 1500) then
                        COMP = '-cheese-h.fits'
                    elseif(IEH .LT. 1500) then
                        COMP = '-cheese-s.fits'
                    else
                        COMP = '-cheese.fits'
                    endif
                elseif(compon .eq. 7) then
                    if(IEL .GE. 1500) then
                        COMP = '-msl-cheese-h.fits'
                    elseif(IEH .LT. 1500) then
                        COMP = '-msl-cheese-s.fits'
                    else
                        COMP = '-msl-cheese.fits'
                    endif 
                endif

                DO I=1,30
                    INFI(N+I) = COMPI(I)
                enddo
                print *, infile
                if(IPNOBJ .eq. 1) then
                    DO I=1,30
                        INFIO(N+I) = COMPIO(I)
                    enddo
                    print *, infileO
                endif

!  Open the FITS file for the component

                call FTOPEN(LUN,infile,0,BLOCK,STATUS)
                if((STATUS .NE. 0) .AND. (compon .eq. 4)) then
                    status = 0
                    print *, 'Missing SP image'
                    print *, 'Am assuming that you know what you ', &
                            'are doing and only'
                    print *, 'have SP images for some of the ', &
                            'observations that you are combining'
                elseif((STATUS .NE. 0) .AND. (compon .eq. 5)) then
                    status = 0
                    print *, 'Missing SWCX image'
                    print *, 'Am assuming that you know what you ', &
                            'are doing and only'
                    print *, 'have SWCX images for some of the ', &
                            'bands/observations you are combining'
                elseif((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                    ERR = 1
                    print *, 'MERGE-COMP-XMM POSITION 09'
                    print *, 'STATUS FTOPEN =', STATUS
                    print *, infile
                else
                    print *, 'Input image: ', infile

!  Read the FITS file header information

                    call FTGHPR (LUN,10,SIMPLE,BITPIX,NAXIS,NAXES, &
     +                          PCOUNT,GCOUNT,EXTEND,STATUS)
                    if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                        ERR = 1
                        print *, 'MERGE-COMP-XMM POSITION 10'
                        print *, 'STATUS FTGHPR =', STATUS
                        print *, infile
                    endif

!  Get the useful keywords

                    call FTGICS (LUN,CRVALO(1),CRVALO(2), &
                            CRPIXO(1),CRPIXO(2), &
                            CDELTO(1),CDELTO(2),CROTO,CTYPO,STATUS)
                    if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                        ERR = 1
                        print *, 'MERGE-COMP-XMM POSITION 11'
                        print *, 'STATUS FTGICS =', STATUS
                        print *, infile
                    endif

!  And a few more

                    call FTGKYS (LUN,'CTYPE1',CTYPEO(1),COM,STATUS)
                    if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                        ERR = 1
                        print *, 'MERGE-COMP-XMM POSITION 12'
                        print *, 'STATUS FTGKYS =', STATUS
                        print *, infile
                    endif
                    call FTGKYS (LUN,'CTYPE2',CTYPEO(2),COM,STATUS)
                    if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                        ERR = 1
                        print *, 'MERGE-COMP-XMM POSITION 13'
                        print *, 'STATUS FTGKYS =', STATUS
                        print *, infile
                    endif

!  Test to see how far the pointing is from the center of the mosaic

                    dpra = drad*CRVALO(1)
                    dpdec = drad*CRVALO(2)

                    call ANGSEP(dracen,ddeccen,dpra,dpdec,delang,ierr)
!                    print *, dracen,ddeccen,dpra,dpdec,delang,ierr
!                    stop
                    if(delang .LT. FIELD) then

!  Set the scale factor for the filter

                        fscal = 1.0
                        if(compon .eq. 2) then
                            call FTGKYS (LUN,'FILTER',FILTER,COM,STATUS)
                            if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                                ERR = 1
                                print *, 'MERGE-COMP-XMM POSITION 14'
                                print *, 'STATUS FTGKYS =', STATUS
                                print *, infile
                            endif
                            ITOT = ITOT + 1

                            print *, 'Detector and Filter: ', idet, &
                                '  ', filt
                            call hard_scale (xmmdir,alpha,idet, &
                                filt,islow,ishigh,fscal)
                            if(filt .eq. 'Thin') then
                                ithin = ithin + 1
                            elseif(filt .eq. 'Medi') then
                                imed = imed + 1
                            elseif(filt .eq. 'Thic') then
                                ithick = ithick + 1
                            endif
                        endif
                        print *, 'Filter scale factor: ', fscal

!  Print the central coords of the observation

                        print *, 'Status: ', STATUS
                        print *, 'Original image coordinate system: ', &
                                CTYPO
                        print *, 'Original image coordinate ID: ', &
                                CTYPEO(1), '  ', CTYPEO(2)
                        print *, 'Original image central coordinates: ', &
                                CRVALO(1), CRVALO(2)
                        print *, 'Original image reference pixel: ', &
                                CRPIXO(1), CRPIXO(2)
                        print *, 'Original image pixel sizes: ', &
                                CDELTO(1), CDELTO(2)
                        print *, 'Original image rotation angle: ', &
                                CROTO
                        print *, 'Original image NAXES, NAXIS: ', &
                                NAXIS, (NAXES(I),I=1,NAXIS)

!  Read in the X-ray image

                        call FTG2DE (LUN,0,0,NAXES(1),NAXES(1),NAXES(2), &
                                INPUT,ANYF,STATUS)
                        if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                            ERR = 1
                            print *, 'MERGE-COMP-XMM POSITION 15'
                            print *, 'STATUS FTG2DE =', STATUS
                            print *, infile
                        endif

!  Close the FITS file

                        call FTCLOS(LUN,STATUS)
                        if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                            ERR = 1
                            print *, 'MERGE-COMP-XMM POSITION 16'
                            print *, 'STATUS FTCLOS =', STATUS
                            print *, infile
                        endif

!  Read in the OOT event file if pn data

                        if(IPNOBJ .eq. 1) then
                            call FTOPEN(LUNO,infileO,0,BLOCK,STATUS)
                            if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                                ERR = 1
                                print *, 'MERGE-COMP-XMM POSITION 17'
                                print *, 'STATUS FTOPEN =', STATUS
                                print *, infileO
                            endif
                            print *, 'Input image: ', infileO
 
!  Read the FITS file header information

                            call FTGHPR (LUNO,10,SIMPLE,BITPIX,NAXIS, &
                                    NAXES,PCOUNT,GCOUNT,EXTEND,STATUS)
                            if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                                ERR = 1
                                print *, 'MERGE-COMP-XMM POSITION 18'
                                print *, 'STATUS FTGHPR =', STATUS
                                print *, infileO
                            endif

!  Get the SUBMODE keyword

                            call FTGKYS (LUNO,'SUBMODE',SUBMODE,COM, &
                                    STATUS)
                            if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                                ERR = 1
                                print *, 'MERGE-COMP-XMM POSITION 19'
                                print *, 'STATUS FTGKYS =', STATUS
                                print *, infileO
                            endif
                            if(SUBMODE .eq. 'PrimeFullWindow') &
                                    OOTSCA = 0.063
                            if(SUBMODE .eq. 'PrimeFullWindowExtended') & 
                                    OOTSCA = 0.023
                            if(SUBMODE .eq. 'PrimeFullWindowExtended5') &
                                    OOTSCA = 0.013
                            print *, 'SUBMODE, OOT scale factor: ', &
                                    SUBMODE, OOTSCA

!  Read in the X-ray image

                            call FTG2DE (LUNO,0,0,NAXES(1),NAXES(1), &
                                NAXES(2),INPUTO,ANYF,STATUS)
                            if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                                ERR = 1
                                print *, 'MERGE-COMP-XMM POSITION 20'
                                print *, 'STATUS FTG2DE =', STATUS
                                print *, infileO
                            endif

!  Close the FITS file

                            call FTCLOS(LUNO,STATUS)
                            if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                                ERR = 1
                                print *, 'MERGE-COMP-XMM POSITION 21'
                                print *, 'STATUS FTCLOS =', STATUS
                                print *, infileO
                            endif

!  Combine the data

                            DO I=1,900
                                DO II=1,900
                                    INPUT(I,II) = INPUT(I,II) - &
                                             OOTSCA*INPUTO(I,II) 
                                enddo
                            enddo
                            IPNOBJ = 0
                        endif

!  Read in point source masks to go with the observation

                        if((maskcon .eq. 1) .or. (maskcon .eq. 11) &
                                .or. (maskcon .eq. 12)) then
                            if(maskcon .eq. 1) then
                                maskna = '-cheese.fits'
                            elseif(maskcon .eq. 11) then
                                maskna = '-cheese-s.fits'
                            elseif(maskcon .eq. 12) then
                                maskna = '-cheese-h.fits'
                            endif

                            do i=1,30
                                infi(n+i) = maski(i)
                            enddo
                            print *, infile
                            call retfit(infile,mask, &
                                    ini4,1,900,900,ctype,crval, &
                                    cdelt,crpix,status)
                            if((status .ne. 0) .and. (err .eq. 0)) then
                                err = 1
                                print *, 'MERGE-COMP-XMM POSITION 22'
                                print *, 'STATUS RETFIT =', status
                                print *, 'Missing a cheese mask'
                                print *, infile
                                stop
                            endif
                            nomask = .FALSE.

!  Read in good area masks

                        elseif(maskcon .eq. 2) then
                            if(iel .ge. 1000) then
                                write(maskna,3610) iel, ieh
 3610                           format('-mask-im-',i4,'-',i4,'.fits')
                            elseif(ieh .lt. 1000) then
                                write(maskna,3620) iel, ieh
 3620                           format('-mask-im-',i3,'-',i3,'.fits')
                            else
                                write(maskna,3630) iel, ieh
 3630                           format('-mask-im-',i3,'-',i4,'.fits')
                            endif
                            do i=1,30
                                infi(n+i) = maski(i)
                            enddo
                            print *, infile
                            call retfit(infile,mask, &
                                    ini4,1,900,900,ctype,crval, &
                                    cdelt,crpix,status)
                            if((status .ne. 0) .and. (err .eq. 0)) then
                                err = 1
                                print *, 'MERGE-COMP-XMM POSITION 23'
                                print *, 'STATUS RETFIT =', status
                                print *, 'Missing a normal mask'
                                print *, infile
                                stop
                            endif
                            nomask = .FALSE.

!  Read in the merged source list masks

                        elseif((maskcon .eq. 3) .or. (maskcon .eq. 31) &
                                .or. (maskcon .eq. 32)) then
                            if(maskcon .eq. 3) then
                                maskna = '-msl-cheese.fits'
                            elseif(maskcon .eq. 31) then
                                maskna = '-msl-cheese-s.fits'
                            elseif(maskcon .eq. 32) then
                                maskna = '-msl-cheese-h.fits'
                            endif
                            do i=1,30
                                infi(n+i) = maski(i)
                            enddo
                            print *, infile
                            call retfit(infile,mask, &
                                    ini4,1,900,900,ctype,crval, &
                                    cdelt,crpix,status)
                            if((status .ne. 0) .and. (err .eq. 0)) then
                                err = 1
                                print *, 'MERGE-COMP-XMM POSITION 24'
                                print *, 'STATUS RETFIT =', status
                                print *, 'Missing a merged mask'
                                print *, infile
                                stop
                            endif
                            nomask = .FALSE.
                        else


!  Make full-field masks

                            do i=1,900
                                do ii=1,900
                                    mask(i,ii) = 1.0
                                enddo
                            enddo
                            nomask = .TRUE.
                        endif

! The coords are year 2000

                        djul = 2451544.5D0

!  Cast the new observation into the mosaic

                        DO I=1,900
                            XPIXIN = I
                            DO II=1,900
                                YPIXIN = II
                                if((INPUT(I,II) .NE. 0.0) .AND. &
                                        (MASK(I,II) .GT. 0)) then
                                    call FTWLDP (XPIXIN,YPIXIN, &
                                        CRVALO(1),CRVALO(2),CRPIXO(1), &
                                        CRPIXO(2),CDELTO(1),CDELTO(2), &
                                        CROTO,CTYPO,CRA,CDEC,STATUS)
                                    if((STATUS .NE. 0) .AND. &
                                            (ERR .eq. 0)) then
                                        ERR = 1
                                        print *, &
                                          'MERGE-COMP-XMM POSITION 25'
                                        print *, 'STATUS FTWLDP =', &
                                                 STATUS
                                        print *, infile
                                    endif

!  Transform from equatorial to ecliptic coordinates

                                    if(coord .eq. 1) then
                                       dlin = drad*CRA
                                        dbin = drad*CDEC
                                        call eqUECL(djul,.FALSE., &
                                             .TRUE.,dlin,dbin,dlout, &
                                             dbout,istat)
                                        if(istat .NE. 0) then
                                            print *, &
                                                  'MERGE-COMP-XMM ', &
                                                  'POSITION 26'
                                            print *, 'istat =', istat
                                            print *, 'Error on return ', &
                                                'from eqUECL'
                                            ierr = 3
                                        else
                                            CRA = dlout/drad
                                            CDEC = dbout/drad
                                        endif

!  Leave in equatorial coordinates

                                    elseif(coord .eq. 2) then
                                        CONTINUE

!  Transform from equatorial to galactic coordinates

                                    elseif(coord .eq. 3) then
                                        dlin = drad*CRA
                                        dbin = drad*CDEC
                                        call eqUGAL(djul,.TRUE.,dlin, &
                                                dbin,dlout,dbout,istat)
                                        if(istat .NE. 0) then
                                            print *, &
                                                  'MERGE-COMP-XMM ', &
                                                  'POSITION 27'
                                            print *, 'Error on return ', &
                                                'from eqUGAL'
                                            ierr = 3
                                        else
                                            CRA = dlout/drad
                                            CDEC = dbout/drad
                                        endif
                                    endif

!  Transform into the merged system

                                    call FTXYPX (CRA,CDEC,crvaln(1), &
                                        crvaln(2),CRPIXN(1),CRPIXN(2), &
                                        CDELTN(1),CDELTN(2),CROTN,CTYPN, &
                                        PIXCRD(1),PIXCRD(2),STATUS)
                                    IPIXCRD(1) = NINT(PIXCRD(1))
                                    IPIXCRD(2) = NINT(PIXCRD(2))
                                    if((STATUS .NE. 0) .AND. &
                                            (ERR .eq. 0)) then
                                        ERR = 1
                                        print *, &
                                          'MERGE-COMP-XMM POSITION 28'
                                        print *, 'STATUS FTXYPX =', &
                                          STATUS
                                        print *, infile
                                    endif

                                    if((IPIXCRD(1) .GE. 1) .AND. &
                                            (IPIXCRD(1) .LE. xdim) .AND. &
                                            (IPIXCRD(2) .GE. 1) .AND. &
                                            (IPIXCRD(2) .LE. ydim)) then
                                        MERG(IPIXCRD(1),IPIXCRD(2)) = &
                                        MERG(IPIXCRD(1),IPIXCRD(2)) + &
                                                 fscal*INPUT(I,II)
                                    endif
                                endif
                            enddo
                        enddo
!                        stop
                    else

!  Close the FITS file

                        call FTCLOS(LUN,STATUS)
                        if((STATUS .NE. 0) .AND. (ERR .eq. 0)) then
                            ERR = 1
                            print *, 'MERGE-COMP-XMM POSITION 29'
                            print *, 'STATUS FTCLOS =', STATUS
                            print *, infile
                        endif
                        print *, 'Observation outside of cast image FOV'
                        print *, 'Offset Angle: ', delang/drad
!                        stop
                    endif
                endif
            endif
        enddo

!  Create the output file name

        print *, OUTFILE

!  Write out the merged array
!
!    compon: Component Selection
!       1 => count image
!       2 => exposure
!       3 => QPB counts
!       4 => SP counts
!       5 => SWCX counts
!       6 => Cheese masks from cheese or cheese-bands
!       7 => Cheese masks from make_mask_merge

        units = '        '
        comment = '        '
        if(compon .eq. 1) then
            units = 'counts'
            comment = 'Merged observed count image'
        elseif(compon .eq. 2) then
            units = 's'
            comment = &
         'Merged exposure map, for true scale by in/out pixel sizes'
        elseif(compon .eq. 3) then
            units = 'counts'
            comment = 'Merged model QPB counts'
        elseif(compon .eq. 4) then
            units = 'counts'
            comment = 'Merged model SP counts'
        elseif(compon .eq. 5) then
            units = 'counts'
            comment = 'Merged model SWCX counts'
        elseif(compon .eq. 6) then
            units = 'None'
            comment = 'Mask values 1=include, 0=not'
        elseif(compon .eq. 7) then
            units = 'None'
            comment = 'Mask values 1=include, 0=not'
        endif
        ifLAG = 1
        BZERO = 0.0
        BSCALE = 1.0
        if(STATUS .eq. 0) then
            call PUTFIT(OUTFILE,MERG,ini4l,ifLAG,units, &
                comment,xdim,ydim,BZERO,BSCALE,ctypen,crvaln, &
                CDELTN,CRPIXN,CLOBBER,STATUS)
        endif
 
        if(compon .eq. 2) then
            print *, 'Total Observations: ', ITHIN+IMED+ITHICK
            print *, 'Thin Filter Observations: ', ITHIN
            print *, 'Medium Filter Observations: ', IMED
            print *, 'Thick Filter Observations: ', ITHICK
        endif

        return

  end subroutine merge_comp_xmm

end module merge_comp_xmm_mod
