!+
! Module: merge_source_list_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
! SLS 4 March 2010
!
! Comments:
!
! $Id: merge_source_list_mod.f90,v 1.4 2009/05/26 12:53:39 aibarra Exp $
!-
!  Version 5.0  9 November 2012
!
!  Merges source lists from emldetect
!
!  MERGE_SOURCE_LIST creates a master list of sources detected from a list
!  of observations which have been processed to the point of a creating
!  an emldetect source list.  Sources are compared and those within 2"
!  of each other have the one with the lower flux thrown out.  It assumes
!  that the source detection has been in two bands, 0.4-1.25 keV and 
!  2.0-7.2 keV, which can be produced by cheese-bands in the default mode.
!
!  INPUT:
!    dirfile: Filename of directory list
!    maxlikelim: Lower ML limit for source detections
!
!    Example: merge_source_list dirfile=dir.dat maxlikelim=15.0
!
!  OUPUT:
!    Merged fits source list

module merge_source_list_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public merge_source_list

contains

  !---------------------------------------------------------------------
  subroutine merge_source_list
  !---------------------------------------------------------------------


        IMPLICIT NONE

        real :: dist(100000), distt(100), flux(100000), &
            fluxerr(100000), flux1(100000), fluxerr1(100000), &
            flux2(100000), fluxerr2(100000), fluxt(100), punc, &
            fluxerrt(100), rate(100000), rateerr(100000), &
            rate1(100000), rateerr1(100000), rate2(100000), &
            rateerr2(100000), ml(100000), mlt(100), &
            ratet(100), rateerrt(100), radecerr(100000), &
            radecerrt(100), scts(100000), sctst(100), &
            tnf1, tnf2, tnr1, tnr2, &
            tf1, tfe1, tf2, tfe2, tr1, tre1, tr2, tre2
        real :: decave, decmax, decmin, delra, deldec,  histmax, &
            histrange, maxlikelim, raave, ramax, ramin, rarange, scale
        double precision :: dang, dec(100000), dect(100), ra(100000), &
            rat(100), dlat1, dlat2, dlong1, dlong2
        integer :: err = 0
        integer :: ierr = 0
        integer :: status = 0
        integer :: tfieldso = 11
        integer :: block, blockl, hdutype, hist1(1000), hist2(1000), &
            i, iband, ibandt(100), icol, idec, idist, &
            iflux, ifluxerr, ii, iinst, instt(100), iml, ios, &
            ira, iradecerr, irate, irateerr, is, iscts, &
            isour, isrc, k, lundir, lunlist, lunout, m, mm, n, &
            nchan,  ncols, nrows, tfields, varidat
        character(len=1) :: dirstr(80), infi(132), eml(52)
        character(len=8) :: tform(300), ttype(300), tunit(300)
        character(len=10) :: colnam(300)
        character(len=20) :: extname
        character(len=60) :: COMM
        character(len=80) :: dir
        character(len=132) :: dirlist, dirfile, infile, outfile
        character(len=8) :: tformo(11) = (/'D', 'D', 'E', 'E', &
            'E', 'E', 'E', 'E', 'E', 'E', 'E'/)
        character(len=8) :: ttypeo(11) = (/'RA      ', 'DEC     ', &
            'RADEC_ER', 'DET_ML  ', 'FLUX_TOT', 'FL_T_ERR', 'FLUX_S  ', &
            'FL_S_ERR', 'FLUX_H  ', 'FL_H_ERR', 'DIST_NN '/)
        character(len=8) :: tunito(11) = (/'deg     ', 'deg     ', &
            'arcsec  ', '        ', 'cgs     ', 'cgs     ', 'cgs     ', &
            'cgs     ', 'cgs     ', 'cgs     ', 'arcsec  '/)
        character(len=20) :: extnameo = 'SRCLIST'
        character(len=52) :: emlist = '/emllist.fits'

        LOGICAL ANYF, clobber

        equivalence (dir,dirstr), (infile,infi), (emlist,eml)

        dirfile = stringParameter("dirfile")
        maxlikelim = realParameter("maxlikelim")
        clobber = booleanParameter("clobber")

       dirlist = dirfile

        do i=1,1000
            hist1(i) = 0
            hist2(i) = 0
        enddo

! Determin program parameters

        print *, 'Directory List: ', dirlist
        print *, 'Maximum Likelihood Threshold: ', maxlikelim

!  Get an unused lun

        call ftgiou(lundir,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_SOURCE_LIST POSITION 01'
            print *, 'STATUS FTGIOU =', status
            stop
        endif

!  Open the file with the directory list

        open(lundir,status='old',form='formatted',file=dirlist)

!  Zero the arrays

        do i=1,100000
            ra(i) = 0.0
            dec(i) = 0.0
            flux(i) = 0.0
            rate(i) = 0.0
            flux1(i) = 0.0
            rate1(i) = 0.0
            flux2(i) = 0.0
            rate2(i) = 0.0
            dist(i) = 10000.0
        enddo

!  Get an unused lun

        call ftgiou(lunlist,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_SOURCE_LIST POSITION 02'
            print *, 'STATUS FTGIOU =', status
        endif

!  Loop over the directories reading in the sources

        ios = 0
        ii = 0
        mm = 0
        do while (ios .eq. 0)
            read(lundir,1010,iostat=ios) dir
 1010       format(a80)
            if(ios .eq. 0) then

!  Make the file name

                do i=1,132
                    infi(i) = ' '
                enddo

                i = 1
                do while (dirstr(i) .ne. ' ')
                    infi(i) = dirstr(i)
                    i = i + 1
                enddo
                n = i - 1
                do i=1,52
                    infi(n+i) = eml(i)
                enddo

!  Open and read in the source list

                call ftopen(lunlist,infile,0,blockl,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'MERGE_SOURCE_LIST POSITION 03'
                    print *, 'STATUS FTOPEN =', status
                    print *, infile
                endif

!  Move to the source list extension
 
                call ftmrhd (lunlist,1,hdutype,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'MERGE_SOURCE_LIST POSITION 04'
                    print *, 'STATUS FTMRHD =', status
                    print *, 'Could not find the source list extension'
                    print *, infile
                else

!  Read the FITS file header information

                    call ftghbn (lunlist,300,nchan,tfields,ttype,tform, &
                            tunit,extname,varidat,status)
                    if((status .ne. 0) .and. (err .eq. 0)) then
                        err = 1
                        print *, 'MERGE_SOURCE_LIST POSITION 05'
                        print *, 'STATUS FTGHBN =', status
                        print *, infile
                    endif
                    if(extname .ne. 'SRCLIST') then
                        print *, extname
                        print *, 'This was supposed to be a source ', &
                                'list extension'
                        stop
                    endif

!  Read in the column names

                    call ftgkns (lunlist,'TTYPE',1,300,colnam, &
                            ncols,status)
                    if((status .ne. 0) .and. (err .eq. 0)) then
                        err = 1
                        print *, 'MERGE_SOURCE_LIST POSITION 06'
                        print *, 'STATUS FTGKNS =', status
                        print *, infile
                    endif

!  Extract the column numbers for the desired columns

                    isour = 0
                    iinst = 0
                    iband = 0
                    iml = 0
                    iflux = 0
                    ifluxerr = 0
                    irate = 0
                    irateerr = 0
                    ira = 0
                    idec = 0
                    iradecerr = 0
                    idist = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'ML_ID_SRC') isour = icol
                        if(colnam(icol) .eq. 'ID_INST') iinst = icol
                        if(colnam(icol) .eq. 'ID_BAND') iband = icol
                        if(colnam(icol) .eq. 'SCTS') iscts = icol
                        if(colnam(icol) .eq. 'DET_ML') iml = icol
                        if(colnam(icol) .eq. 'FLUX') iflux = icol
                        if(colnam(icol) .eq. 'FLUX_ERR') ifluxerr = icol
                        if(colnam(icol) .eq. 'RATE') irate = icol
                        if(colnam(icol) .eq. 'RATE_ERR') irateerr = icol
                        if(colnam(icol) .eq. 'RA') ira = icol
                        if(colnam(icol) .eq. 'DEC') idec = icol
                        if(colnam(icol) .eq. 'RADEC_ERR')  &
                                iradecerr = icol
                        if(colnam(icol) .eq. 'DIST_NN') idist = icol
                    enddo

!  Now read in the data

                    n = 1
                    isrc = 0
                    i = 1

!  Loop through all the data

                    do while (i .lt. nchan)

!  Read the source number

                        call ftgcvj(lunlist,isour,i,1,1,0,is, &
                                    anyf,status)

!  Check to see if it is a new source

                        if(is .ne. isrc) then
                            isrc = is

!  Read in the source parameters, this is for the first of the set
!  This is the total bands, total instruments entry

                            if (iinst .ne. 0)  &
                                call ftgcvj(lunlist,iinst,i,1,1,0, &
                                        instt(1),anyf,status)
                            if (iband .ne. 0)  &
                                call ftgcvj(lunlist,iband,i,1,1,0, &
                                        ibandt(1),anyf,status)
                            if (iscts .ne. 0) then
                                call ftgcve(lunlist,iscts,i,1,1,-999, &
                                        sctst(1),anyf,status)
                                if(anyf) sctst(1) = 0.0
                            endif
                            if (iml .ne. 0) then
                                call ftgcve(lunlist,iml,i,1,1,-999, &
                                        mlt(1),anyf,status)
                                if(anyf) mlt(1) = 0.0
                            endif
                            if (iflux .ne. 0) then
                                call ftgcve(lunlist,iflux,i,1,1,-999, &
                                        fluxt(1),anyf,status)
                                if(anyf) fluxt(1) = 0.0
                            endif
                            if (ifluxerr .ne. 0) then
                                call ftgcve(lunlist,ifluxerr,i,1,1,-999, &
                                        fluxerrt(1),anyf,status)
                                if(anyf) fluxerrt(1) = 0.0
                                if(fluxt(1) .eq. 0.0) fluxerrt(1) = 0.0
                            endif
                            if (irate .ne. 0) then
                                call ftgcve(lunlist,irate,i,1,1,-999, &
                                        ratet(1),anyf,status)
                                if(anyf) ratet(1) = 0.0
                            endif
                            if (irateerr .ne. 0) then
                                call ftgcve(lunlist,irateerr,i,1,1,-999, &
                                        rateerrt(1),anyf,status)
                                if(anyf) rateerrt(1) = 0.0
                                if(ratet(1) .eq. 0.0) rateerrt(1) = 0.0
                            endif
                            if (ira .ne. 0)  &
                                call ftgcvd(lunlist,ira,i,1,1,0, &
                                        rat(1),anyf,status)
                            if (idec .ne. 0)  &
                                call ftgcvd(lunlist,idec,i,1,1,0, &
                                        dect(1),anyf,status)
                            if (iradecerr .ne. 0)  &
                                call ftgcve(lunlist,iradecerr,i,1,1,0, &
                                        radecerrt(1),anyf,status)
                            if (idist .ne. 0)  &
                                call ftgcve(lunlist,idist,i,1,1,0, &
                                        distt(1),anyf,status)
                            if((status .ne. 0) .and. (err .eq. 0)) then
                                err = 1
                                print *, 'MERGE_SOURCE_LIST POSITION 07'
                                print *, 'STATUS FTGCV =', status
                            endif
                            if((instt(1) .eq. 0) .and.  &
                                    (ibandt(1) .eq. 0)) then
                                mm = mm + 1
                                ra(mm) = rat(1)
                                dec(mm) = dect(1)
                                radecerr(mm) = radecerrt(1)
                                ml(mm) = mlt(1)
                                dist(mm) = distt(1)
                                rate(mm) = ratet(1)
                                rateerr(mm) = rateerrt(1)
                                flux(mm) = fluxt(1)
                                fluxerr(mm) = fluxerrt(1)
                                scts(mm) = sctst(1)
                                if(ml(mm) .lt. maxlikelim)  &
                                        flux(mm) = 0.0

!  Loop through the rest of the source entries,
!  or to the end of the table
!  These are the values for the individual bands and instruments

                                m = 0
                                do while ((is .eq. isrc) .and. &
                                        (i .lt. nchan))
                                    i = i + 1
                                    m = m + 1

!  Read in the source parameters

                                    if(i .le. nchan) then
                                      if (isour .ne. 0)  &
                                        call ftgcvj(lunlist,isour,i,1, &
                                              1,0,is,anyf,status)
                                      if (iinst .ne. 0)  &
                                        call ftgcvj(lunlist,iinst,i,1, &
                                              1,0,instt(m),anyf,status)
                                      if (iband .ne. 0)  &
                                        call ftgcvj(lunlist,iband,i,1, &
                                              1,0,ibandt(m),anyf,status)
                                      if (iscts .ne. 0) then
                                        call ftgcve(lunlist,iscts,i,1, &
                                            1,-999,sctst(m),anyf,status)
                                         if(anyf) sctst(m) = 0.0
                                      endif
                                      if (iflux .ne. 0) then
                                        call ftgcve(lunlist,iflux,i,1, &
                                            1,-999,fluxt(m),anyf,status)
                                        if(anyf) fluxt(m) = 0.0
                                      endif
                                      if (ifluxerr .ne. 0) then
                                        call ftgcve(lunlist,ifluxerr,i, &
                                               1,1,-999,fluxerrt(m), &
                                               anyf,status)
                                        if(anyf) fluxerrt(m) = 0.0
                                        if(fluxt(m) .eq. 0.0)  &
                                               fluxerrt(m) = 0.0
                                      endif
                                      if (irate .ne. 0) then
                                        call ftgcve(lunlist,irate,i,1, &
                                            1,-999,ratet(m),anyf,status)
                                        if(anyf) ratet(m) = 0.0
                                      endif
                                      if (irateerr .ne. 0) then
                                        call ftgcve(lunlist,irate,i,1, &
                                              1,-999,rateerrt(m),anyf, &
                                              status)
                                        if(anyf) rateerrt(m) = 0.0
                                        if(ratet(m) .eq. 0.0)  &
                                              rateerrt(m) = 0.0
                                      endif
                                      if((status .ne. 0) .and.  &
                                              (err .eq. 0)) then
                                        err = 1
                                        print *, 'MERGE_SOURCE_LIST ', &
                                              'POSITION 08'
                                        print *, 'STATUS FTGCV=',status
                                      endif
                                    endif
                                enddo
                            endif

!  Step back one entry if the current entry is a new source
!  The other option is the end of the table

                            if(is .ne. isrc) m = m - 1
                            tr1 = 0.0
                            tf1 = 0.0
                            tre1 = 0.0
                            tfe1 = 0.0
                            tnr1 = 0.0
                            tnf1 = 0.0
                            tr2 = 0.0
                            tf2 = 0.0
                            tre2 = 0.0
                            tfe2 = 0.0
                            tnr2 = 0.0
                            tnf2 = 0.0

!  Loop through the source entries

                            do k=1,m

!  First band

                                if(ibandt(k) .eq. 1) then
                                    if(ratet(k) .gt. 0.0) then
                                        tr1 = tr1 + sctst(k)*ratet(k)
                                        tre1 = tre1 +  &
                                            (sctst(k)*rateerrt(k))**2
                                        tnr1 = tnr1 + sctst(k)
                                    endif
                                    if(fluxt(k) .gt. 0.0) then
                                        tf1 = tf1 + sctst(k)*fluxt(k)
                                        tfe1 = tfe1 + &
                                            (sctst(k)*fluxerrt(k))**2 
                                        tnf1 = tnf1 + sctst(k)
                                    endif

! Second band

                                elseif(ibandt(k) .eq. 2) then
                                    if(ratet(k) .gt. 0.0) then
                                        tr2 = tr2 + sctst(k)*ratet(k)
                                        tre2 = tre2 +  &
                                            (sctst(k)*rateerrt(k))**2
                                        tnr2 = tnr2 + sctst(k)
                                    endif
                                    if(fluxt(k) .gt. 0.0) then
                                        tf2 = tf2 + sctst(k)*fluxt(k)
                                        tfe2 = tfe2 +  &
                                            (sctst(k)*fluxerrt(k))**2
                                        tnf2 = tnf2 + sctst(k)
                                    endif
                                endif
                            enddo

                            if(tnr1 .gt. 0.0) tr1 = tr1/tnr1
                            if(tnr1 .gt. 0.0) tre1 = sqrt(tre1)/tnr1
                            if(tnf1 .gt. 0.0) tf1 = tf1/tnf1
                            if(tnf1 .gt. 0.0) tfe1 = sqrt(tfe1)/tnf1
                            if(tnr2 .gt. 0.0) tr2 = tr2/tnr2
                            if(tnr2 .gt. 0.0) tre2 = sqrt(tre2)/tnr2
                            if(tnf2 .gt. 0.0) tf2 = tf2/tnf2
                            if(tnf2 .gt. 0.0) tfe2 = sqrt(tfe2)/tnf2

                            rate1(mm) = tr1
                            rateerr1(mm) = tre1
                            flux1(mm) = tf1
                            fluxerr1(mm) = tfe1
                            rate2(mm) = tr2
                            rateerr2(mm) = tre2
                            flux2(mm) = tf2
                            fluxerr2(mm) = tfe2
                        endif
                    enddo
                endif
                call ftclos (lunlist,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'MERGE_SOURCE_LIST POSITION 09'
                    print *, 'STATUS FTCLOS =', status
                endif
            endif
        enddo
        n = mm
        print *, 'Total Sources: ', n

!  Sort through the sources, compare the all bands, all instruments
!  dectection information

        do i=1,n-1
            if(flux(i) .gt. 0.0) then
                dlong1 = 0.017453293*ra(i)
                dlat1 = 0.017453293*dec(i)
                do ii=i+1,n
                    if(flux(ii) .gt. 0.0) then

!  Get the angle between the two sources

                        dlong2 = 0.017453293*ra(ii)
                        dlat2 = 0.017453293*dec(ii)
                        call angsep (dlong1,dlat1,dlong2,dlat2, &
                                dang,ierr)

!  Calculate the position uncertainty, add in quadrature the 
!  statistical uncertainty from the source list and the astrometric
!  uncertainty (2") for each source, and then add them together in
!  quadrature.

                        if(ierr .eq. 0) then
                            punc = sqrt(4.0 + radecerr(i)*radecerr(i) &
                                  + 4.0 + radecerr(ii)*radecerr(ii))
                            punc = punc*4.84814e-6

                            k = 1.5*dang/(punc) + 1
                            if(k .eq. 0) k = 1
                            if(k .gt. 1000) k = 1000
                            hist1(k) = hist1(k) + 1

!  If the angle is less than 1.5x the uncertainty in the distance 
!  between the two detections get rid of the fainter detection

                            if(dang .lt. 1.5*punc) then
                                if(flux(i) .lt. flux(ii)) then
                                    flux(i) = 0
                                else
                                    flux(ii) = 0
                                endif
                            else
                                if(dang .lt. dist(i)) dist(i) = dang
                                if(dang .lt. dist(ii)) dist(ii) = dang
                            endif
                        endif
                    endif
                enddo
            endif
        enddo

!  Now get rid of the fainter detections from the list

        ii = 0
        do i=1,n
            if(flux(i) .gt. 0.0) then
                ii = ii + 1
                ra(ii) = ra(i)
                dec(ii) = dec(i)
                radecerr(ii) = radecerr(i)
                ml(ii) = ml(i)
                k = dist(i)/(2.0*2.4240684d-5) + 1
                if(k .eq. 0) k = 1
                if(k .gt. 1000) k = 1000
                hist2(k) = hist2(k) + 1
                dist(ii) = 2.06265e5*dist(i)
                flux(ii) = flux(i)
                fluxerr(ii) = fluxerr(i)
                rate(ii) = rate(i)
                rateerr(ii) = rateerr(i)
                flux1(ii) = flux1(i)
                fluxerr1(ii) = fluxerr1(i)
                flux2(ii) = flux2(i)
                fluxerr2(ii) = fluxerr2(i)
                rate1(ii) = rate1(i)
                rateerr1(ii) = rateerr1(i)
                rate2(ii) = rate2(i)
                rateerr2(ii) = rateerr2(i)
            endif
        enddo
        nrows = ii
        print *, 'Total Unique Sources: ', nrows

!  Write out the sources
!  Get an unused lun

        call ftgiou(lunout,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_SOURCE_LIST POSITION 10'
            print *, 'STATUS FTGIOU =', status
        endif

!  Check to see if there is an existing source list

        outfile = 'merged-source-list.fits'
        call ftopen (lunout,outfile,0,block,status)
        if(status .eq.  0) then
            print *, 'File exists with output file name'
            if(clobber) then
                print *, 'Clobber the file and continue'
                call ftdelt(lunout,status)
            else
                print *, 'MERGE_SOURCE_LIST POSITION 11a'
                print *, 'Will not clobber file'
                stop
            endif
        endif
        status = 0

!  Initialize the output fits file

        call ftinit(lunout,outfile,0,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_SOURCE_LIST POSITION 11b'
            print *, 'STATUS FTINIT =', status
            print *, outfile
        endif

!  Create an extension

        call ftibin(lunout,nrows,tfieldso,ttypeo,tformo,tunito, &
            extnameo,0,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_SOURCE_LIST POSITION 12'
            print *, 'STATUS FTIBIN =', status
            print *, outfile
        endif

!  Add some keywords

        comm = &
       'This file was produced by the ESAS task merge_source_list'
        call ftpcom(lunout,comm,status)

!  Put the data into the tables

        call ftpcnd(lunout,1,1,1,nrows,ra,999,status)
        comm = 'Source Right Ascension'
        call ftmcom (lunout,'TTYPE1',comm,status)
        call ftpcnd(lunout,2,1,1,nrows,dec,999,status)
        comm = 'Source Declination'
        call ftmcom (lunout,'TTYPE2',comm,status)
        call ftpcne(lunout,3,1,1,nrows,radecerr,999,status)
        comm = 'Source Position Error (arc seconds)'
        call ftmcom (lunout,'TTYPE3',comm,status)
        call ftpcne(lunout,4,1,1,nrows,ml,999,status)
        comm = 'Maximum Likelihood Detection Value'
        call ftmcom (lunout,'TTYPE4',comm,status)
        call ftpcne(lunout,5,1,1,nrows,flux,999,status)
        comm = 'Source Total Flux'
        call ftmcom (lunout,'TTYPE5',comm,status)
        call ftpcne(lunout,6,1,1,nrows,fluxerr,999,status)
        comm = 'Source Total Flux Uncertainty'
        call ftmcom (lunout,'TTYPE6',comm,status)
        call ftpcne(lunout,7,1,1,nrows,flux1,999,status)
        comm = 'Source Soft Flux'
        call ftmcom (lunout,'TTYPE7',comm,status)
        call ftpcne(lunout,8,1,1,nrows,fluxerr1,999,status)
        comm = 'Source Soft Flux Uncertainty'
        call ftmcom (lunout,'TTYPE8',comm,status)
        call ftpcne(lunout,9,1,1,nrows,flux2,999,status)
        comm = 'Source Hard Flux'
        call ftmcom (lunout,'TTYPE9',comm,status)
        call ftpcne(lunout,10,1,1,nrows,fluxerr2,999,status)
        comm = 'Source Hard Flux Uncertainty'
        call ftmcom (lunout,'TTYPE10',comm,status)
        call ftpcne(lunout,11,1,1,nrows,dist,999,status)
        comm = 'Separation Angle from the Nearest Source (arcsec)'
        call ftmcom (lunout,'TTYPE11',comm,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_SOURCE_LIST POSITION 13'
            print *, 'STATUS FTPCN =', status
            print *, outfile
        endif

        call ftclos (lunout,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_SOURCE_LIST POSITION 14'
            print *, 'STATUS FTCLOS =', status
        endif

        histmax = 0.0
        histrange = 0.0
        do i=2,999
            if(hist1(i) .gt. histmax) histmax = hist1(i)
            if(hist1(i) .gt. 0) histrange = i
        enddo
        histmax = 1.05*histmax
        histrange = histrange*10.0/60.0

        open(lunout,status='unknown',form='formatted', &
            file='sep1.qdp')
        write(lunout,*) 'viewport 0.2 0.11 0.95 0.9'
        write(lunout,*) 'font roman'
        write(lunout,*) 'cs 1.5'
        write(lunout,*) 'la t Source Separation'
        write(lunout,*) 'la x Separation (units of position error)'
        write(lunout,*) 'la y Number'
        write(lunout,*) 'la f'
        write(lunout,*) 'col 1 on 2'
        write(lunout,*) 'line step'
        write(lunout,*) 'r 0 ', histrange, '  0 ', histmax
        do i=1,1000
            write(lunout,*) (i-0.5)*10.0/60.0, hist1(i)
        enddo
        close(lunout)

        histmax = 0.0
        histrange = 0.0
        do i=2,999
            if(hist2(i) .gt. histmax) histmax = hist2(i)
            if(hist2(i) .gt. 0) histrange = i
        enddo
        histmax = 1.05*histmax
        histrange = histrange*10.0/60.0

        open(lunout,status='unknown',form='formatted', &
            file='sep2.qdp')
        write(lunout,*) 'viewport 0.2 0.11 0.95 0.9'
        write(lunout,*) 'font roman'
        write(lunout,*) 'cs 1.5'
        write(lunout,*) 'la t Minimum Source Separation'
        write(lunout,*) 'la x Separation (arc minutes)'
        write(lunout,*) 'la y Number'
        write(lunout,*) 'la f'
        write(lunout,*) 'col 1 on 2'
        write(lunout,*) 'line step'
        write(lunout,*) 'r 0 ', histrange, '  0 ', histmax
        do i=1,1000
            write(lunout,*) (i-0.5)*10.0/60.0, hist2(i)
        enddo
        close(lunout)

        ramin = 1000.0
        ramax = 0.0
        decmin = 90.0
        decmax = -90.0
        do i=1,nrows
            if(ra(i) .lt. ramin) ramin = ra(i)
            if(ra(i) .gt. ramax) ramax = ra(i)
            if(dec(i) .lt. decmin) decmin = dec(i)
            if(dec(i) .gt. decmax) decmax = dec(i)
        enddo
        if((ramax - ramin) .gt. 180.0) then
            do i=1,nrows
                if(ra(i) .gt. 180.0) ra(i) = ra(i) - 360.0
            enddo
            ramin = 1000.0
            ramax = -1000.0
            do i=1,nrows
                if(ra(i) .lt. ramin) ramin = ra(i)
                if(ra(i) .gt. ramax) ramax = ra(i)
            enddo
        endif
        delra = ramax - ramin
        deldec = decmax - decmin
        raave = (ramax + ramin)/2.0
        decave = (decmax + decmin)/2.0
        scale = cos(decave*3.14159265/180.0)
        rarange = scale*delra
        if(rarange .lt. deldec) then
            ramin = raave - 1.025*0.5*delra/scale
            ramax = raave + 1.025*0.5*delra/scale
            decmin = decave - 1.05*deldec/2.0
            decmax = decave + 1.05*deldec/2.0
        else
            ramin = raave - 1.05*delra/2.0
            ramax = raave + 1.05*delra/2.0
            decmin = decave - 0.5*delra*rarange/deldec
            decmax = decave + 0.5*delra*rarange/deldec
        endif

        open(lunout,status='unknown',form='formatted', &
            file='pos.qdp')
        write(lunout,*) 'viewport 0.2 0.11 0.78515 0.9'
        write(lunout,*) 'font roman'
        write(lunout,*) 'cs 1.5'
        write(lunout,*) 'la t Source Positions'
        write(lunout,*) 'la x Right Ascension'
        write(lunout,*) 'la y Declination'
        write(lunout,*) 'la f'
        write(lunout,*) 'col 1 on 2'
        write(lunout,*) 'line off 2'
        write(lunout,*) 'mark 2 on 2'
        write(lunout,*) 'r ', ramax, ramin, decmin, decmax
        do i=1,nrows
            write(lunout,*) ra(i), dec(i)
        enddo
        close(lunout)

        return

  end subroutine merge_source_list

end module merge_source_list_mod
