!+
! Module: pn_back_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: pn_back_mod.f90,v 1.1 2009/02/06 14:10:35 aibarra Exp $
!-
!
!  SAS Version 13.0
!  Version 5.0  6 November 2012
!
!  Creates particle background spectra and images for the pn
!
!  pn-back is part of the XMM-ESAS package and it produces model 
!  particle background spectra and images (in detector coordinates) 
!  from the output files of the perl script pn-spectra.
!
!  Input:
!    prefix: Exposure ID prefix - e.g., prefix=S001
!    caldb: QPB data directory - e.g., caldb=/XMM/esas/caldb
!    diag: diagnostic flag - e.g., diag=1
!    elow: Lower energy limit for the energy band in eV, if only spectra
!        are desired set to 0 - e.g., elow=0 or elow=400
!    ehigh: Upper energy limit for the energy band in eV, if only spectra
!        are desired set to 0 - e.g., ehigh=0 or ehigh=1250
!    quad1: CCD control for quadrant #1, 1 to include, 0 to not - e.g.,
!        quad1=0 or quad1=1
!    quad2: CCD control for quadrant #2, 1 to include, 0 to not - e.g.,
!        quad2=0 or quad2=1
!    quad3: CCD control for quadrant #3, 1 to include, 0 to not - e.g.,
!        quad3=0 or quad3=1
!    quad4: CCD control for quadrant #4, 1 to include, 0 to not - e.g.,
!        quad4=0 or quad4=1
!
!    Example: pn_back prefix=S003 caldb=/CalDB diag=0 elow=0
!                 ehigh=0 quad1=1 quad2=1 quad3=1 quad4=1 

module pn_back_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public pn_back

contains

  !---------------------------------------------------------------------
  subroutine pn_back
  !---------------------------------------------------------------------

        character (len=1) :: db_fi(132), qpb_fi(25), xmmd(132)
        character (len=4) :: prefix
        character (len=8) :: ctype(2), tform(10), ttype(10), tunit(10)
        character (len=20) :: extname
        character (len=24) :: title
        character (len=25) :: mode
        character (len=80) :: back_file, imo_name, obj_file, &
            obj_rmf, qdp_file, spec_file, spec_name, specos_file
        character (len=132) :: im_name, im_name_oot, xmmdir
        character (len=25) :: qpb_file = '/pn-qpb.fits.gz'
        character (len=80) :: db_file = '      '

        logical clobber

        integer :: bitpix, block, chan(4096), channel(4096), &
            diag, eh, el, errr, gcount, group, grup(4096), &
            hdutype, i, i1, i2, ichan, ii, iii, ini4(780,780), it, &
            j, jj, luna, lunb, lunc, n, naxes(2), &
            naxis, nchan, nchanrin, nchanrout, nord, &
            obsid, obsiddum, outh(2), outl(2), pcount, &
            plc_all(2), plc_alk(2), plc_hard(2), plc_hi(2), &
            plc_hih(2), plc_lo(2), quad(4), quad1, quad2, quad3, &
            quad4, qual(4096), revl, tfields, varidat
        integer, dimension(2) :: band = (/0, 0/)
        integer :: err = 0
        integer :: ipix = 1
        integer :: nels = 4
        integer :: norm = 1
        integer :: pflag = 1
        integer :: smo_fact = 12
        integer :: status = 0

!  mine and maxe are the channel ranges for data not affected 
!  by instrumental lines

        integer, dimension(4) :: mine = (/130, 350,  950, 2000/)
        integer, dimension(4) :: maxe = (/230, 870, 1030, 2650/)

        real :: area, aug_expo, aug_rate, &
            aug_spec(4096), avtot, back, back_oot, bandl, &
            bandh,  coef(10), cor_area_fwc(4), cor_area_obs(4), &
            cor_exp_obs(4), cor_exp_fwc(4), c_val(3), detm(4096,4096), &
            emax(4096), emaxin(4096), emin(4096), eminin(4096), &
            ener(4096), expo, expo_oot, fc1(4096), fc_spec(4096,4), &
            fc_uncr(4096,4), fillin(4096), hard, hard_rate, high, hih, &
            hihrat, image(780,780),  image_oot(780,780), lim_area, &
            low, mask(4096), mk1(4096), modl(4096), ntot, obj_back, &
            obj_cnts(4096), obj_expo, obs_spec(4096,4), &
            obs_uncr(4096,4), oc_mask(4096,4)
        real :: oc_spec(4096,4), oc_uncr(4096,4), oc1(4096), &
            outim(780,780), rate, reg_area_fwc(4), reg_area_obs(4), &
            reg_exp_fwc(4), reg_exp_obs(4), reg_mask(4096,4), &
            reg_rati(4096,4), reg_spec(4096,4), reg_uncr(4096,4), &
            sighard, sigrate, soft, soft_rate, spec(4096), &
            spec_oot(4096), stat(4096), t1, t2, t3, temp_bot(4096), &
            temp_oc(4096), temp_top(4096), tempe(4096), temps(4096), &
            tempsp, tempu(4096), tempunc, tmodcounts, tot, tot0, &
            tot1, tot2, tot3, uc1(4096), uncr(4096), uncr_oot(4096), &
            ymax, ymin 
        real :: amsca = 1440000.0
        real, dimension(3) :: aug_scal = (/0.0001, 0.2, 50.0/)
!        real, dimension(3) :: aug_scal = (/0.001, 1.0, 0.0/)
!        real, dimension(3) :: aug_scal = (/0.0001, 0.1, 100.0/)
!        real, dimension(3) :: aug_scal = (/0.0001, 0.2, 100.0/)
!        real, dimension(3) :: aug_scal = (/0.000, 0., 100.0/)
        real :: aug_time = 1.0e6
        real :: ootsca = 0.063

        double precision cdelt(2), crpix(2), crval(2)

        equivalence (db_file,db_fi)
        equivalence (qpb_file,qpb_fi)
        equivalence (xmmdir,xmmd)

        prefix = stringParameter("prefix")
        xmmdir = stringParameter("caldb")
        diag = intParameter("diag")
        el = intParameter("elow")
        eh = intParameter("ehigh")
        quad1 = intParameter("quad1")
        quad2 = intParameter("quad2")
        quad3 = intParameter("quad3")
        quad4 = intParameter("quad4")
        clobber = booleanParameter("clobber")

!  Create the quiescent particle background file path and name

        ii = 1
        do while (xmmd(ii) .ne. ' ')
            db_fi(ii) = xmmd(ii)
            ii = ii + 1
        enddo
        do iii=0,20
            db_fi(ii+iii) = qpb_fi(iii+1)
        enddo

!  Zero a few arrays

        errr = 0
        do i=1,4096
            fc1(i) = 0.0
            fillin(i) = 0.0
            mask(i) = 0.0
            mk1(i) = 0.0
            modl(i) = 0.0
            oc1(i) = 0.0
            spec(i) = 0.0
            spec_oot(i) = 0.0
            uc1(i) = 0.0
            uncr(i) = 0.0
            uncr_oot(i) = 0.0
        enddo
        do i=1,780
            do ii=1,780
                outim(i,ii) = 0.0
            enddo
        enddo

!  Process the input

        quad(1) = quad1
        quad(2) = quad2
        quad(3) = quad3
        quad(4) = quad4
        print *, 'xmmdir: ', xmmdir
        print *, 'Quiescent particle background data file:'
        print *, db_file
        print *, 'diag: ', diag
        print *, 'prefix: ', prefix
        print *, 'el, eh: ', el, eh
        print *, 'Quadrant selection', quad
        print *, '  '

        lim_area = 0.0
        bandl = el/1000.0
        bandh = eh/1000.0

!  Get information about the object spectrum and create an 
!  OOT subtracted spectrum

        print *, 'Create the OOT subtracted spectrum'
        print *, ' '

!  First read in the object spectrum and get certain keywords

        write(spec_name,2991) prefix
 2991   format('pn',a4,'-obj.pi')
        print *, 'Observed spectrum: ', spec_name
        print *, ' '
        call read_pha (spec_name,4096,nchan,channel,spec,uncr,qual, &
                grup,obj_expo,obj_back,area,revl,obsid,mode,err)
        if(diag .ge. 2) then
            print *, 'nchan, obj_expo, obj_back, ', &
                'obj_back/amsca, area, revl, err'
            print 2992, nchan, obj_expo, obj_back, &
                obj_back/amsca, area, revl, err
 2992       format(i5,f10.1,f12.0,f8.2,f8.3,2i5)
            print *, ' '
        endif

        if(err .ge. 1) then
            print *, 'PN_BACK POSITION 01'
            print *, 'Error on return from READ_PHA'
            print *,'missing file: ',spec_name,' in pn-back'
            stop
        else

!  Set the PN mode

            if(mode .eq. 'PrimeFullWindow') then
                print *, 'PN Mode: ', mode
                ootsca = 0.0630
                print 2993, ootsca
 2993           format(' OOT Scale: ',f8.4)
                print *, ' '
            elseif(mode .eq. 'PrimeFullWindowExtended3 ') then
                print *, 'PN Mode: ', mode
                ootsca = 0.0232
                print 2993, ootsca
                print *, ' '
            elseif(mode .eq. 'PrimeFullWindowExtended5 ') then
                print *, 'PN Mode: ', mode
                ootsca = 0.0163
                print 2993, ootsca
                print *, ' '
            else
                print *, 'PN_BACK POSITION 02'
                print *, 'Bad submode: ', mode
                stop
            endif

!  Get the channel to energy conversion from the rmf

            write(obj_rmf,2994) prefix
 2994       format('pn',a4,'.rmf')
            print *, 'Spectrum RMF: ', obj_rmf
            print *, ' '
            call read_rmf (obj_rmf,4096,detm,chan,eminin,emaxin, &
                    nchanrin,emin,emax,nchanrout)
            do ichan=1,nchan
                ener(ichan) = (emin(ichan) + emax(ichan))/2.
            enddo

!  Read in the OOT events

            write(spec_name,2995) prefix
 2995       format('pn',a4,'-obj-oot.pi')
            print *, 'OOT spectrum: ', spec_name
            print *, ' '
            call read_pha (spec_name,4096,nchan,channel,spec_oot, &
                    uncr_oot,qual,grup,expo_oot,back_oot,area,revl, &
                    obsid,mode,err)
            if(err .ge. 1) then
                print *, 'PN_BACK POSITION 03'
                print *, 'Error on return from READ_PHA'
                print *,'missing file: ',spec_name,' in pn-back'
                stop
            else
                if(diag .ge. 2) then
                    print *, 'nchan, expo_oot, back_oot, ', &
                        'back_oot/amsca, area, revl, err'
                    print 2992, nchan, expo_oot, back_oot, &
                        back_oot/amsca, area, revl, err
                    print *, ' '
                endif

!  Subtract off the OOT events, scale the spectrum both by the
!  ratio of the exposure and the ratio of the backscale value,
!  as well as the OOT scale factor.

                do ichan=1,nchan
                    obj_cnts(ichan) = spec(ichan) - &
                        ootsca*spec_oot(ichan)*(obj_expo/expo_oot)* &
                        (obj_back/back_oot)
                    stat(ichan) = sqrt(uncr(ichan)**2 + &
                        (ootsca*uncr_oot(ichan)*(obj_expo/expo_oot)* &
                        (obj_back/back_oot))**2)
                enddo
            endif
        endif
        print 2996, obj_expo
 2996   format(' Exposure: ',f9.1,' s')
        print *, ' '

!  Now that we have the energy binning information, set some
!  band limits for both evaluation bands and for detector 
!  background line avoidance.

        do i=1,nchan-1

!  Set channel ranges for some energy bands, total, soft, hard

            if((ener(i) .lt. 0.4) .and. (ener(i+1) .ge. 0.4)) &
                    plc_all(1) = i + 1
            if((ener(i) .le. 12.0) .and. (ener(i+1) .gt. 12.0)) &
                    plc_all(2) = i
            if((ener(i) .lt. 0.6) .and. (ener(i+1) .ge. 0.6)) &
                    plc_lo(1) = i + 1
            if((ener(i) .lt. 1.3) .and. (ener(i+1) .ge. 1.3)) &
                    plc_lo(2) = i
            if((ener(i) .lt. 1.65) .and. (ener(i+1) .ge. 1.65)) &
                    plc_hi(1) = i + 1
            if((ener(i) .lt. 7.2) .and. (ener(i+1) .ge. 7.2)) &
                    plc_hi(2) = i
            if((ener(i) .lt. 9.2) .and. (ener(i+1) .ge. 9.2)) &
                    plc_hih(1) = i + 1
            if((ener(i) .lt. 13.5) .and. (ener(i+1) .ge. 13.5)) &
                    plc_hih(2) = i

!  Set channel ranges for the Al Kalpha and high energy 
!  line contamination

            if((ener(i) .lt. 1.3) .and. (ener(i+1) .ge. 1.3)) &
                    plc_alk(1) = i + 1
            if((ener(i) .le. 1.65) .and. (ener(i+1) .gt. 1.65)) &
                    plc_alk(2) = i
            if((ener(i) .lt. 7.2) .and. (ener(i+1) .ge. 7.2)) &
                    plc_hard(1) = i + 1
            if((ener(i) .le. 9.2) .and. (ener(i+1) .gt. 9.2)) &
                    plc_hard(2) = i

!  Set channel ranges for energy band

            if((bandl .eq. 0.0) .and. (bandh .eq. 0.0)) then

!  If user defined band limits are 0 0 then set default values

                if((ener(i) .lt. 0.4) .and. (ener(i+1) .ge. 0.4)) &
                        band(1) = i + 1
                if((ener(i) .le. 7.2) .and. (ener(i+1) .gt. 7.2)) &
                        band(2) = i
            else

!  Set channel ranges for the user selected energy band

                if((ener(i) .lt. bandl) .and. (ener(i+1) .ge. bandl)) &
                        band(1) = i + 1
                if((ener(i) .le. bandh) .and. (ener(i+1) .gt. bandh)) &
                        band(2) = i
            endif

!  Set channel ranges for the interpolation bridge, currently the 
!  same as the settings for the line contamination

            if((ener(i) .lt. 1.3) .and. (ener(i+1) .ge. 1.3)) &
                    outl(1) = i + 1
            if((ener(i) .le. 1.65) .and. (ener(i+1) .gt. 1.65)) &
                    outl(2) = i
            if((ener(i) .lt. 7.2) .and. (ener(i+1) .ge. 7.2)) &
                    outh(1) = i + 1
            if((ener(i) .le. 9.2) .and. (ener(i+1) .gt. 9.2)) &
                    outh(2) = i
        enddo
 
        if(diag .ge. 2) then
            print *, 'Total band channel range: ', plc_all
            print *, 'Soft band channel range: ', plc_lo
            print *, 'Hard band channel range: ', plc_hi
            print *, 'Hardest band channel range: ', plc_hih
            print *, 'Selected band channel range: ', band
            print *, 'Low bridge band channel range: ', outl
            print *, 'High bridge band channel range: ', outh
            print *, '  '
        endif

!  Read the observation chip spectra to get the exposure.  The
!  code also calculates the OOT subtracted spectra and uncertainty,
!  but the program doesn't use them for anything.

!  The obj data are the observation data

        print *, 'Process the individual quadrant observation data ', &
                'and create OOT '
        print *, 'subtracted spectra and exposures which are area ', &
                'and exposure normalized.'
        print *, ' '
        if(diag .ge. 2) then
            print *, 'List individual file names and  nchan, expo, ', &
                        'back, back/amsca, area, revl, err'
        endif
        do i=1,4
            if(quad(i) .eq. 1) then
                write(spec_name,3000) prefix, i
 3000           format('pn',a4,'-',i1,'obj.pi')
                call read_pha (spec_name,4096,nchan,channel,spec,uncr, &
                        qual,grup,expo,back,area,revl,obsid,mode,err)
                if(diag .ge. 1) print *, spec_name
                if(diag .ge. 2) print 2992, nchan, expo,  &
                        back, back/amsca, area, revl, err
                if((back .gt. 0) .and. (expo .gt. 0)) then

!  If err = 0 then the file exists

                    if(err .eq. 0) then
                        reg_area_obs(i) = back
                        reg_exp_obs(i) = expo
                    else
                        errr=1
                        print *, 'PN_BACK POSITION 04'
                        print *,'missing file: ',spec_name, &
                                ' in pn-back'
                        stop
                    endif
 
!  Now do the OOT events

                    write(spec_name,3001) prefix, i
 3001               format('pn',a4,'-',i1,'obj-oot.pi')
                    call read_pha (spec_name,4096,nchan,channel, &
                            spec_oot,uncr_oot,qual,grup,expo_oot, &
                            back_oot,area,revl,obsid,mode,err)
                    if(diag .ge. 1) print *, spec_name
                    if(diag .ge. 2) print 2992, nchan, expo_oot,  &
                            back_oot, back_oot/amsca, area, revl, err

!  If err = 0 then the file exists so create the OOT subtracted spectrum

                    if(err .eq. 0) then
                        do ichan=1,nchan
                            obs_spec(ichan,i) =  &
                                amsca*(spec(ichan)/back/expo - &
                                ootsca*spec_oot(ichan)/ &
                                back_oot/expo_oot)
                            obs_uncr(ichan,i) =  &
                                amsca*(sqrt(uncr(ichan)**2 +  &
                                (ootsca*uncr_oot(ichan)*(back/back_oot)* &
                                (expo/expo_oot))**2)/back/expo)
                        enddo
                    else
                        errr=1
                        print *, 'PN_BACK POSITION 05'
                        print *,'missing file: ', spec_name, &
                                ' in pn-back'
                        stop
                    endif
                else
                    quad(i) = 0
                endif
            endif
        enddo
        print *, '  '
        if(diag .ge. 1) then
            print *, 'Backscale and exposure from the observation data' 
            do i=1,4
                if(quad(i) .eq. 1) then
                    print 2997, i, reg_area_obs(i),  &
                        reg_area_obs(i)/amsca, reg_exp_obs(i)
 2997               format(' Quadrant',i2,'  backscale=',f10.0, &
                        ', ',f7.2,'  exposure =',f9.1)
                endif
            enddo
            print *, '  '
        endif

!  Read the fwc spectra from the object region
!  Figure out which chips are involved
!  Use reg_area as a vector describing which chips are in use

!  The ff data are the FWC data from the object region

        print *, 'Process the FWC data from the object region and ', &
                'create OOT '
        print *, 'subtracted and area and exposure normalized ', &
                'spectra'
        print *, ' '
        if(diag .ge. 2) then
            print *, 'List individual file names and nchan, expo, ', &
                        'back, back/1440000.0, area, revl, err'
        endif
        do i=1,4
            if(quad(i) .eq. 1) then

!  Read in the normal data

                write(spec_name,3005) prefix, i
 3005           format('pn',a4,'-',i1,'ff.pi')
                call read_pha (spec_name,4096,nchan,channel,spec,uncr, &
                      qual,grup,expo,back,area,revl,obsiddum,mode,err)
                if(diag .ge. 1) print *, spec_name
                if(diag .ge. 2) print 2992, nchan, expo,  &
                        back, back/amsca, area, revl, err

!  If err = 0 then the file exists

                if(err .eq. 0) then
                    reg_exp_fwc(i) = expo
                    reg_area_fwc(i) = back
                else
                    errr=1
                    print *, 'PN_BACK POSITION 06'
                    print *,'missing file: ',spec_name, &
                            ' in pn-back'
                    stop
                endif

!  Read in the OOT data

                write(spec_name,3006) prefix, i
 3006           format('pn',a4,'-',i1,'ff-oot.pi')
                call read_pha (spec_name,4096,nchan,channel, &
                    spec_oot,uncr_oot,qual,grup,expo_oot,back_oot, &
                    area,revl,obsiddum,mode,err)
                if(diag .ge. 1) print *, spec_name
                if(diag .ge. 2) print 2992, nchan, expo_oot,  &
                        back_oot, back_oot/amsca, area, revl, err
                if(err .eq. 0) then
                    if((back .ne. 0) .and. (expo .ne. 0)) then 

!  There is active area on this chip, normalize the spectrum
!  and subtract the OOT data.  Data are normalized to per area
!  per exposure

                        do ichan=1,nchan
                            reg_spec(ichan,i) =  &
                                    amsca*(spec(ichan)/back/expo - &
                                    ootsca*spec_oot(ichan)/ &
                                    back_oot/expo_oot)
                            reg_uncr(ichan,i) = &
                                    amsca*sqrt(uncr(ichan)**2 + &
                                    (ootsca*uncr_oot(ichan)* &
                                    (back/back_oot)* &
                                    (expo/expo_oot))**2)/back/expo
                        enddo
                    else
                        do ichan=1,nchan
                            reg_spec(ichan,i) = 0.0
                            reg_uncr(ichan,i) = 0.0
                        enddo
                        reg_area_fwc(i) = 0.0
                    endif
                else
                    errr = 1
                    print *, 'PN_BACK POSITION 07'
                    print *,'missing file: ',spec_name, &
                            ' in pn-back'
                    stop
                endif
            endif
        enddo

        if(diag .ge. 1) then
            print *, '  '
            print *, 'Region backscale and exposure from the FWC data'
            do i=1,4
                if(quad(i) .eq. 1) then
                    print 2997, i, reg_area_fwc(i), &
                            reg_area_fwc(i)/amsca, reg_exp_fwc(i)  
                endif
            enddo
            print *, '  '
        endif

!  Remove any quadrant that has less area than the limiting area, lim_area

        do i=1,4
            if(quad(i) .eq. 1) then
                if((reg_area_fwc(i) .lt. lim_area) .or. &
                            (reg_area_fwc(i) .eq. 0.0) .or.  &
                            (reg_area_obs(i) .eq. 0.0)) then
                    reg_area_fwc(i) = 0.0
                    reg_exp_fwc(i) = 0.0
                    do ichan=1,nchan
                        reg_spec(ichan,i) = 0.0
                        reg_uncr(ichan,i) = 0.0
                    enddo
                endif
            endif
        enddo

!  Read corner spectra from the observation
!  First set up the spectrum masks
!  This eliminates any deselected quadrants

        do i=1,4
            if(quad(i) .eq. 1) then
                do ichan=1,nchan
                    oc_mask(ichan,i) = 1
                enddo
            else
                do ichan=1,nchan
                    oc_mask(ichan,i) = 0
                enddo
            endif
        enddo

!  The oc data are observation data from the corners

        print *, 'Process the observation data from the corner ', &
                'regions and create '
        print *, 'OOT subtracted and area and exposure ', &
                'normalized spectra'
        print *, ' '
        do i=1,4
            print *, 'Quadrant: ', i
            if(quad(i) .eq. 1) then
                write(spec_name,3010) prefix, i
 3010           format('pn',a4,'-',i1,'oc.pi')

!  Read in the normal data

                call read_pha (spec_name,4096,nchan,channel,spec,uncr, &
                        qual,grup,expo,back,area,revl,obsid,mode,err)
                if(diag .ge. 1) print *, spec_name
                if(diag .ge. 2) then 
                    print *, 'nchan, expo, back, back/amsca, area, ', &
                            'revl, err'
                    print 2992, nchan, expo, back, back/amsca, area,  &
                            revl, err
                endif

!  If err = 0 then the file exists

                if(err .eq. 0) then
                    cor_area_obs(i) = back 
                    cor_exp_obs(i) = expo
                else
                    errr=1
                    print *,'missing file: ',spec_name, &
                            ' in pn-back'
                    print *, 'PN_BACK POSITION 08'
                    stop
                endif

!  Read in the OOT data

                write(spec_name,3011) prefix, i
 3011           format('pn',a4,'-',i1,'oc-oot.pi')

!  Read in the OOT data

                call read_pha (spec_name,4096,nchan,channel, &
                            spec_oot,uncr_oot,qual,grup,expo_oot, &
                            back_oot,area,revl,obsid,mode,err)
                if(diag .ge. 1) print *, spec_name
                if(diag .ge. 2) then 
                    print 2992, nchan, expo_oot, back_oot, &
                            back_oot/amsca, area, revl, err
                endif
                if(err .eq. 0) then
                    rate = 0.0
                    soft = 0.0
                    hard = 0.0
                    high = 0.0
                    low = 0.0
                    hih = 0.0

!  Make the corner OOT corrected spectrum and get the hardness and 
!  count rate parameters for the augmentation selection.  Note, this 
!  spectra is not normalized (units are counts).

                    do ichan=1,nchan
                        spec(ichan) = spec(ichan) -  &
                                 ootsca*spec_oot(ichan)*(expo/expo_oot) &
                                 *(back/back_oot)
                        uncr(ichan) = sqrt(uncr(ichan)**2 +  &
                                 (ootsca*ootsca*uncr_oot(ichan)*  &
                                 (expo/expo_oot)*(back/back_oot)))
                    enddo

!  Find the number of counts in the observed spectrum and get the 
!  count rate

                    do ichan=plc_all(1),plc_all(2)
                        rate = rate + spec(ichan)
                    enddo
                    print 2998, rate, expo
 2998               format(' Counts (0.4-12.0 keV): ', f9.1, &
                            '  Exposure: ', f9.1)

!  Find the hard and soft count rates and hardness ratio

                    do ichan=plc_hih(1),plc_hih(2)
                        hih = hih + spec(ichan)
                    enddo
                    do ichan=plc_hi(1),plc_hi(2)
                        high = high + spec(ichan)
                    enddo
                    do ichan=plc_lo(1),plc_lo(2)
                        low = low + spec(ichan) 
                    enddo
                    hard = high/low
                    sighard = sqrt((high/(low**2)) + &
                            ((high**2)*low)/low**4)

                    rate = low + high
                    sigrate = sqrt(rate)
                    if(diag .ge. 2) then 
                        print *, 'low, high, hard, sighard, ', &
                                'rate/expo, sigrate/expo, expo'
                        print 2999, low, high, hard, sighard,  & 
                                 rate/expo, sigrate/expo, expo
 2999                   format(2f8.1,2f8.3,2f8.5,f8.1)
                    endif
                    rate = amsca*rate/cor_exp_obs(i)/cor_area_obs(i)
                    sigrate = amsca*sigrate/ &
                             cor_exp_obs(i)/cor_area_obs(i)
                    soft_rate = amsca*low/cor_exp_obs(i)/ &
                             cor_area_obs(i)
                    hard_rate = amsca*high/cor_exp_obs(i)/ &
                             cor_area_obs(i)
                    hihrat = amsca*hih/cor_exp_obs(i)/cor_area_obs(i)

!  Set the search values for the augmentation spectra

                    c_val(1) = rate
                    c_val(2) = hard
                    c_val(3) = revl
                    if(diag .ge. 1) then
                        print 3021, c_val(1), sigrate
 3021                   format(' Normalized Count Rate: ',f8.6,  &
                                ' +/-',f8.6)
                        print 3022, c_val(2), sighard
 3022                   format(' Hardness: ',f8.3,' +/-',f8.3)
                        print 3023, low, high, hih
 3023                   format(' Counts: ', 3f8.1)
                        print 3123, soft_rate, hard_rate, hihrat
 3123                   format(' Count Rates: ', 3f8.5)
                        print *, 'CalDB File: ', db_file
                        print 2024, c_val(3)
 2024                   format(' Revolution: ',f8.0)
                        print *, ' '
                    endif

!  Augment the observed corner spectrum - normalized /s/arcmin^2

                    print *, 'Augment the corner spectrum'
                    call aug_get_pnexp(db_file,c_val,aug_scal,aug_time, &
                            emin,emax,pflag,aug_spec,aug_expo, &
                            i,4096,obsid,prefix,mode,plc_lo,plc_hi)

                    do ichan=1,nchan
!          print *, ichan, spec(ichan), uncr(ichan), &
!          aug_expo, cor_area_obs(i), amsca, aug_spec(ichan)
!                        if(ichan .eq. 400) stop 
                        uncr(ichan) = sqrt(uncr(ichan)**2 +  &
                                aug_expo*(cor_area_obs(i)/amsca)* &
                                aug_spec(ichan))/ &
                                (cor_area_obs(i)/amsca)/ &
                                (cor_exp_obs(i) + aug_expo)
                        spec(ichan) = (spec(ichan) +  &
                                aug_expo*(cor_area_obs(i)/amsca)* &
                                aug_spec(ichan))/ &
                                (cor_area_obs(i)/amsca)/ &
                                (cor_exp_obs(i) + aug_expo)
                        if(spec(ichan) .lt. 0.0) then 
                            spec(ichan) = 0.0
                            uncr(ichan) = 0.0
                         endif
!                        print *, spec(ichan), uncr(ichan)
                    enddo
                    if(diag .ge. 1) then
                        print 3025, expo, aug_expo
 3025                   format(' Original and augmented exposure ', &
                            'times: ',f8.1,f11.1)
                        print *, ' '
                    endif

!  Add the observation exposure to the

                    expo = cor_exp_obs(i) + aug_expo
                    cor_exp_obs(i) = expo

!  Test the  augmentation of the background spectrum
!  Find and print out some count rates
!  Soft, Hard, and Very Hard bands

                    low = 0.0
                    high = 0.0
                    hih = 0.0
                    do ichan=plc_lo(1),plc_lo(2)
                        low = low + spec(ichan) 
                    enddo
                    do ichan=plc_hi(1),plc_hi(2)
                        high = high + spec(ichan)
                    enddo
                    do ichan=plc_hih(1),plc_hih(2)
                        hih = hih + spec(ichan)
                    enddo

!  The augmented count rate is the sum of the soft and 
!  hard band count rates

                    aug_rate = low + high

!  The hardness ratio is the ration of the hard to 
!  soft bands

                    if(diag .ge. 1) then
                        print *, 'Comparison of the count rate'
                        print 3026, c_val(1), aug_rate, &
                                aug_rate/c_val(1)
 3026                   format(' Before, After, Ratio: ',2f8.5,f8.3)
                        print *, 'Comparison of the very hard band'
                        print 3026, hihrat, hih, hih/hihrat
                        print *, 'Comparison of the hard band'
                        print 3026, hard_rate, high, high/hard_rate
                        print *, 'Comparison of the soft band'
                        print 3026, soft_rate, low, low/soft_rate
                        print *, 'Comparison of the hardness ratio'
                        print 3026, c_val(2), high/low, & 
                                (high/low)/c_val(2)
                        print *, 'Augmented low, high, and hardness'
                        print 3026, low, high, high/low
                    endif

!  Scale by count rate and then test the scaling of the 
!  background spectrum

                    if(norm .eq. 1) then
                        print *, ' '
                        print *, 'The data are being scaled'
                        do ichan=1,nchan
                            spec(ichan) = spec(ichan)*c_val(1)/aug_rate
                            uncr(ichan) = uncr(ichan)*c_val(1)/aug_rate
                        enddo

                        low = 0.0
                        high = 0.0
                        do ichan=plc_lo(1),plc_lo(2)
                            low = low + spec(ichan) 
                        enddo
                        do ichan=plc_hi(1),plc_hi(2)
                            high = high + spec(ichan)
                        enddo

                        if(diag .ge. 1) then
                            print *, 'Comparison of the count rate'
                            print 3026, c_val(1), low+high, &
                                      (low+high)/c_val(1)
                            print *, 'Comparison of the hard band'
                            print 3026, hard_rate, high, high/hard_rate
                            print *, 'Comparison of the soft band'
                            print 3026, soft_rate, low, low/soft_rate
                            print *, 'Comparison of the hardness ratio'
                            print 3026, c_val(2), high/low, & 
                                    (high/low)/c_val(2)
                        endif
                    endif

!  The spectrum is normalized to per unit area (arcmin^2) per second

                    do ichan=1,nchan
                        oc_spec(ichan,i) = spec(ichan)
                        oc_uncr(ichan,i) = uncr(ichan)
                    enddo
                    cor_exp_obs(i) = expo
                    if(diag .ge. 1) then
                        print *, ' '
                        print 3027, i, cor_area_obs(i)/amsca, &
                                 cor_exp_obs(i)
 3027                   format(' Quad:',i2,' Back:',f6.2, &
                                 ' Exposure:',f11.1) 
                    endif
                else
                    do ichan=1,nchan
                        oc_spec(ichan,i) = 0.0
                        oc_uncr(ichan,i) = 0.0
                    enddo
                    cor_area_obs(i) = 0.0
                    if(reg_area_fwc(i) .ne.  0.0) then
                        errr = 1
                        do ichan=1,nchan
                            reg_spec(ichan,i) = 0.0
                            reg_uncr(ichan,i) = 0.0
                        enddo
                        reg_area_fwc(i) = 0.0
                        print *, 'missing file: ',spec_name
                        print *, 'PN_BACK POSITION 09'
                        stop
                    endif
                endif
            endif
            print *, '  '
        enddo
        if(diag .ge. 1) then
            print *, 'Backscale and augmented exposure from the ', &
                    'corners of the observation data'
            do i=1,4
                if(quad(i) .eq. 1) then
                    print *,'Quadrant ', i, ' backscale=', &
                            cor_area_obs(i)/amsca, &
                            ' exposure =', cor_exp_obs(i)            
                endif
            enddo
        endif

!  The fc data are the FWC data from the corners

        print *, '  '
        print *, 'Process the FWC corner data'
        do i=1,4
            if(quad(i) .eq. 1) then
                write(spec_name,3030) prefix, i
 3030           format('pn',a4,'-',i1,'fc.pi')
                call read_pha (spec_name,4096,nchan,channel,spec,uncr, &
                      qual,grup,expo,back,area,revl,obsiddum,mode,err)
                if(diag .ge. 1) print *, spec_name
                if(diag .ge. 2) print *, nchan,  &
                        expo, back, area, revl, err
                if(err .ne. 1) then 
                    cor_area_fwc(i) = back
                    cor_exp_fwc(i) = expo
                else
                    if((reg_area_fwc(i) .ne. 0.0) .or.  &
                            (cor_area_obs(i) .ne. 0)) then
                        errr=1
                        print *, 'PN_BACK POSITION 10'
                        print *, 'missing file: ',spec_name, &
                            ' in xmm-back-image'
                        stop
                    endif
                endif

!  Now do the OOT events

                write(spec_name,3031) prefix, i
 3031           format('pn',a4,'-',i1,'fc-oot.pi')
                call read_pha (spec_name,4096,nchan,channel, &
                        spec_oot,uncr_oot,qual,grup,expo_oot, &
                        back_oot,area,revl,obsiddum,mode,err)
                if(diag .ge. 1) print *, spec_name
                if(diag .ge. 2) print *, nchan, &
                        expo_oot, back_oot, area, revl, err
                if(err .ne. 1) then 

!  Subtract the OOT events from the spectrum and normalize the 
!  result to counts/s/unit area (arcmin pixels).

!                    print *, ootsca, amsca, back, expo, &
!                        back_oot, expo_oot

                    do ichan=1,nchan
                        fc_uncr(ichan,i) = amsca*sqrt(spec(ichan) +  &
                                ootsca*ootsca*spec_oot(ichan)* &
                                (back/back_oot)* &
                                (expo/expo_oot))/back/expo
                        fc_spec(ichan,i) =  &
                            amsca*(spec(ichan)/back/expo - &
                            ootsca*spec_oot(ichan)/back_oot/expo_oot)
                    enddo
                else
                    if((reg_area_fwc(i) .ne. 0.0) .or.  &
                            (cor_area_obs(i) .ne. 0)) then
                        errr = 1
                        print *, 'PN-BACK POSITION 11'
                        print *, 'missing file: ',spec_name, &
                            ' in xmm-back-image'
                        stop
                    endif
                endif
            endif
        enddo

        if(diag .ge. 1) then
            print *, 'Backscale and exposure from the corners ', &
                        'of the FWC data'
            do i=1,4
                if(quad(i) .eq. 1) then
                    print *,'Quadrant ', i, ' backscale=', &
                            cor_area_fwc(i)/amsca, &
                            ' exposure =', cor_exp_fwc(i)            
                endif
            enddo
        endif

!  Create the ratio of region/corner for FWC data
 
!        it = 30
!        do i=1,4
!            tt1 = 0.0
!            tt2 = 0.0
!            tt3 = 0.0
!            do ii=plc_hi(1),plc_hi(2)
!                tt1 = tt1 + reg_spec(ii,i)
!                tt2 = tt2 + fc_spec(ii,i)
!                tt3 = tt3 + oc_spec(ii,i)
!            enddo
!            print *, 'FWC FOV, FWC Cor, Obs Cor: ', tt1, tt2, tt3
!            it = it + 1
!            do ii=1,nchan
!                write(it,2927) ii, reg_spec(ii,i), fc_spec(ii,i), &
!                        oc_spec(ii,i), reg_spec(ii,i)/oc_spec(ii,i), &
!                        fc_spec(ii,i)/oc_spec(ii,i), &
!                        reg_spec(ii,i)/fc_spec(ii,i)
! 2927           format(1x,i5,3f12.8,3f10.2)
!            enddo
!        enddo

        do i=1,4
            if((reg_area_fwc(i) .ne. 0.0) .and. (quad(i) .eq. 1)) then
!
!  Set up for smoothing the data
!
                do ichan=1,nchan
                    i1 = ichan - smo_fact
                    i2 = ichan + smo_fact
                    if(i1 .lt. 1) i1 = 1
                    if(i2 .gt. nchan) i2 = nchan

!  Check to see if the channel (ichan) is in a bridge region,
!  and if so ignore

                    if(((ichan .ge. plc_alk(1)) .and. &
                            (ichan .le. plc_alk(2))) .or. &
                            ((ichan .ge. plc_hard(1)) .and. &
                            (ichan .le. plc_hard(2)))) then
                        temp_top(ichan) = 0.0
                        temp_bot(ichan) = 0.0
                        temp_oc(ichan) = 0.0
                    else 

!  Exclude data if they are in a bridge region - note that 
!  when the channel being smoothed is close to a bridge region
!  that there will be one-sided smoothing but this should not
!  be a significant effect as the values only show up as a ratio
!  where the effect should be very similar for both the numberator
!  and denominator.

                        if(i1 .lt. plc_alk(1)) then
                            if(i2 .gt. plc_alk(1)) i2 = plc_alk(1)
                        elseif(i1 .lt. plc_hard(1)) then
                            if(i1 .lt. plc_alk(2)) then
                                i1 = plc_alk(2)
                            elseif(i2 .gt. plc_hard(1)) then
                                i2 = plc_hard(1)
                            endif
                        else
                            if(i1 .lt. plc_hard(2)) i1 = plc_hard(2)
                        endif

!  The code used to check to make sure that bins had non-zero data
!  I don't see the reason as it would bias the data higher

                        t1 = 0.0
                        t2 = 0.0
                        t3 = 0.0
                        it = 0
                        do ii=i1,i2
                            t1 = t1 + reg_spec(ii,i)
                            t2 = t2 + fc_spec(ii,i)
                            t3 = t3 + oc_spec(ii,i)
                            it = it + 1
                        enddo
                        if(it .gt. 0) then 
                            temp_top(ichan) = t1/it
                            temp_bot(ichan) = t2/it
                            temp_oc(ichan) = t3/it
                        else
                            temp_top(ichan) = 0.0
                            temp_bot(ichan) = 0.0
                            temp_oc(ichan) = 0.0
                        endif
                    endif
                enddo
                do ichan=1,nchan
                    if(temp_bot(ichan) .ne. 0) then
                        reg_mask(ichan,i) = 1.0
                        reg_rati(ichan,i) = &
                                temp_top(ichan)/temp_bot(ichan) 
                    else 
                        reg_mask(ichan,i) = 0.0
                        reg_rati(ichan,i) = 0.0
                    endif
                enddo
            endif
        enddo
        if(diag .ge. 3) then
            print *, 'TEMP_TOP: ', temp_top
            print *, 'TEMP_BOT: ', temp_bot
        endif

!  Print out some diagnostic information

        if(diag .gt. 1) then
            print *, '  '
            print *, 'Backscale for the FWC corner ', &
                    'and FWC region data'
            do i=1,4
                print *,'Chip ', i, ' backscale=', &
                        cor_area_fwc(i)/amsca, &
                        ' backscale =', reg_area_fwc(i)/amsca
            enddo
        endif

        t1 = 0.0
        t2 = 0.0
        do i=1,4
            t1 = t1 + cor_area_fwc(i)
            t2 = t2 + reg_area_fwc(i)
        enddo
        if(diag .ge. 1) then
            print *, 'FWC Region Area, FWC Corner Area, Corner ', &
                    'Fraction: '
            print *, t1/amsca, t2/amsca, t1/(t1 + t2)
        endif

!  Create the masks to remove strong instrumental lines
!  Remove the Al line from all spectra

        do ichan=plc_alk(1),plc_alk(2)
            do i=1,4
                oc_mask(ichan,i) = 0
            enddo
        enddo
        do ichan=plc_hard(1),plc_hard(2)
            do i=1,4
                oc_mask(ichan,i) = 0
            enddo
        enddo

!  Sum the background over all active chips and make an image

        if(diag .ge. 2) then
            write(qdp_file,3039) prefix
 3039       format('pn',a4,'-quad-spec.qdp')
            open(89,form='formatted',status='unknown',file=qdp_file)
            write(89,*) 'read serr 2 3 4 5'
            write(89,*) 'font roman'
            write(89,*) 'cs 1.5'
            write(89,*) 'log y'
        endif

        do ichan=1,4096
             spec(ichan) = 0.0
             uncr(ichan) = 0.0
        enddo

        do i=1,4

!  Loop over the four quadrants

            if((reg_area_fwc(i) .ne. 0) .and. (quad(i) .eq. 1)) then

!  Loop over the spectral energy bins and add to the count spectrum
!  and the spectrum uncertainty for the entire region.

                do ichan=1,nchan
                    if((reg_mask(ichan,i) .ne. 0.0) .and. &
                            (oc_mask(ichan,i) .ne. 0.0)) then
                        tempsp = oc_spec(ichan,i)* &
                                reg_rati(ichan,i)* &
                                reg_exp_obs(i)* &
                                reg_area_obs(i)/amsca
                        tempunc = oc_uncr(ichan,i)* &
                                reg_rati(ichan,i)* &
                                reg_exp_obs(i)* &
                                reg_area_obs(i)/amsca
                        spec(ichan) = spec(ichan) + tempsp
                        uncr(ichan) = uncr(ichan) + tempunc**2
                        mask(ichan) = mask(ichan) + reg_area_fwc(i)

                        if(diag .ge. 2) then
                            if(i .eq. 1) then
                                write(89,*) ichan, tempsp, tempunc,  &
                                      '  0.0 0.0 0.0 0.0 0.0 0.0'
                            elseif(i .eq. 2) then
                                write(89,*) ichan, '  0.0 0.0',  &
                                      0.1*tempsp, 0.1*tempunc, &
                                      '  0.0 0.0 0.0 0.0'
                            elseif(i .eq. 3) then
                                write(89,*) ichan, '  0.0 0.0 0.0 0.0', & 
                                      0.01*tempsp, &
                                      0.01*tempunc, '  0.0 0.0'
                            elseif(i .eq. 4) then
                                write(89,*) ichan, '  0.0 0.0 0.0 0.0', &
                                      ' 0.0 0.0', 0.001*tempsp, &
                                      0.001*tempunc
                            endif
                        endif
                    endif
                enddo 
                tmodcounts = 0.0
                tot1 = 0.0
                tot2 = 0.0

!  Get the number of counts in the band

                do ichan=band(1),band(2)
                    tmodcounts = tmodcounts + oc_spec(ichan,i)* &
                        reg_rati(ichan,i)*reg_exp_obs(i)* &
                        reg_area_obs(i)/amsca
                    tot1 = tot1 + oc_spec(ichan,i)* &
                        reg_rati(ichan,i)*reg_exp_obs(i)* &
                        reg_area_obs(i)/amsca
                    tot2 = tot2 + reg_rati(ichan,i)
                enddo 
                if(diag .ge. 1) then
                    print *, ' '
                    print *, 'Chip Band-Counts Ratio  FWC-Backscale ', &
                            'Obs-Backscale Exposure:'
                    print *, i, tot1, tot2, reg_area_fwc(i)/amsca, &
                            reg_area_obs(i)/amsca, reg_exp_obs(i)
                endif

!  Read in the fwc image

                if(eh .gt. 0) then
                    if(el .ge. 10000) then
                        write(im_name,3040) prefix, i, el, eh
 3040                   format('pn',a4,'-im',i1,'-',i5,'-',i5,'.fits')
                        write(im_name_oot,3041) prefix, i, el, eh
 3041                   format('pn',a4,'-im',i1,'-',i5,'-',i5, &
                                '-oot.fits')
                    elseif(el .ge. 1000) then
                        if(eh .ge. 10000) then
                            write(im_name,3042) prefix, i, el, eh
 3042                       format('pn',a4,'-im',i1,'-',i4,'-', &
                                    i5,'.fits')
                            write(im_name_oot,3043) prefix, i, el, eh
 3043                       format('pn',a4,'-im',i1,'-',i4,'-', &
                                    i5,'-oot.fits')
                        else
                            write(im_name,3044) prefix, i, el, eh
 3044                       format('pn',a4,'-im',i1,'-',i4,'-', &
                                    i4,'.fits')
                            write(im_name_oot,3045) prefix, i, el, eh
 3045                       format('pn',a4,'-im',i1,'-',i4,'-', &
                                    i4,'-oot.fits')
                        endif
                    else
                        if(eh .ge. 10000) then
                            write(im_name,3046) prefix, i, el, eh 
 3046                       format('pn',a4,'-im',i1,'-',i3,'-', &
                                    i5,'.fits')
                            write(im_name_oot,3047) prefix, i, el, eh 
 3047                       format('pn',a4,'-im',i1,'-',i3,'-', &
                                    i5,'-oot.fits')
                        elseif(eh .ge. 1000) then
                            write(im_name,3048) prefix, i, el, eh 
 3048                       format('pn',a4,'-im',i1,'-',i3,'-', &
                                    i4,'.fits')
                            write(im_name_oot,3049) prefix, i, el, eh 
 3049                       format('pn',a4,'-im',i1,'-',i3,'-', &
                                    i4,'-oot.fits')
                        else
                            write(im_name,3050) prefix, i, el, eh 
 3050                       format('pn',a4,'-im',i1,'-',i3,'-', &
                                    i3,'.fits')
                            write(im_name_oot,3051) prefix, i, el, eh 
 3051                       format('pn',a4,'-im',i1,'-',i3,'-', &
                                    i3,'-oot.fits')
                        endif
                    endif
                    print *, im_name
                    call retfit(im_name,image,ini4, &
                        1,780,780,ctype,crval,cdelt,crpix,status)
                    print *, im_name_oot
                    call retfit(im_name_oot,image_oot,ini4, &
                        1,780,780,ctype,crval,cdelt,crpix,status)
                    if(status .eq. 0) then

!  Check for bright pixels

                        tot = 0.0
                        ntot = 0.0
                        do j=1,780
                            do jj=1,780
                                if(image(j,jj) .gt. 0) then
                                    image(j,jj) = image(j,jj) - &
                                            ootsca*image_oot(j,jj)
                                    tot = tot + image(j,jj)
                                    ntot = ntot + 1.0
                                endif
                            enddo
                        enddo
                        avtot = tot/ntot
                        if(diag .gt. 1) then
                            print *, 'Average non-zero value of FWC ', &
                                    'image: ', avtot
                        endif

!  Reset bright pixels (5x average) to the average value

                        tot = 5.0*avtot
                        do j=1,780
                            do jj=1,780
                                if(image(j,jj) .gt. tot) then
                                    image(j,jj) = avtot
                                endif
                            enddo
                        enddo 

                        tot = 0.0
                        do j=1,780
                            do jj=1,780
                                tot = tot + image(j,jj)
                            enddo
                        enddo
                        if(diag .gt. 1) then
                            print *, 'FWC image scaling for chip: '
                            print *, 'Chip Obs-Corner-Band-Counts ', &
                                    'FWC-Corner-Band-Counts Ratio:'
                            print *, i, tmodcounts, tot, tmodcounts/tot
                        endif
                        do j=1,780
                            do jj=1,780
                                outim(j,jj) = outim(j,jj) +  &
                                        image(j,jj)*tmodcounts/tot
                            enddo
                        enddo
                    endif
                endif
            endif
            if(diag .ge. 2) then
                write(89,*) 'no no no'
            endif
        enddo 
        do ichan=1,nchan
            uncr(ichan) = sqrt(uncr(ichan))
        enddo 
        if(diag .ge. 2) close(89)

        if(diag .ge. 1) then
            print *, 'Exposure for the FWC and observation data'
            do i=1,4
                if(quad(i) .eq. 1) then
                    print *,'Chip ', i, ' exposure=', reg_exp_fwc(i), &
                            ' exposure =', reg_exp_obs(i)            
                endif
            enddo
        endif

!  Search for and eliminate hot pixels

        if(eh .gt. 0) then
            if(ipix .eq. 1) then
                tot1 = 0.0
                tot2 = 0.0
                do i=1,780
                    do ii=1,780
                        if(outim(i,ii) .gt. 0.0) then
                            tot1 = tot1 + outim(i,ii)
                        tot2 = tot2 + 1.0
                        endif
                    enddo
                enddo
                tot1 = 2.0*tot1/tot2
                do i=1,780
                    do ii=1,780
                        if(outim(i,ii) .gt. tot1) then
                            outim(i,ii) = tot1
                        endif
                    enddo
                enddo
                tot1 = 0.0
                tot2 = 0.0
            endif        
        endif        

!  Write out some more diagnostic information

        if(diag .ge. 2) then
            write(qdp_file,3059) prefix
 3059       format('pn',a4,'-back-spec.qdp')
            open(89,form='formatted',status='unknown',file=qdp_file)
            write(89,*) 'read serr 2'
            write(89,*) 'font roman'
            write(89,*) 'cs 1.5'
            write(89,*) 'log y'
            do ichan=1,nchan
                if(mask(ichan) .ne. 0.0) then
                    write(89,*) ichan, spec(ichan), uncr(ichan)
                else
                    spec(ichan) = 0.0
                    uncr(ichan) = 0.0
                endif
            enddo
            if(diag .ge. 2) close(89)
        endif

!  Read in the observation images (normal and OOT) in detector 
!  coordinates

!  First create the file name

        if(eh .gt. 0) then
            if(el .ge. 10000) then
                write(im_name,3060) prefix, el, eh
 3060           format('pn',a4,'-obj-im-det-',i5,'-',i5,'.fits')
                write(im_name_oot,3061) prefix, el, eh
 3061           format('pn',a4,'-obj-im-det-',i5,'-',i5,'-oot.fits')
            elseif(el .ge. 1000) then
                if(eh .ge. 10000) then
                    write(im_name,3062) prefix, el, eh
 3062               format('pn',a4,'-obj-im-det-',i4,'-',i5,'.fits')
                    write(im_name_oot,3063) prefix, el, eh
 3063               format('pn',a4,'-obj-im-det-',i4,'-',i5,'-oot.fits')
                else
                    write(im_name,3064) prefix, el, eh
 3064               format('pn',a4,'-obj-im-det-',i4,'-',i4,'.fits')
                    write(im_name_oot,3065) prefix, el, eh
 3065               format('pn',a4,'-obj-im-det-',i4,'-',i4,'-oot.fits')
                endif
            else
                if(eh .ge. 10000) then
                    write(im_name,3066) prefix, el, eh 
 3066               format('pn',a4,'-obj-im-det-',i3,'-',i5,'.fits')
                    write(im_name_oot,3067) prefix, el, eh 
 3067               format('pn',a4,'-obj-im-det-',i3,'-',i5,'-oot.fits')
                elseif(eh .ge. 1000) then
                    write(im_name,3068) prefix, el, eh 
 3068               format('pn',a4,'-obj-im-det-',i3,'-',i4,'.fits')
                    write(im_name_oot,3069) prefix, el, eh 
 3069               format('pn',a4,'-obj-im-det-',i3,'-',i4,'-oot.fits')
                else
                    write(im_name,3070) prefix, el, eh 
 3070               format('pn',a4,'-obj-im-det-',i3,'-',i3,'.fits')
                    write(im_name_oot,3071) prefix, el, eh 
 3071               format('pn',a4,'-obj-im-det-',i3,'-',i3,'-oot.fits')
                endif
            endif
 
!  Get an unused lun

            call ftgiou(luna,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTGIOU =', status
                print *, 'PN-BACK POSITION 12'
            endif

            call retfit(im_name,image,ini4, &
                    1,780,780,ctype,crval,cdelt,crpix,status)
            if(status .ne. 0) then
                print *, 'Cant find observation image in detector ', &
                        'coords'
                print *, im_name
                stop
            endif

            call retfit(im_name_oot,image_oot,ini4, &
                    1,780,780,ctype,crval,cdelt,crpix,status)
            if(status .ne. 0) then
                print *, 'PN-BACK POSITION 13'
                print *, 'Cant find observation image in detector ', &
                        'coords'
                print *, im_name_oot
                stop
            endif

            tot0 = 0.0
            tot1 = 0.0
            tot2 = 0.0
            tot3 = 0.0
            do j=1,780
                do jj=1,780
                    tot0 = tot0 + image(j,jj)
                    image(j,jj) = image(j,jj) - ootsca*image_oot(j,jj)
                    tot1 = tot1 + image(j,jj)
                    tot2 = tot2 + outim(j,jj)
                    tot3 = tot3 + ootsca*image_oot(j,jj)
                enddo
            enddo
            if(diag .ge. 1) then
                print *,'Total observation counts in band ',tot0
                print *,'Total OOT counts in band ',tot3
                print *,'Total OOT subtracted counts in band ',tot1
                print *,'Total background counts in band ',tot2
            endif

!  Write out the background image

!  First get an unused lun

            call ftgiou(luna,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTGIOU =', status
                print *, 'PN-BACK POSITION 14'
            endif
            call ftgiou(lunb,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTGIOU =', status
                print *, 'PN-BACK POSITION 15'
            endif
            call ftgiou(lunc,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTGIOU =', status
                print *, 'PN-BACK POSITION 16'
            endif

!  Create the output background file name

            if(el .ge. 10000) then
                write(imo_name,3080) prefix, el, eh
 3080           format('pn',a4,'-back-im-det-',i5,'-',i5,'.fits')
            elseif(el .ge. 1000) then
                if(eh .ge. 10000) then
                    write(imo_name,3081) prefix, el, eh
 3081               format('pn',a4,'-back-im-det-',i4,'-',i5,'.fits')
                else
                    write(imo_name,3082) prefix, el, eh
 3082               format('pn',a4,'-back-im-det-',i4,'-',i4,'.fits')
                endif
            else
                if(eh .ge. 10000) then
                    write(imo_name,3083) prefix, el, eh 
 3083               format('pn',a4,'-back-im-det-',i3,'-',i5,'.fits')
                elseif(eh .ge. 1000) then
                    write(imo_name,3084) prefix, el, eh 
 3084               format('pn',a4,'-back-im-det-',i3,'-',i4,'.fits')
                else
                    write(imo_name,3085) prefix, el, eh 
 3085               format('pn',a4,'-back-im-det-',i3,'-',i3,'.fits')
                endif
            endif
            print *, 'Output background image: ', imo_name

!  Open the old image FITS file

            call ftopen (luna,im_name,0,block,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTOPEN =', status
                print *, 'PN-BACK POSITION 17'
                print *, im_name
            endif

!  Check to see if there is an existing file

            call ftopen (lunb,imo_name,0,block,status)
            if(status .eq.  0) then
                print *, 'File exists with output file name'
                print *, imo_name
                if(clobber) then
                    status = 0
                    print *, 'Clobber the file and continue'
                    call ftdelt(lunb,status)
                else
                    print *, 'PN-BACK POSITION 18'
                    print *, 'Will not clobber file'
                    stop
                endif
            endif
            status = 0

!  Create the output FITS file using the original file as a template

            call ftinit (lunb,imo_name,2880,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTINIT =', status
                print *, 'PN-BACK POSITION 19'
                print *, imo_name
            endif

            call ftcphd (luna,lunb,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTCPHD =', status
                print *, 'PN-BACK POSITION 20'
            endif

            bitpix = -32
            naxis = 2
            naxes(1) = 780
            naxes(2) = 780
            pcount = 0
            gcount = 1
            group = 1
            call ftpdef (lunb,bitpix,naxis,naxes,pcount, &
                    gcount,.FALSE.,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTPDEF =', status
                print *, 'PN-BACK POSITION 21'
            endif
            call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTMKYJ =', status
                print *, 'PN-BACK POSITION 22'
            endif

!  Add some keywords

            call ftpkyf(lunb,'BZERO',0.0,4,' ',status)
            call ftpkyf(lunb,'BSCALE',1.0,8,' ',status)
            call ftpkys(lunb,'BUNIT','counts', &
                    'Units of data, model background counts',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTPKYF or FTPKYS =', status
                print *, 'PN-BACK POSITION 22a'
                print *, imo_name
            endif
            if(err .eq. 0) then

!  Write the output image

                call ftp2de (lunb,group,naxes(1),naxes(1),naxes(2), &
                        outim,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'STATUS FTP2DE =', status
                    print *, 'PN-BACK POSITION 23'
                    print *, 'Could not write the background image'
                endif

!  Close the output file

                call ftclos(lunb,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'STATUS FTCLOS =', status
                    print *, 'PN-BACK POSITION 24'
                    print *, 'Could not close the output file'
                endif
            endif
            call ftfiou (luna,status)
            call ftfiou (lunb,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTFIOU =', status
                print *, 'PN-BACK POSITION 25'
                print *, 'Could not close the output file'
            endif
        endif

!  Bridge the Al and high-energy gaps
!    Set a number of line-free windows
!    Extract the data from those windows
!    Fit a simple polynomial to the log of the data
!    Use the result to interpolate over the Al gap

        n = 0
        do i=1,nels
            do ichan=mine(i),maxe(i)
                if((ener(ichan) .gt. 0.1) .and.  &
                        (spec(ichan) .gt. 0.0)) then
                    n = n + 1
                    tempe(n) = alog(ener(ichan))
                    temps(n) = alog(spec(ichan))
                    tempu(n) = 0.2
                endif
            enddo
        enddo

        nord = 5
        call poly_fit(tempe,temps,tempu,n,nord,coef)
        if(diag .ge. 2) then
            write(qdp_file,3089) prefix
 3089       format('pn',a4,'-bridge-fit.qdp')
            open(89,form='formatted',status='unknown', &
                    file=qdp_file)
            write(89,*) 'read serr 2'
            write(89,*) 'font roman'
            write(89,*) 'cs 1.5'
        endif
        do ichan=1,nchan 
            do i=0,nord
                modl(ichan) = modl(ichan) + &
                        coef(i+1)*(alog(ener(ichan))**i)
            enddo
            if((diag .ge. 2) .and. (ener(ichan) .gt. 0.1)) then
                write(89,*) alog(ener(ichan)), alog(spec(ichan)), &
                    0.05*(alog(spec(ichan))), modl(ichan)
            endif
        enddo

        if(diag .ge. 2) close(89)

!  Finish the bridge

        do ichan = plc_alk(1),plc_alk(2)
            fillin(ichan) = exp(modl(ichan))
            spec(ichan) = spec(ichan) + fillin(ichan)
            uncr(ichan) = uncr(ichan) + fillin(ichan)/20.0
        enddo
        do ichan = plc_hard(1),plc_hard(2)
            fillin(ichan) = exp(modl(ichan))
            spec(ichan) = spec(ichan) + fillin(ichan)
            uncr(ichan) = uncr(ichan) + fillin(ichan)/20.0
        enddo
        ymin = 1.e8
        ymax = 0
        
        do ichan = 7,nchan
            if((ener(ichan) .gt. 0.2) .and. &
                    (ener(ichan) .lt. 16.0)) then
                if (obj_cnts(ichan)-stat(ichan) .lt. ymin) &
                        ymin = obj_cnts(ichan)-stat(ichan)
                if (obj_cnts(ichan)+stat(ichan) .gt. ymax) &
                        ymax = obj_cnts(ichan)+stat(ichan)
                if (spec(ichan)-uncr(ichan) .lt. ymin) ymin = &
                        spec(ichan) - uncr(ichan)
                if (spec(ichan)+uncr(ichan) .gt. ymax) ymax = &
                        spec(ichan) + uncr(ichan)
            endif
        enddo

        write(spec_file,3090) prefix
 3090   format('pn',a4,'-spec.qdp')
        open(89,form='formatted',status='unknown',file=spec_file)
        write(89,*) 'viewport 0.2 0.2 0.9 0.85'
        write(89,*) 'la f'
        write(title,3110) obsid, prefix
 3110   format('"',i10,' -- PN',a4,'"')
        write(89,*) 'la t ', title
        write(89,*) 'la x Energy (keV)'
        write(89,*) 'la y Counts'
        write(89,*) 'la 1 p 3 ', 0.5*ymax, ' just left ', &
               '"Observed (red), Particle Background (green)"'
        write(89,*) 'log y'
        write(89,*) 'r x 0 12'
        write(89,*) 'r y ', ymin, ymax
        write(89,*) 'read serr 2'
        write(89,*) 'read serr 3'
        write(89,*) 'read serr'
        write(89,*) 'font roman'
        write(89,*) 'cs 1.8'
        do ichan = 1,nchan
            write(89,*) ener(ichan), obj_cnts(ichan), stat(ichan), &
                        spec(ichan), uncr(ichan)
        enddo
        close(89)

!  Get three unused luns

        call ftgiou(luna,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'PN-BACK POSITION 26'
        endif
        call ftgiou(lunb,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'PN-BACK POSITION 27'
        endif
        call ftgiou(lunc,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'PN-BACK POSITION 28'
        endif

!  Open the spectrum FITS file

        write(obj_file,2991) prefix
        call ftopen (luna,obj_file,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTOPEN =', status
            print *, 'PN-BACK POSITION 29'
            print *, spec_name
        endif

!  Create the output background FITS file using the original spectrum 
!  file as a template

        write(back_file,3100) prefix
 3100   format('pn',a4,'-back.pi')

!  Check to see if there is an existing file

        call ftopen (lunb,back_file,0,block,status)
        if(status .eq.  0) then
            print *, 'File exists with output file name'
            print *, back_file
            if(clobber) then
                status = 0
                print *, 'Clobber the file and continue'
                call ftdelt(lunb,status)
            else
                print *, 'PN-BACK POSITION 18'
                print *, 'Will not clobber file'
                stop
            endif
        endif
        status = 0

        call fttplt (lunb,back_file,obj_file,status)
        if(diag .ge. 1) then
            print *, 'Create background spectral file using the '
            print *, 'observation spectral file as a template'
            print *, 'Observation spectral file name'
            print *, obj_file
            print *, 'Background spectral file name'
            print *, back_file
        endif
        if((status .ne. 0) .and. (err .eq. 0)) then
            if(status .eq. 105) then
                print *, 'The file already exists ', back_file
                stop
            else
                err = 1
                print *, 'STATUS FTINIT =', status
                print *, 'PN-BACK POSITION 30'
            endif
        endif

        if(err .eq. 0) then

!  Move to the input file spectrum extension

            call ftmahd (lunb,2,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTMHD =', status
                print *, 'PN-BACK POSITION 31'
                print *, 'Could not find the spectrum extension'
            endif

!  Check to make sure that we have the spectrum extension

            call ftghbn (lunb,10,nchan,tfields,ttype,tform, &
                tunit,extname,varidat,status)
            if(extname .ne. 'SPECTRUM') then
                print *, 'PN-BACK POSITION 32'
                print *, extname
                print *, 'This was supposed to be a spectrum ext'
                stop
            endif

!  Get rid of extra extensions
!  Move to the next extension

            call ftmrhd (lunb,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTMHD =', status
                print *, 'PN-BACK POSITION 33'
                print *, 'No more extensions'
                status = 0

!  Move to the input file spectrum extension

                call ftmahd (lunb,2,hdutype,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'STATUS FTMHD =', status
                    print *, 'PN-BACK POSITION 34'
                    print *, 'Could not find the spectrum extension'
                endif
            endif

!  Check to make sure that we have the spectrum extension

            call ftghbn (lunb,10,nchan,tfields,ttype,tform, &
                tunit,extname,varidat,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTGHBN =', status
                print *, 'PN-BACK POSITION 35'
                print *, 'Could not read extension info'
            endif

            if(extname .ne. 'SPECTRUM') then
                do while (extname .ne. 'SPECTRUM')
                    call ftdhdu (lunb,hdutype,status)
                    if((status .ne. 0) .and. (err .eq. 0)) then
                        err = 1
                        print *, 'STATUS FTDHDU =', status
                        print *, 'PN-BACK POSITION 36'
                        print *, 'Could not delete an extension'
                    endif
                    call ftghbn (lunb,10,nchan,tfields,ttype,tform, &
                        tunit,extname,varidat,status)
!                    print *, lunb,nchan,tfields,ttype,tform, &
!                        tunit,extname,varidat,status,err
                    if((status .ne. 0) .and. (err .eq. 0)) then
                        err = 1
                        print *, 'STATUS FTGHBN =', status
                        print *, 'PN-BACK POSITION 37'
                        print *, 'Could not read extension info'
                    endif
                enddo
            endif

!  Write the output spectrum

            call ftukye(lunb,'EXPOSURE',obj_expo,4, &
                    'Background exposure',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTUKYE =', status
                print *, 'PN-BACK POSITION 38'
                print *, 'Could not write the exposure keyword'
            endif
            call ftukye(lunb,'BACKSCAL',obj_back,4, &
                    'Scaling factor for background',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTUKYE =', status
                print *, 'PN-BACK POSITION 39'
                print *, 'Could not write the backscale keyword'
            endif

!  Delete the existing column information

            call ftdcol(lunb,2,status)
            call ftdcol(lunb,1,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTUKYE =', status
                print *, 'PN-BACK POSITION 40'
                print *, 'Could not delete the existing column info'
            endif

!  Create the new columns

            call fticol(lunb,1,'CHANNEL','1J',status)
            call fticol(lunb,2,'RATE','1E',status)
            call fticol(lunb,3,'STAT_ERR','1E',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTUKYE =', status
                print *, 'PN-BACK POSITION 41'
                print *, 'Could not the new columns'
            endif

!  Update a few keywords

            call ftukys(lunb,'TUNIT1','channel', &
                    'physical unit of field',status)
            call ftukys(lunb,'TUNIT2','count/s', &
                    'physical unit of field',status)
            call ftukys(lunb,'TUNIT3','count', &
                    'physical unit of field',status)
            call ftukyl(lunb,'POISSERR',.FALSE., &
                    'physical unit of field',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTUKYE =', status
                print *, 'PN-BACK POSITION 42'
                print *, 'Could not write the various keywords'
            endif

!  Prepare the spectrum data

            do ichan=1,nchan
                chan(ichan) = ichan - 1
                spec(ichan) = spec(ichan)/obj_expo
                uncr(ichan) = uncr(ichan)/obj_expo
            enddo

!  Write the column data

            call ftpclj(lunb,1,1,1,nchan,chan,status)
            call ftpcle(lunb,2,1,1,nchan,spec,status)
            call ftpcle(lunb,3,1,1,nchan,uncr,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTUKYE =', status
                print *, 'PN-BACK POSITION 43'
                print *, 'Could not write the column information'
            endif

!  Close the output file

            call ftclos(lunb,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTUKYE =', status
                print *, 'PN-BACK POSITION 44'
                print *, 'Could not close the output file'
            endif
        endif

!  Create the output OOT-subtracted spectrum FITS file using the 
!  original spectrum file as a template

        write(specos_file,3101) prefix
 3101   format('pn',a4,'-obj-os.pi')
        if(diag .ge. 1) then
            print *, ' Create OOT subtracted spectral file using the'
            print *, ' observation spectral file as a template'
            print *, ' Observation spectral file name'
            print *, obj_file
            print *, ' Background  OOT spectral file name'
            print *, specos_file
        endif

!  Check to see if there is an existing file

        call ftopen (lunc,specos_file,0,block,status)
        if(status .eq.  0) then
            print *, 'File exists with output file name'
            print *, specos_file
            if(clobber) then
                status = 0
                print *, 'Clobber the file and continue'
                call ftdelt(lunc,status)
            else
                print *, 'PN-BACK POSITION 18'
                print *, 'Will not clobber file'
                stop
            endif
        endif
        status = 0

        call fttplt (lunc,specos_file,obj_file,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            if(status .eq. 105) then
                print *, 'PN-BACK POSITION 45'
                print *, 'The file already exists ', specos_file
                stop
            else
                err = 1
                print *, 'PN-BACK POSITION 46'
                print *, 'STATUS FTINIT =', status
            endif
        endif

        if(err .eq. 0) then

!  Move to the input file spectrum extension

            call ftmahd (lunc,2,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 47'
                print *, 'STATUS FTMHD =', status
                print *, 'Could not find the spectrum extension'
            endif

!  Check to make sure that we have the spectrum extension

            call ftghbn (lunc,10,nchan,tfields,ttype,tform, &
                tunit,extname,varidat,status)
            if(extname .ne. 'SPECTRUM') then
                print *, 'PN-BACK POSITION 48'
                print *, extname
                print *, 'This was supposed to be a spectrum ext'
                stop
            endif

!  Get rid of extra extensions
!  Move to the next extension

            call ftmrhd (lunc,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 49'
                print *, 'STATUS FTMHD =', status
                print *, 'No more extensions'
                status = 0

!  Move to the input file spectrum extension

                call ftmahd (lunc,2,hdutype,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'PN-BACK POSITION 50'
                    print *, 'STATUS FTMHD =', status
                    print *, 'Could not find the spectrum extension'
                endif
            endif

!  Check to make sure that we have the spectrum extension

            call ftghbn (lunc,10,nchan,tfields,ttype,tform, &
                tunit,extname,varidat,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 51'
                print *, 'STATUS FTGHBN =', status
                print *, 'Could not read extension info'
            endif

            if(extname .ne. 'SPECTRUM') then
                do while (extname .ne. 'SPECTRUM')
                    call ftdhdu (lunc,hdutype,status)
                    if((status .ne. 0) .and. (err .eq. 0)) then
                        err = 1
                        print *, 'PN-BACK POSITION 52'
                        print *, 'STATUS FTDHDU =', status
                        print *, 'Could not delete an extension'
                    endif
                    call ftghbn (lunc,10,nchan,tfields,ttype,tform, &
                        tunit,extname,varidat,status)
!                    print *, lunc, nchan, tfields, ttype, tform, &
!     +                  tunit, extname, varidat, status, err
                    if((status .ne. 0) .and. (err .eq. 0)) then
                        err = 1
                        print *, 'PN-BACK POSITION 53'
                        print *, 'STATUS FTGHBN =', status
                        print *, 'Could not read extension info'
                    endif
                enddo
            endif

!  Write the output spectrum

            call ftukye(lunc,'EXPOSURE',obj_expo,4, &
                    'Observsation exposure',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 54'
                print *, 'STATUS FTUKYE =', status
                print *, 'Could not write the exposure keyword'
            endif
            call ftukye(lunc,'BACKSCAL',obj_back,4, &
                    'Scaling factor for the spectrum',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 55'
                print *, 'STATUS FTUKYE =', status
                print *, 'Could not write the backscale keyword'
            endif

!  Delete the existing column information

            call ftdcol(lunc,2,status)
            call ftdcol(lunc,1,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 56'
                print *, 'STATUS FTUKYE =', status
                print *, 'Could not delete the existing column info'
            endif

!  Create the new columns

            call fticol(lunc,1,'CHANNEL','1J',status)
            call fticol(lunc,2,'RATE','1E',status)
            call fticol(lunc,3,'STAT_ERR','1E',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 57'
                print *, 'STATUS FTUKYE =', status
                print *, 'Could not the new columns'
            endif

!  Update a few keywords

            call ftukys(lunc,'TUNIT1','channel', &
                    'physical unit of field',status)
            call ftukys(lunc,'TUNIT2','count/s', &
                    'physical unit of field',status)
            call ftukys(lunc,'TUNIT3','count/s', &
                    'physical unit of field',status)
            call ftukyl(lunc,'POISSERR',.FALSE., &
                    'physical unit of field',status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 58'
                print *, 'STATUS FTUKY =', status
                print *, 'Could not write the various keywords'
            endif

!  Convert spectrum counts to count rate.  This gets past
!  the grppha truncation of a real pn oot subtracted count
!  spectrum to integers.

            do ichan=1,nchan
                chan(ichan) = ichan - 1
                obj_cnts(ichan) = obj_cnts(ichan)/obj_expo
                stat(ichan) = stat(ichan)/obj_expo
            enddo

!  Write the column data

            call ftpclj(lunc,1,1,1,nchan,chan,status)
            call ftpcle(lunc,2,1,1,nchan,obj_cnts,status)
            call ftpcle(lunc,3,1,1,nchan,stat,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 59'
                print *, 'STATUS FTPCL =', status
                print *, 'Could not write the column information'
            endif

!  Close the output file

            call ftclos(lunc,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PN-BACK POSITION 60'
                print *, 'STATUS FTCLOS =', status
                print *, 'Could not close the output file'
            endif

        endif

!  Close the input lun

        call ftclos(luna,status)
        call ftfiou (luna,status)
        call ftfiou (lunb,status)
        call ftfiou (lunc,status)

        return

    end subroutine pn_back

end module pn_back_mod
