!+
! Module: point_source_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: point_source_mod.f90,v 1.1 2009/02/06 14:10:35 aibarra Exp $
!-
!
!  Author: K. D. Kuntz
!
! Calculates the xspec normalization per square arcminute for the
! Cosmic X-ray background after removal of point sources to some
! limiting level (min_flux)
! ELF is shorthand for the logN-logS
!
! We assume that integral[ELF] from 0 to infinity is less
! than cxrb_norm, or that
!               CXRB = X + integral[ELF]_0^infinity.
! Therefore, for any given 'blank sky' observations where
! the brightest point source has a flux smax, the total
! X-ray emission in the field would be
!                       X + integral[ELF]_0^smax
!
! Inputs:
!    func: Name of logN-logS function
!        current choices:
!            hms: Hasinger, Miyaji, & Schmidt (2005), from ROSAT, 
!                 XMM, & Chandra)'
!            mushotzky: (????)'
!            cappelluti: Cappelluti et al (2008), from COSMOS'
!            mateos: Mateos et al (2008), from XMM'
!    min_src_flux: Source flux cutoff in erg/cm2/s
!    cxrb_norm: Normalization of the cosmic X-ray background in units 
!            of photons/cm2/s/sr/keV at 1 keV.  The use of the Deluca 
!            and molendi value of 11.6 for the integral of everything
!            is a good match to our value for the RASS which was 9.5 
!            for s<4.8e-14 erg/cm2/s (i.e. CFG)
!    index: Photon power law index for the cosmic X-ray background,
!            optional parameter for index of unresolved spectrum
!            defaults to 1.46
!
!  Output:
!     Xspec normalization for power law in units of
!            photons/cm2/s/am2/keV
!
!  Examples:
!     point-source func=mateos min_src_flux=1.e-14 cxrb_norm=10.6 
!     point-source func=mateos min_src_flux=5.e-14 cxrb_norm=10.6 index=1.40

module point_source_mod

    use dal
    use param
    use errorHandling

    implicit none

    private

    public point_source

contains

  !---------------------------------------------------------------------
  subroutine point_source
  !---------------------------------------------------------------------
        character (len=10) :: func
        real :: aflg, bias, cxrb_norm, degree_per_ster, erg_per_ev, &
            flux, index, min_flux, min_src_flux, numb, tot_flux, &
            vari, xnorm

        func = stringParameter("func")
        min_src_flux = realParameter("min_src_flux")
        cxrb_norm = realParameter("cxrb_norm")
        index = realParameter("index")

        degree_per_ster = 3282.81
        erg_per_ev = 1.60184e-12

        print *, 'func: ', func
        print *, 'min_src_flux: ', min_src_flux
        print *, 'cxrb_norm: ', cxrb_norm

! calculate the integral from 0 to infinity
! as well as the integral from 0 to min_flux
! the output of the xxxx_elf function is numb/deg2,
!                0.5-2.0 keV flux/deg2, variance in flux

        if(func .eq. 'mateos') then
            call mate_elf(0.0,1e-32,1000.,numb,tot_flux,vari)
            call mate_elf(0.0,1e-32,min_src_flux,numb,min_flux,vari)
        elseif(func .eq. 'cappelluti') then
            call capp_elf(0.0,1e-32,1000.,numb,tot_flux,vari)
            call capp_elf(0.0,1e-32,min_src_flux,numb,min_flux,vari)
        elseif(func .eq. 'mushotzky') then
            call mush_elf(0.0,1e-32,1000.,numb,tot_flux,vari)
            call mush_elf(0.0,1e-32,min_src_flux,numb,min_flux,vari)
        elseif(func .eq. 'hms') then
            call mush_elf(0.0,1e-32,1000.,numb,tot_flux,vari)
            call mush_elf(0.0,1e-32,min_src_flux,numb,min_flux,vari)
        else
            call mate_elf(0.0,1e-32,1000.,numb,tot_flux,vari)
            call mate_elf(0.0,1e-32,min_src_flux,numb,min_flux,vari)
        endif

! print,'T,M (erg/cm2/s/deg2)',tot_flux,min_flux

! convert values from ergs/cm2/s/deg2 to ergs/cm2/s/sr

        tot_flux = tot_flux*degree_per_ster
        min_flux = min_flux*degree_per_ster

! print,'T,M (erg/cm2/s/sr)',tot_flux,min_flux

! determine the conversion from energy in 0.5-2.0 to normalization

! calculate the energy in 0.5-2.0 in keV
! for a powerlaw with a normalization of 1.0

        if(index .ne. 2.0) then
            flux = (2.0**(2-index)-0.5**(2-index))/(2-index)
        else
            flux = alog(2.0)-alog(0.5)
        endif

! convert keV to erg

        flux = flux*1000*erg_per_ev

! now convert erg/cm2/s/sr to keV/cm2/s/sr/keV

        tot_flux = tot_flux/flux
        min_flux = min_flux/flux

! print,'T,M (keV/cm2/s/sr)',tot_flux,min_flux
! calculate the bias

        bias = cxrb_norm-tot_flux

! print,'Bias: ',bias
! the Xspec normalization per sr

        xnorm = bias + min_flux

! print,'X(/sr): ',xnorm

! convert to per arcmin

        xnorm = xnorm/degree_per_ster/3600.

        print *, 'X(/am2): ', xnorm
!
        return
        end subroutine point_source


        subroutine mate_elf(aflg,minflux,maxflux,numb,flux,vari)

! inputs
!      aflg   :0 for Hasinger upper break, 1 for using one that works
!      minflux:minimum flux of point sources in ergs/cm^2/s (0.5-2.0)keV
!      maxflux:maximum flux of point sources in erge/cm^2/s (0.5-2.0)keV
! outputs
!      numb   :the number of point sources degree^-2
!      flux   :the flux due to the number of point sources (0.5-2.0)kev degree^-2
!      vari   :the variance in the flux due to point sources degree^-2

! verified that flux was correct as of 13/03/00

        implicit none

        real aflg, b0, b1, b2, flux, hi, k, lo, minflux, maxflux, &
            n0, n1, n2, numb, reg0hi, reg0lo, reg1hi, reg1lo, reg2hi, &
            reg2lo, sbb, sbf, vari

        lo = minflux/1e-14
        hi = maxflux/1e-14

! sat=3.98e-3

        sbb = 2.42
        sbf = 0.80

        k = 46.0*sbb

        b0 = 2.44
        b1 = 2.10
        b2 = 1.61

        n0=(k/sbb)
        n1=(k/sbb)
        n2=(k/sbb)*(sbf/sbb)**(b2-b1)

        reg2lo = 0.0
        reg2hi = 0.0
        reg1lo = 0.0
        reg1hi = 0.0
        reg0lo = 0.0
        reg0hi = 0.0

        if(lo .lt. sbf) then
            reg2lo = lo
            if(hi .lt. sbf) then
                reg2hi = hi
            else
                reg2hi=sbf
                if(hi .gt. sbb)then
                    reg1hi = sbb
                    reg1lo = sbf
                    reg0hi = hi
                    reg0lo = sbb
                else
                    reg1hi = hi
                    reg1lo = sbf
                endif
            endif
        else
            if(lo .lt. sbb)then
                reg1lo = lo
                if(hi .lt. sbb) then
                    reg1hi = hi
                else
                    reg1hi = sbb
                    reg0lo = sbb
                    reg0hi = hi
                endif
            else
                reg0lo=lo
                reg0hi=hi
            endif
        endif

!        print *, reg2lo, reg2hi
!        print *, reg1lo, reg1hi
!        print *, reg0lo, reg0hi

        reg0hi = reg0hi/sbb
        reg0lo = reg0lo/sbb
        reg1hi = reg1hi/sbb
        reg1lo = reg1lo/sbb
        reg2hi = reg2hi/sbb
        reg2lo = reg2lo/sbb

        numb = 0.0
        flux = 0.0
        vari = 0.0
        if(reg2lo .ne. 0.0) then
            numb = numb + (n2/(1-b2))*(reg2hi**(1-b2) - reg2lo**(1-b2))
            flux = flux + (n2/(2-b2))*(reg2hi**(2-b2) - reg2lo**(2-b2))
            vari = vari + (n2/(3-b2))*(reg2hi**(3-b2) - reg2lo**(3-b2))
        endif

        if(reg1lo .ne. 0.0) then
            numb = numb + (n1/(1-b1))*(reg1hi**(1-b1) - reg1lo**(1-b1))
            flux = flux + (n1/(2-b1))*(reg1hi**(2-b1) - reg1lo**(2-b1))
            vari = vari + (n1/(3-b1))*(reg1hi**(3-b1) - reg1lo**(3-b1))
        endif

        if(reg0lo .ne. 0) then
            numb = numb + (n0/(1-b0))*(reg0hi**(1-b0) - reg0lo**(1-b0))
            flux = flux + (n0/(2-b0))*(reg0hi**(2-b0) - reg0lo**(2-b0))
            vari = vari + (n0/(3-b0))*(reg0hi**(3-b0) - reg0lo**(3-b0))
        endif

        flux = flux*1e-14*sbb
        vari = vari*1e-14*1e-14*sbb*sbb

        return
        end subroutine mate_elf

        subroutine capp_elf(aflg,minflux,maxflux,numb,flux,vari)

! inputs
!     aflg   :currently meaningless
!     minflux:minimum flux of point sources in ergs/cm^2/s (0.5-2.0)keV
!     maxflux:maximum flux of point sources in erge/cm^2/s (0.5-2.0)keV
! outputs
!     numb   :the number of point sources degree^-2
!     flux   :the flux due to the number of point sources (0.5-2.0)kev degree^-2
!     vari   :the variance in the flux due to point sources degree^-2

        implicit none

        real :: aflg, b1, b2, flux, hi, lo, minflux, maxflux, n1, n2, &
            numb, reg1hi, reg1lo, reg2hi, reg2lo, sbb, vari

        lo = minflux/1e-14
        hi = maxflux/1e-14

        sbb = 1.55

        n1 = 123.0
        n2 = 81.1129

        b1 = 2.60
        b2 = 1.65

        reg2lo = 0
        reg2hi = 0
        reg1lo = 0
        reg1hi = 0

        if(lo .lt. sbb) then
            reg2lo = lo
            if(hi .lt. sbb) then
                reg2hi = hi
            else
                reg2hi = sbb
                reg1lo = sbb
                reg1hi = hi
            endif
        else
            reg1lo = lo
            reg1hi = hi
        endif

!        print *, reg2lo, reg2hi
!        print *, reg1lo, reg1hi

        numb = 0.0
        flux = 0.0
        vari = 0.0
        if(reg2lo .ne. 0) then
            numb = numb + (n2/(1-b2))*(reg2hi**(1-b2) - reg2lo**(1-b2))
            flux = flux + (n2/(2-b2))*(reg2hi**(2-b2) - reg2lo**(2-b2))
            vari = vari + (n2/(3-b2))*(reg2hi**(3-b2) - reg2lo**(3-b2))
        endif

!        print *, numb, flux, vari

        if(reg1lo .ne. 0) then
            numb = numb + (n1/(1-b1))*(reg1hi**(1-b1) - reg1lo**(1-b1))
            flux = flux + (n1/(2-b1))*(reg1hi**(2-b1) - reg1lo**(2-b1))
            vari = vari + (n1/(3-b1))*(reg1hi**(3-b1) - reg1lo**(3-b1))
        endif

!        print *, numb, flux, vari

        flux = flux*1e-14
        vari = vari*1e-14*1e-14

!        print *, numb, flux, vari

        return
        end subroutine capp_elf

        subroutine mush_elf (aflg,minflux,maxflux,numb,flux,vari)

! inputs
!     aflg   :currently meaningless
!     minflux:minimum flux of point sources in ergs/cm^2/s (0.5-2.0)keV
!     maxflux:maximum flux of point sources in erge/cm^2/s (0.5-2.0)keV
! outputs
!     numb   :the number of point sources degree^-2
!     flux   :the flux due to the number of point sources (0.5-2.0)kev degree^-2
!     vari   :the variance in the flux due to point sources degree^-2

        implicit none

        real :: aflg, b1, b2, flux, hi, lo, minflux, maxflux, n1, n2, &
            numb, reg1hi, reg1lo, reg2hi, reg2lo, sbb, vari

        lo = minflux/7e-15
        hi = maxflux/7e-15

        sbb = 1.0

        n1 = 277.5
        n2 = 129.5

        b1 = 2.50
        b2 = 1.70

        reg2lo = 0
        reg2hi = 0
        reg1lo = 0
        reg1hi = 0

        if(lo .lt. sbb) then
            reg2lo = lo
            if(hi .lt. sbb) then
                reg2hi = hi
            else
                reg2hi = sbb
                reg1lo = sbb
                reg1hi = hi
            endif
        else
            reg1lo = lo
            reg1hi = hi
        endif

!        print *, reg2lo, reg2hi
!        print *, reg1lo, reg1hi

        numb = 0.0
        flux = 0.0
        vari = 0.0
        if(reg2lo .ne. 0.0) then
            numb = numb + (n2/(1-b2))*(reg2hi**(1-b2) - reg2lo**(1-b2))
            flux = flux + (n2/(2-b2))*(reg2hi**(2-b2) - reg2lo**(2-b2))
            vari = vari + (n2/(3-b2))*(reg2hi**(3-b2) - reg2lo**(3-b2))
        endif

!        print,numb,flux,vari

        if(reg1lo .ne. 0.0) then
            numb = numb + (n1/(1-b1))*(reg1hi**(1-b1) - reg1lo**(1-b1))
            flux = flux + (n1/(2-b1))*(reg1hi**(2-b1) - reg1lo**(2-b1))
            vari = vari + (n1/(3-b1))*(reg1hi**(3-b1) - reg1lo**(3-b1))
        endif

!        print *, numb, flux, vari

        flux = flux*7e-15
        vari = vari*7e-15*7e-15

!        print *, numb, flux, vari

        return
        end subroutine mush_elf

end module point_source_mod
