!+
! Module: proton_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: proton_mod.f90,v 1.3 2009/02/26 16:39:45 aibarra Exp $
!-
!  Version 5.0  9 November 2012
!
!  Casts a map of the soft proton background using the fitted
!  spectral parameters for a power law (works) and broken power
!  law (in progress).  Soft proton detector maps developed by
!  K. Kuntz are used for the distributions
!
!  Usage:
!     prefix - exposure ID prefix
!     xmmdir - ESAS CCF directory 
!     ccd1-ccd7 - MOS ccd#1 - ccd#7 or PN quad#1 - quad #4 inclusion control
!     elow - lower energy band limit in eV
!     ehigh - upper energy band limit in eV
!     spectrumcontrol - spectrum control
!           1 - power law,  2 - broken power law
!     spectral parameters from Xspec
!     spectrumcontrol=1
!         pindex - power law index
!         pnorm - power law normalization
!     spectrumcontrol=2
!         bindl - soft power law index
!         break - power law break energy (in keV)
!         bindh - hard power law index
!         bnorm - broken power law normalization
!
!     Examples:
!         proton prefix=1S001 caldb=CalDB elow=400 ehigh=1250 ccd1=1 
!               ccd2=1 ccd3=1 ccd1=4 ccd5=1 ccd6=0 ccd1=7 spectrumcontrol=1 
!               pindex=0.8 pnorm=0.1
!         proton prefix=1S001 caldb=CalDB elow=400 ehigh=1250 ccd1=1 
!               ccd2=1 ccd3=1 ccd1=4 ccd5=1 ccd6=0 ccd1=7 spectrumcontrol=1 
!               bindl=0.8 break=3.0 bindh=0.3 pnorm=0.1

module proton_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public proton

contains

  !---------------------------------------------------------------------
  subroutine proton
  !---------------------------------------------------------------------

        character (len=1) :: db_fi(132), detnam, sp_fi(19), xmmd(132)
        character (len=4) :: prefi
        character (len=5) :: prefix, prefixin
        character (len=8) :: extname
        character (len=8) :: tform(10), ttype(10), tunit(10)
        character (len=19) :: sp_file
        character (len=60) :: com
        character (len=132) :: db_file, objfile, outfile, &
            spec_name, xmmdir
        logical :: anyf, clobber, extend, extendn, simple, simplen
        integer :: bitpix, bitpixn, block, ccd(7), ccd1, ccd2, ccd3, &
            ccd4, ccd5, ccd6, ccd7, ehigh, elow, gcount, gcountn, &
            group, hdutype, i, i1, i2, ib, ibb(7), idet, iext, ih, &
            ii, ii1, ii2, iii, il, ispec, j, jj, k, kk, lun, &
            luna, lunb, luns, n, naxes(2), naxesn(2), naxis, naxisn, &
            nchan, pcount, pcountn, revl, tfields, varidat
        integer :: err = 0
        integer :: status = 0
        real :: bindl, bbreak, break, bindh, bnorm, bnormh, bnorml, &
            eh, el, exposen, fb(7), inarr(780,780), pindex, pnorm, &
            prot, rind, rn, scale, sphb, splb, spmap(780,780), temp
        real :: elim(7) = (/300.0, 750.0, 1250.0, 2000.0, 4000.0, &
            8000.0, 12000.0/)

        equivalence (db_file,db_fi)
        equivalence (sp_file,sp_fi)
        equivalence (prefix,prefi)
        equivalence (xmmdir,xmmd)

        prefix = stringParameter("prefix")
        xmmdir = stringParameter("caldb")
        spec_name = stringParameter("specname")
        ccd1 = intParameter("ccd1")
        ccd2 = intParameter("ccd2")
        ccd3 = intParameter("ccd3")
        ccd4 = intParameter("ccd4")
        ccd5 = intParameter("ccd5")
        ccd6 = intParameter("ccd6")
        ccd7 = intParameter("ccd7")
        elow = intParameter("elow")
        ehigh = intParameter("ehigh")
        ispec = intParameter("spectrumcontrol")
        pindex = realParameter("pindex")
        pnorm = realParameter("pnorm")
        bindl = realParameter("bindl")
        bbreak = realParameter("bbreak")
        bindh = realParameter("bindh")
        bnorm = realParameter("bnorm")
        clobber = booleanParameter("clobber")

        break = bbreak
        prefixin = prefix

        do i=1,780
            do ii=1,780
                spmap(i,ii) = 0.0
            enddo
        enddo

!  Get the input parameters

!  Get the prefix

        read(prefix,2998) detnam
 2998   format(a1)
        if((detnam .eq. '1') .or. (detnam .eq. '2')) then
            read(prefix,2999) idet
 2999       format(i1)
            if(idet .eq. 1) then
                sp_file = '/mos1-flare.fits.gz'
            else
                sp_file = '/mos2-flare.fits.gz'
            endif
        else
            idet = 3
            sp_file = '/pn-flare.fits.gz  '
            detnam = 'p'
        endif

        ccd(1) = ccd1
        ccd(2) = ccd2
        ccd(3) = ccd3
        ccd(4) = ccd4
        ccd(5) = ccd5
        ccd(6) = ccd6
        ccd(7) = ccd7

        print *, 'xmmdir: ', trim(xmmdir)
        print *, 'prefix: ', prefix
        if(idet .lt. 3) then
            print *, 'CCD selection', ccd(1), ccd(2), ccd(3), ccd(4), &
                    ccd(5), ccd(6), ccd(7)
        else
            print *, 'Quad selection', ccd(1), ccd(2), ccd(3), ccd(4)
        endif
        print *, 'spectrum name: ', spec_name
        print *, 'elow, ehigh: ', elow, ehigh
        print *, 'idet: ', idet
        print *, 'ispec: ', ispec
        if(ispec .eq. 1) then
            print *, 'Photon Index and Norm: ', pindex, pnorm
        elseif(ispec .eq. 2) then
            print *, 'Low E Index, Break Energy, High E Index, Norm: '
            print *, bindl, break, bindh, bnorm
        endif

!  Set the bands to be included

        il = 1
        do while ((elow .gt. elim(il)) .and. (il .le. 7))
            il = il + 1
        enddo
        if(il .eq. 8) then
            print *, 'Low energy bound out of range'
            stop
        endif
        ih = il
        do while ((ehigh .gt. elim(ih)) .and. (ih .lt. 7))
            ih = ih + 1
        enddo

        if(il .eq. ih) then
            ibb(1) = il - 1
            if(ibb(1) .eq. 0) ibb(1) = 1
            fb(1) = 1.0
            n = 1
        else
            if(il .lt. 2) il = 2
            do ib=il,ih
                ibb(ib-il+1) = ib - 1
                if(ib .eq. il) then
                    fb(ib-il+1) = (elim(ib) - elow)/ &
                            (elim(ib) - elim(ib-1))
                    if(fb(ib-il+1) .gt. 1.0) fb(ib-il+1) = 1.0
                elseif(ib .eq. ih) then
                    fb(ib-il+1) = (ehigh - elim(ib-1))/ &
                            (elim(ib) - elim(ib-1))
                    if(fb(ib-il+1) .gt. 1.0) fb(ib-il+1) = 1.0
                else
                    fb(ib-il+1) = 1.0
                endif
            enddo
            n = ih - il + 1
        endif

        print *, 'Bands and scale factors for the SP maps'
        do i=1,n
            print *, ibb(i), fb(i)
        enddo

!  Create the SP map template

        ii = 1
        do while (xmmd(ii) .ne. ' ')
            db_fi(ii) = xmmd(ii)
            ii = ii + 1
        enddo
        do iii=0,18
            db_fi(ii+iii) = sp_fi(iii+1)
        enddo
        do iii=ii+19,132
            db_fi(iii) = ' '
        enddo
        print *, db_file

!  Get an unsed LUN

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 01'
            print *, 'Status FTGIOU =', status
        endif

!  Open the FITS file for the SP image

        call ftopen (lun,db_file,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 02'
            print *, 'Status FTOPEN =', status
            print *, db_file
        endif

!  Loop through the selected SP bands

        do j=1,n

!  Co-add the data from all filters

            do jj=0,2
                iext = ibb(j) + 2 + jj*7
                print *, ibb(j), iext

!  Move to the correct extension

                call ftmahd (lun,iext,hdutype,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'PROTON Position 03'
                    print *, 'Status FTGHPR =', status
                    print *, db_file
                    stop
                endif

!  Read the extension header information

                call ftghpr (lun,10,simple,bitpix,naxis,naxes, &
                    pcount,gcount,extend,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'PROTON Position 04'
                    print *, 'Status FTGHPR =', status
                    print *, db_file
                    stop
                endif

!  Read and print out the extension name

                call ftgkys (lun,'EXTNAME',extname,com,status)
                if(status .ne. 0) then
                    print *, 'PROTON POSITION 05'
                    print *, 'Missing EXTNAME keyword'
                    print *, 'Status FTGKYS =', status
                    print *, db_file
                    stop
                endif
                print *, extname

!  Return the SP image

                call ftg2de (lun,0,0,naxes(1),naxes(1),naxes(2), &
                    inarr,anyf,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'PROTON Position 06'
                    print *, 'Status FTG2DE =', status
                    print *, db_file
                endif

!  Add data to the array

                do k=1,780
                    do kk=1,780
                        spmap(k,kk) = spmap(k,kk) + fb(j)*inarr(k,kk)
                    enddo
                enddo
            enddo
        enddo

!  Close the file

        call ftclos(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 05'
            print *, 'Status FTCLOS =', status
            print *, db_file
        endif

!  Mask chips if necessary

        if(ccd(1) .eq. 0) then
            if(idet .eq. 1) then
                i1 = 260
                i2 = 522
                ii1 = 259
                ii2 = 520
            elseif(idet .eq. 2) then
                i1 = 259
                i2 = 522
                ii1 = 258
                ii2 = 520
            else
                i1 = 23
                i2 = 346
                ii1 = 368
                ii2 = 684
            endif
            do i=i1,i2
                do ii=ii1,ii2
                    spmap(i,ii) = 0.0
                enddo
            enddo
        endif
        if(ccd(2) .eq. 0) then
            if(idet .eq. 1) then
                i1 = 392
                i2 = 653
                ii1 = 1
                ii2 = 251
            elseif(idet .eq. 2) then
                i1 = 390
                i2 = 654
                ii1 = 1
                ii2 = 254
            else
                i1 = 346
                i2 = 669
                ii1 = 368
                ii2 = 684
            endif
            do i=i1,i2
                do ii=ii1,ii2
                    spmap(i,ii) = 0.0
                enddo
            enddo
        endif
        if(ccd(3) .eq. 0) then
            if(idet .eq. 1) then
                i1 = 523
                i2 = 780
                ii1 = 252
                ii2 = 516
            elseif(idet .eq. 2) then
                i1 = 523
                i2 = 780
                ii1 = 253
                ii2 = 518
            else
                i1 = 346
                i2 = 670
                ii1 = 53
                ii2 = 368
            endif
            do i=i1,i2
                do ii=ii1,ii2
                    spmap(i,ii) = 0.0
                enddo
            enddo
        endif
        if(ccd(4) .eq. 0) then
            if(idet .eq. 1) then
                i1 = 394
                i2 = 656
                ii1 = 520
                ii2 = 780
            elseif(idet .eq. 2) then
                i1 = 391
                i2 = 654
                ii1 = 520
                ii2 = 780
            else
                i1 = 23
                i2 = 347
                ii1 = 54
                ii2 = 368
            endif
            do i=i1,i2
                do ii=ii1,ii2
                    spmap(i,ii) = 0.0
                enddo
            enddo
        endif
        if(idet .lt. 3) then
            if(ccd(5) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 129
                    i2 = 395
                    ii1 = 521
                    ii2 = 780
                else
                    i1 = 126
                    i2 = 391
                    ii1 = 522
                    ii2 = 780
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        spmap(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(ccd(6) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 1
                    i2 = 260
                    ii1 = 256
                    ii2 = 520
                else
                    i1 = 1
                    i2 = 258
                    ii1 = 256
                    ii2 = 522
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        spmap(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(ccd(7) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 127
                    i2 = 392
                    ii1 = 1
                    ii2 = 255
                else
                    i1 = 124
                    i2 = 389
                    ii1 = 1
                    ii2 = 254
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        spmap(i,ii) = 0.0
                    enddo
                enddo
            endif
        endif

!  Find the number of counts in the detector map

        rn = 0.0
        do i=1,780
            do ii=1,780
                rn = rn + spmap(i,ii)
            enddo
        enddo

!  Get the exposure time from the spectrum file
!  First get an unused lun

        call ftgiou(luns,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'READ_PHA POSITION 01'
        endif

!  Open the FITS file

        call ftopen (luns,spec_name,0,block,status)
        if(status .ne. 0) then
            err = 1
            print *, 'STATUS FTOPEN =', status
            print *, spec_name
            stop
        endif

!  Move to the spectrum extension

        call ftmrhd (luns,1,hdutype,status)
        if(status .ne. 0) then
            err = 1
            print *, 'STATUS FTMRHD =', status
            print *, 'Could not find the spectrum extension'
            print *, spec_name
            stop
        endif

!  Read the FITS file header information

        call ftghbn (luns,10,nchan,tfields,ttype,tform, &
                tunit,extname,varidat,status)
        if(extname .ne. 'SPECTRUM') then
            print *, extname
            print *, 'This was supposed to be a spectrum ext'
            stop
        endif

!  Read the exposure from the header

        call ftgkye (luns,'EXPOSURE',exposen,com,status)
        if(status .ne. 0) then
            print *, 'Missing EXPOSURE keyword in spectrum file'
            print *, spec_name
            stop
        endif

!  Clean up

        call ftclos (luns,status)
        call ftfiou (luns,status)

!  Determine the number of SP counts in the band

        eh = ehigh/1000.0
        el = elow/1000.0
        if(ispec .eq. 1) then
            rind = 1.0 - pindex
            prot = pnorm*(eh**rind - el**rind)/rind
        elseif(ispec .eq. 2) then
            splb = break**(-bindl)
            sphb = break**(-bindh)
            if(break .lt. 1.0) then
                bnorml = bnorm*sphb/splb
                bnormh = bnorm
            elseif (break .eq. 1.0) then
                bnorml = bnorm
                bnormh = bnorm
            else
                bnorml = bnorm
                bnormh = bnorm*splb/sphb
            endif
            if(eh .le. break) then
                rind = 1.0 - bindl
                prot = bnorm*(eh**rind - el**rind)/rind
            elseif(el .ge. break) then
                rind = 1.0 - bindh
                prot = bnorm*(break**(bindh-bindl))* &
                        (eh**rind - el**rind)/rind
            else
                rind = 1.0 - bindl
                temp = bnorm*(break**rind - el**rind)/rind
                rind = 1.0 - bindh
                prot = temp + bnorm*(break**(bindh-bindl))* &
                        (eh**rind - break**rind)/rind
            endif
!            if(eh .le. break) then
!                rind = 1.0 - bindl
!                prot = bnorml*(eh**rind - el**rind)/rind
!            elseif(el .ge. break) then
!                rind = 1.0 - bindh
!                prot = bnormh*(eh**rind - el**rind)/rind
!            else
!                rind = 1.0 - bindl
!                temp = bnorml*(break**rind - el**rind)/rind
!                rind = 1.0 - bindh
!                prot = temp + bnormh*(eh**rind - break**rind)/rind
!            endif
        else
            print *, 'Error in spectral type for SP background'
            print *, 'ispec: ', ispec
            stop
        endif
        print *, 'Soft Proton Event Rate: ', prot

!  Open up a template event image

!  First get an unused lun

        call ftgiou(luna,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 08'
            print *, 'STATUS FTGIOU =', status
        endif

!  Create the file name

        if(elow .ge. 1000) then
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(objfile,3000) prefix, elow, ehigh
 3000               format('mos',a5,'-obj-im-det-',i4,'-',i5,'.fits')
                else
                    write(objfile,3002) prefi, elow, ehigh
 3002               format('pn',a4,'-obj-im-det-',i4,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(objfile,3004) prefix, elow, ehigh
 3004               format('mos',a5,'-obj-im-det-',i4,'-',i4,'.fits')
                else
                    write(objfile,3006) prefi, elow, ehigh
 3006               format('pn',a4,'-obj-im-det-',i4,'-',i4,'.fits')
                endif
            endif
        elseif(ehigh .lt. 1000) then
            if(idet .lt. 3) then
                write(objfile,3010) prefix, elow, ehigh
 3010           format('mos',a5,'-obj-im-det-',i3,'-',i3,'.fits')
            else
                write(objfile,3012) prefi, elow, ehigh
 3012           format('pn',a4,'-obj-im-det-',i3,'-',i3,'.fits')
            endif
        else
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(objfile,3020) prefix, elow, ehigh
 3020               format('mos',a5,'-obj-im-det-',i3,'-',i5,'.fits')
                else
                    write(objfile,3022) prefi, elow, ehigh
 3022               format('pn',a4,'-obj-im-det-',i3,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(objfile,3024) prefix, elow, ehigh
 3024               format('mos',a5,'-obj-im-det-',i3,'-',i4,'.fits')
                else
                    write(objfile,3026) prefi, elow, ehigh
 3026               format('pn',a4,'-obj-im-det-',i3,'-',i4,'.fits')
                endif
            endif
        endif
        print *, 'Template image'
        print *, objfile

!  Open the FITS file

        call ftopen (luna,objfile,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 09'
            print *, 'STATUS FTOPEN =', status
            print *, objfile
        endif

!  Read the FITS file header information

        call ftghpr (luna,10,simplen,bitpixn,naxisn,naxesn, &
            pcountn,gcountn,extendn,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 10'
            print *, 'Status FTGHPR =', status
            print *, objfile
        endif

!  Get the revolution

        call ftgkyj (luna,'REVOLUT',revl,com,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 11b'
            print *, 'Status FTGKY =', status
            print *, objfile
        endif

!  Check to see whether MOS1 CCD#6 is missing

        if((idet .eq. 1) .and. (revl .gt. 961)) then
            print *, 'Observation took place after MOS1 CCD6 ', &
                        'was lost'
            if(ccd(6) .eq. 1) then
                print *, 'Set ccd(6) equal to 0'
                ccd(6) = 0
                print *, 'Revised CCD selection: ', ccd(1), ccd(2), &
                        ccd(3), ccd(4), ccd(5), ccd(6), ccd(7)
                i1 = 1
                i2 = 258
                ii1 = 257
                ii2 = 520
                do i=i1,i2
                    do ii=ii1,ii2
                        spmap(i,ii) = 0.0
                    enddo
                enddo
                rn = 0.0
                do i=1,780
                    do ii=1,780
                        rn = rn + spmap(i,ii)
                    enddo
                enddo
            endif
            if(revl .eq. 961) then
                print *, 'Observation took place during the ', &
                    'observation where MOS1 CCD6 was lost.  '
                print *, 'Exposure U002 has CCD6 data, U003 does not'
            endif
        endif


!  Set the scale factor

        prot = exposen*prot
        print *, 'Observation exposure: ', exposen
        print *, 'Detector map counts', rn
        print *, 'Proton flaring events: ', prot
        scale = prot/rn
        do i=1,780
            do ii=1,780
                spmap(i,ii) = scale*spmap(i,ii)
            enddo
        enddo

!  Write out the SP detector map

!  First get an unused lun

        call ftgiou(lunb,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 12'
            print *, 'STATUS FTGIOU =', status
        endif

!  Create the file name

        if(elow .ge. 1000) then
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(outfile,3100) prefix, elow, ehigh
 3100               format('mos',a5,'-prot-im-det-',i4,'-',i5,'.fits')
                else
                    write(outfile,3102) prefi, elow, ehigh
 3102               format('pn',a4,'-prot-im-det-',i4,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(outfile,3104) prefix, elow, ehigh
 3104               format('mos',a5,'-prot-im-det-',i4,'-',i4,'.fits')
                else
                    write(outfile,3106) prefi, elow, ehigh
 3106               format('pn',a4,'-prot-im-det-',i4,'-',i4,'.fits')
                endif
            endif
        elseif(ehigh .lt. 1000) then
            if(idet .lt. 3) then
                write(outfile,3110) prefix, elow, ehigh
 3110           format('mos',a5,'-prot-im-det-',i3,'-',i3,'.fits')
            else
                write(outfile,3112) prefi, elow, ehigh
 3112           format('pn',a4,'-prot-im-det-',i3,'-',i3,'.fits')
            endif
        else
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(outfile,3120) prefix, elow, ehigh
 3120               format('mos',a5,'-prot-im-det-',i3,'-',i5,'.fits')
                else
                    write(outfile,3122) prefi, elow, ehigh
 3122               format('pn',a4,'-prot-im-det-',i3,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(outfile,3124) prefix, elow, ehigh
 3124               format('mos',a5,'-prot-im-det-',i3,'-',i4,'.fits')
                else
                    write(outfile,3126) prefi, elow, ehigh
 3126               format('pn',a4,'-prot-im-det-',i3,'-',i4,'.fits')
                endif
            endif
        endif
        print *, 'Output file name'
        print *, outfile

!  Check to see if there is an existing file

        call ftopen (lunb,outfile,0,block,status)
        if(status .eq.  0) then
            print *, 'File exists with output file name'
            if(clobber) then
                print *, 'Clobber the file and continue'
                call ftdelt(lunb,status)
            else
                print *, 'PROTON Position 13'
                print *, 'Will not clobber file'
                stop
            endif
        endif
        status = 0

!  Create the output FITS file using the original file as a template

        call ftinit (lunb,outfile,2880,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 14'
            print *, 'STATUS FTINIT =', status
            print *, outfile
        endif

!  Copy the header information from the old to the new file

        call ftcphd (luna,lunb,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 15'
            print *, 'STATUS FTCPHD =', status
        endif

        call ftpdef (lunb,bitpixn,naxisn,naxesn,pcountn, &
                    gcountn,.FALSE.,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 16'
            print *, 'STATUS FTPDEF =', status
        endif
        call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'PROTON Position 17'
            print *, 'STATUS FTMKYJ =', status
        endif
        if(err .eq. 0) then

!  Write the output image

            group = 1
            call ftp2de (lunb,group,naxes(1),naxes(1),naxes(2), &
                        spmap,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PROTON Position 18'
                print *, 'STATUS FTP2DE =', status
                print *, 'Could not write the background image'
            endif

!  Close the output file

            call ftclos(lunb,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'PROTON Position 19'
                print *, 'STATUS FTCLOS =', status
                print *, 'Could not close the output file'
            endif
        endif

        return

  end subroutine proton

end module proton_mod
