!+
! Module: swcx_mod
!
! SAS Version 12.0
! Version 1.1
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: swcx_mod.f90,v 1.0 2010/06/24 16:39:45 Snowden $
!-
!  SAS Version 13.0
!  Version 5.0  9 November 2012
!
!  Casts a map of the SWCX background using the fitted spectral parameters 
!  for Gaussians at the SWCX energies.  Exposure maps for the energies of
!  the SWCX lines are used for the detector maps.
!
!  Usage:
!     prefixin - exposure ID prefix
!     xmmdir - ESAS CCF directory 
!     xmmd - ESAS CCF directory 
!     ccd1-ccd7 - MOS ccd#1 - ccd#7 or PN quad#1 - quad #4 inclusion control
!     elow - lower energy band limit in eV
!     ehigh - upper energy band limit in eV
!     nlines - number of lines
!     enlinelist - energies of the lines to be included 
!     gnormlist - Gaussian normalizations from Xspec
!     obj_rmf - input RMF
!     obj_arf - input ARF
!     obj_spec - input spectrum
!
!     Examples:
!         swcx prefix=1S001 caldb=/CalDB elow=400 ehigh=1250 enlines='0.56 0.65', ccd1=1 
!               ccd2=1 ccd3=1 ccd1=4 ccd5=1 ccd6=0 ccd1=7 gnorm='0.1 0.05'

module swcx_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public swcx

contains

  !---------------------------------------------------------------------
  subroutine swcx
  !---------------------------------------------------------------------

        character (len=1) :: db_fi(132), detnam, swcx_fi(19), xmmd(132)
        character (len=4) :: prefi
        character (len=5) :: prefix, prefixin
        character (len=8) :: extname
        character (len=19) :: swcx_file
        character (len=25) :: mode
        character (len=60) :: com
        character (len=132) :: db_file, obj_arf, objfile, obj_rmf, &
            obj_spec, outfile, xmmdir
        logical :: anyf, clobber, extend, extendn, simple, simplen

        integer :: bitpix, bitpixn, block, ccd(7), ccd1, ccd2, ccd3, &
            ccd4, ccd5, ccd6, ccd7, chanrmf(4096), channel(4096), &
            ehigh, elow, gcount, gcountn, gr, group(4096), hdutype, i, &
            i1, i2, idet, ien, iext, ii, ii1, ii2, iii, iline, irmf, &
            k, kk, lun, luna, lunb, naxes(2), naxesn(2), naxis, &
            naxisn, nchana, nchanrin, nchanrout, nchans, &
            nlines, obsid, pcount, pcountn, quality(4096), revl
        integer :: err = 0
        integer :: status = 0
        real :: area, areascal, arf(4096), backscal, counts(4096), &
            detm(4096,4096), emax(4096), emaxin(4096), emaxout(4096), &
            emin(4096), eminin(4096), eminout(4096), &
            exposure, frac, inarr(780,780), rn, rnorm, stat(4096), &
            swcxmap(780,780), test
        real, dimension(:), allocatable :: gnormlist
        real, dimension(:), allocatable :: enlinelist
        real :: enim(10) = (/0.56, 0.65, 0.81, 0.91, 1.34, 0.0, 0.0, &
            0.0, 0.0, 0.0/)
        double precision exposen

        equivalence (db_file,db_fi)
        equivalence (swcx_file,swcx_fi)
        equivalence (prefix,prefi)
        equivalence (xmmdir,xmmd)

        prefixin = stringParameter("prefix")
        prefix = prefixin
        xmmdir = stringParameter("caldb")
        ccd1 = intParameter("ccd1")
        ccd2 = intParameter("ccd2")
        ccd3 = intParameter("ccd3")
        ccd4 = intParameter("ccd4")
        ccd5 = intParameter("ccd5")
        ccd6 = intParameter("ccd6")
        ccd7 = intParameter("ccd7")
        elow = intParameter("elow")
        ehigh = intParameter("ehigh")
        obj_rmf = stringParameter("objrmf")
        obj_arf = stringParameter("objarf")
        obj_spec = stringParameter("objspec")
        clobber = booleanParameter("clobber")
        nlines = parameterCount("linelist")
        allocate(enlinelist(nlines))
        allocate(gnormlist(nlines))
        do i=1, nlines
            enlinelist(i) = realParameter("linelist",i-1)
            gnormlist(i) = realParameter("gnormlist",i-1)
        enddo

!  Make sure that the line energies are in keV

        do i=1,nlines
            if(enlinelist(i) .gt. 20.0) then
                 enlinelist(i) = enlinelist(i)/1000.0
            endif
        enddo

        do i=1,780
            do ii=1,780
                swcxmap(i,ii) = 0.0
            enddo
        enddo

!  Get the spectrum

        call read_pha (obj_spec,4096,nchans,channel,counts,stat, &
                quality,group,exposure,backscal,areascal,revl,obsid, &
                mode,err)
        if(err .ne. 0) then
            print *, 'SWCX POSITION 01'
            print *, 'Error on return from READ_PHA'
            stop
        endif
        backscal = backscal/1440000.0

!  Get the arf information
 
        call read_arf (obj_arf,4096,nchana,arf,emin,emax,err)
        if(err .ne. 0) then
            print *, 'SWCX POSITION 02'
            print *, 'Error on return from READ_ARF'
            stop
        endif

!  Get the rmf information

        call read_rmf (obj_rmf,4096,detm,chanrmf,eminin,emaxin, &
                    nchanrin,eminout,emaxout,nchanrout)
        if(err .ne. 0) then
            print *, 'SWCX POSITION 03'
            print *, 'Error on return from READ_RMF'
            stop
        endif

!  Get the input parameters

!  Get the prefix

        read(prefix,2998) detnam
 2998   format(a1)
        if((detnam .eq. '1') .or. (detnam .eq. '2')) then
            read(prefix,2999) idet
 2999       format(i1)
        else
            idet = 3
            detnam = 'p'
        endif
        swcx_file = '/swcx-im.fits.gz'

        ccd(1) = ccd1
        ccd(2) = ccd2
        ccd(3) = ccd3
        ccd(4) = ccd4
        ccd(5) = ccd5
        ccd(6) = ccd6
        ccd(7) = ccd7

        print *, 'xmmdir: ', xmmdir
        print *, 'prefix: ', prefix
        if(idet .lt. 3) then
            print *, 'CCD selection', ccd(1), ccd(2), ccd(3), ccd(4), &
                    ccd(5), ccd(6), ccd(7)
        else
            print *, 'Quad selection', ccd(1), ccd(2), ccd(3), ccd(4)
        endif

!  Check to see whether MOS1 CCD#6 is missing

        if((idet .eq. 1) .and. (revl .gt. 961)) then
            print *, 'Observation took place after MOS1 CCD6 ', &
                        'was lost'
           if(ccd(6) .eq. 1) then
                print *, 'Set ccd(6) equal to 0'
                ccd(6) = 0
            endif
            print *, 'Revised CCD selection: ', ccd(1), ccd(2), &
                    ccd(3), ccd(4), ccd(5), ccd(6), ccd(7)
            if(revl .eq. 961) then
                print *, 'Observation took place during the ', &
                    'observation where MOS1 CCD6 was lost.  '
                print *, 'Exposure U002 has CCD6 data, U003 does not'
            endif
        endif

        print *, 'elow, ehigh: ', elow, ehigh
        print *, 'idet: ', idet
        do i=1,nlines
             print *, 'Line and Norm: ', enlinelist(i), &
                     '  ', gnormlist(i)
        enddo

!  Create the SWCX map file name

        ii = 1
        do while (xmmd(ii) .ne. ' ')
            db_fi(ii) = xmmd(ii)
            ii = ii + 1
        enddo
        do iii=0,18
            db_fi(ii+iii) = swcx_fi(iii+1)
        enddo
        do iii=ii+19,132
            db_fi(iii) = ' '
        enddo
        print *, db_file

!  Get an unsed LUN

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 04'
            print *, 'Status FTGIOU =', status
        endif

!  Open the FITS file for the SWCX image

        call ftopen (lun,db_file,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 05'
            print *, 'Status FTOPEN =', status
            print *, db_file
        endif

!  Loop through the selected SP bands

        do iline=1,nlines

!  Co-add the data from all lines

            test = 10000.0
            do i=1,5
                if(abs(enlinelist(iline) - enim(i)) .lt. test) then
                    test = abs(enlinelist(iline) - enim(i))
                    ien = i
                endif
            enddo
            iext = (idet - 1)*5 + ien + 1

!  Move to the correct extension

            call ftmahd (lun,iext,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 06'
                print *, 'Status FTGHPR =', status
                print *, db_file
                stop
            endif

!  Read the extension header information

            call ftghpr (lun,10,simple,bitpix,naxis,naxes, &
                pcount,gcount,extend,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 07'
                print *, 'Status FTGHPR =', status
                print *, db_file
                stop
            endif

!  Read and print out the extension name
 
            call ftgkys (lun,'EXTNAME',extname,com,status)
            if(status .ne. 0) then
                print *, 'SWCX POSITION 08'
                print *, 'Missing EXTNAME keyword'
                print *, 'Status FTGKYS =', status
                print *, db_file
                stop
            endif
            print *, extname

!  Return the exposure image

            call ftg2de (lun,0,0,naxes(1),naxes(1),naxes(2), &
                inarr,anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 09'
                print *, 'Status FTG2DE =', status
                print *, db_file
            endif

!  Mask chips if necessary

            if(ccd(1) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 259
                    i2 = 522
                    ii1 = 259
                    ii2 = 522
                elseif(idet .eq. 2) then
                    i1 = 259
                    i2 = 522
                    ii1 = 259
                    ii2 = 522
                else
                    i1 = 23
                    i2 = 346
                    ii1 = 368
                    ii2 = 684
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        inarr(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(ccd(2) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 392
                    i2 = 652
                    ii1 = 1
                    ii2 = 251
                elseif(idet .eq. 2) then
                    i1 = 392
                    i2 = 652
                    ii1 = 1
                    ii2 = 251
                else
                    i1 = 346
                    i2 = 669
                    ii1 = 368
                    ii2 = 684
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        inarr(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(ccd(3) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 524
                    i2 = 780
                    ii1 = 253
                    ii2 = 516
                elseif(idet .eq. 2) then
                    i1 = 524
                    i2 = 780
                    ii1 = 253
                    ii2 = 516
                else
                    i1 = 346
                    i2 = 670
                    ii1 = 53
                    ii2 = 368
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        inarr(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(ccd(4) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 394
                    i2 = 654
                    ii1 = 521
                    ii2 = 780
                elseif(idet .eq. 2) then
                    i1 = 394
                    i2 = 654
                    ii1 = 521
                    ii2 = 780
                else
                    i1 = 23
                    i2 = 347
                    ii1 = 54
                    ii2 = 368
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        inarr(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(idet .lt. 3) then
                if(ccd(5) .eq. 0) then
                    if(idet .eq. 1) then
                        i1 = 130
                        i2 = 393
                        ii1 = 521
                        ii2 = 780
                    else
                        i1 = 128
                        i2 = 390
                        ii1 = 523
                        ii2 = 780
                    endif
                    do i=i1,i2
                        do ii=ii1,ii2
                            inarr(i,ii) = 0.0
                        enddo
                    enddo
                endif
                if(ccd(6) .eq. 0) then
                    if(idet .eq. 1) then
                        i1 = 1
                        i2 = 258
                        ii1 = 257
                        ii2 = 520
                    else
                        i1 = 1
                        i2 = 257
                        ii1 = 258
                        ii2 = 521
                    endif
                    do i=i1,i2
                        do ii=ii1,ii2
                            inarr(i,ii) = 0.0
                        enddo
                    enddo
                endif
                if(ccd(7) .eq. 0) then
                    if(idet .eq. 1) then
                        i1 = 129
                        i2 = 391
                        ii1 = 1
                        ii2 = 253
                    else
                        i1 = 125
                        i2 = 388
                        ii1 = 1
                        ii2 = 254
                    endif
                    do i=i1,i2
                        do ii=ii1,ii2
                            inarr(i,ii) = 0.0
                        enddo
                    enddo
                endif
            endif

!  Find the total exposure of the exposure map

            rn = 0.0
            do k=1,780
                do kk=1,780
                    rn = rn + inarr(k,kk)
                enddo
            enddo

!  Find the number of counts

            do i=1,nchana
                 if((enlinelist(iline) .le. emax(i)) .and. &
                         (enlinelist(iline) .gt. emin(i))) area = arf(i)
            enddo
            rnorm = gnormlist(iline)*area*exposure*backscal
            print 2110, enlinelist(iline), rnorm
 2110       format(1x,'Total SWCX counts from the line at ', &
                f6.3,' keV:',f8.1) 

!  Now find the number of counts in the band

            do i=1,nchanrin
                 if((enlinelist(iline) .le. emaxin(i)) .and. &
                         (enlinelist(iline) .gt. eminin(i))) irmf = i
            enddo
            frac = 0.0
            do i=1,nchanrout
                if((1000.0*(eminout(i)+emaxout(i))/2.0 .le. ehigh)  &
                        .and. (1000.0*(eminout(i)+emaxout(i))/2.0  &
                        .gt. elow)) then
                    frac = frac + detm(irmf,i)
                    write(56,*) i, irmf,  &
                        eminout(i), emaxout(i), detm(irmf,i)
                endif
            enddo
            print 2120, enlinelist(iline), frac*rnorm
 2120       format(1x,'Number of SWCX counts in the chosen band from ',  &
                'the line at ',f6.3,' keV:',f8.1)
            rnorm = frac*rnorm/rn

!  Add data to the array

            do k=1,780
                do kk=1,780
                    swcxmap(k,kk) = swcxmap(k,kk) + rnorm*inarr(k,kk)
                enddo
            enddo
        enddo
        rn = 0.0
        do k=1,780
            do kk=1,780
                rn = rn + swcxmap(k,kk)
            enddo
        enddo
        print 2140, rn
 2140   format(1x,'Total SWCX counts in band:',f8.1)

!  Close the file

        call ftclos(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 10'
            print *, 'Status FTCLOS =', status
            print *, db_file
        endif

!  Open up a template event image

!  First get an unused lun

        call ftgiou(luna,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 11'
            print *, 'STATUS FTGIOU =', status
        endif

!  Create the file name

        if(elow .ge. 1000) then
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(objfile,3000) prefix, elow, ehigh
 3000               format('mos',a5,'-obj-im-det-',i4,'-',i5,'.fits')
                else
                    write(objfile,3002) prefi, elow, ehigh
 3002               format('pn',a4,'-obj-im-det-',i4,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(objfile,3004) prefix, elow, ehigh
 3004               format('mos',a5,'-obj-im-det-',i4,'-',i4,'.fits')
                else
                    write(objfile,3006) prefi, elow, ehigh
 3006               format('pn',a4,'-obj-im-det-',i4,'-',i4,'.fits')
                endif
            endif
        elseif(ehigh .lt. 1000) then
            if(idet .lt. 3) then
                write(objfile,3010) prefix, elow, ehigh
 3010           format('mos',a5,'-obj-im-det-',i3,'-',i3,'.fits')
            else
                write(objfile,3012) prefi, elow, ehigh
 3012           format('pn',a4,'-obj-im-det-',i3,'-',i3,'.fits')
            endif
        else
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(objfile,3020) prefix, elow, ehigh
 3020               format('mos',a5,'-obj-im-det-',i3,'-',i5,'.fits')
                else
                    write(objfile,3022) prefi, elow, ehigh
 3022               format('pn',a4,'-obj-im-det-',i3,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(objfile,3024) prefix, elow, ehigh
 3024               format('mos',a5,'-obj-im-det-',i3,'-',i4,'.fits')
                else
                    write(objfile,3026) prefi, elow, ehigh
 3026               format('pn',a4,'-obj-im-det-',i3,'-',i4,'.fits')
                endif
            endif
        endif
        print *, 'Template image'
        print *, objfile

!  Open the FITS file

        call ftopen (luna,objfile,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 12'
            print *, 'STATUS FTOPEN =', status
            print *, objfile
        endif

!  Read the FITS file header information

        call ftghpr (luna,10,simplen,bitpixn,naxisn,naxesn, &
            pcountn,gcountn,extendn,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 13'
            print *, 'Status FTGHPR =', status
            print *, objfile
        endif

!  Get the exposure keyword

        call ftgkyd (luna,'EXPOSURE',exposen,com,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 14'
            print *, 'Status FTGKY =', status
            print *, objfile
        endif

!  Write out the SWCX count detector map

!  First get an unused lun

        call ftgiou(lunb,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION15'
            print *, 'STATUS FTGIOU =', status
        endif

!  Create the file name

        if(elow .ge. 1000) then
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(outfile,3100) prefix, elow, ehigh
 3100               format('mos',a5,'-swcx-im-det-',i4,'-',i5,'.fits')
                else
                    write(outfile,3102) prefi, elow, ehigh
 3102               format('pn',a4,'-swcx-im-det-',i4,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(outfile,3104) prefix, elow, ehigh
 3104               format('mos',a5,'-swcx-im-det-',i4,'-',i4,'.fits')
                else
                    write(outfile,3106) prefi, elow, ehigh
 3106               format('pn',a4,'-swcx-im-det-',i4,'-',i4,'.fits')
                endif
            endif
        elseif(ehigh .lt. 1000) then
            if(idet .lt. 3) then
                write(outfile,3110) prefix, elow, ehigh
 3110           format('mos',a5,'-swcx-im-det-',i3,'-',i3,'.fits')
            else
                write(outfile,3112) prefi, elow, ehigh
 3112           format('pn',a4,'-swcx-im-det-',i3,'-',i3,'.fits')
            endif
        else
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(outfile,3120) prefix, elow, ehigh
 3120               format('mos',a5,'-swcx-im-det-',i3,'-',i5,'.fits')
                else
                    write(outfile,3122) prefi, elow, ehigh
 3122               format('pn',a4,'-swcx-im-det-',i3,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(outfile,3124) prefix, elow, ehigh
 3124               format('mos',a5,'-swcx-im-det-',i3,'-',i4,'.fits')
                else
                    write(outfile,3126) prefi, elow, ehigh
 3126               format('pn',a4,'-swcx-im-det-',i3,'-',i4,'.fits')
                endif
            endif
        endif
        print *, 'Output file name'
        print *, outfile

!  Check to see if there is an existing file

        call ftopen (lunb,outfile,0,block,status)
        if(status .eq.  0) then
            print *, 'File exists with output file name'
            if(clobber) then
                print *, 'Clobber the file and continue'
                call ftdelt(lunb,status)
            else
                print *, 'SWCX POSITION 16'
                print *, 'Will not clobber file'
                stop
            endif
        endif
        status = 0

!  Create the output FITS file using the original file as a template

        call ftinit (lunb,outfile,2880,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 17'
            print *, 'STATUS FTINIT =', status
            print *, outfile
        endif

!  Copy the header information from the old to the new file

        call ftcphd (luna,lunb,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 18'
            print *, 'STATUS FTCPHD =', status
        endif

        call ftpdef (lunb,bitpixn,naxisn,naxesn,pcountn, &
                    gcountn,.FALSE.,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 19'
            print *, 'STATUS FTPDEF =', status
        endif
        call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX Position 20'
            print *, 'STATUS FTMKYJ =', status
        endif
        if(err .eq. 0) then

!  Write the output image

            gr = 1
            call ftp2de (lunb,gr,naxes(1),naxes(1),naxes(2), &
                        swcxmap,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 21'
                print *, 'STATUS FTP2DE =', status
                print *, 'Could not write the background image'
            endif

!  Close the output file

            call ftclos(lunb,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 22'
                print *, 'STATUS FTCLOS =', status
                print *, 'Could not close the output file'
            endif
        endif

        return

  end subroutine swcx

end module swcx_mod
